function [d, pred] = dijkstra_local(A,u)
% DIJKSTRA_LOCAL native MATLAB implementation of Dijkstra's shortest path algorithm
% Supports graphs with positive edge weights
dsp = getappdata(gcf,'displayData');
nVert = size(A,1);

d = inf(nVert,1);
pred = nan(nVert,1);
inS = false(nVert,1);
arcs = cell(nVert,1);

d(u) = 0;
pred(u) = 0;

S = u;
notS = 1:nVert; notS(u) = [];
actS = u;

arcs{u} = find(A(u,:));

while ~isempty(notS)
    head = nan;
    currBest = inf;
    for iTail = actS
        for iHead = arcs{iTail}
            
            % If the new head is already in S skip it and delete the arc
            if inS(iHead)
                arcs{iTail} = arcs{iTail}(arcs{iTail} ~= iHead);
                % If all arcs have been deleted delete the tail from the 
                % list of active nodes                
                if isempty(arcs{iTail})
                    actS(actS == iTail) = [];
                end 
            else
                % Update distance labels
                dist = d(iTail) + A(iTail,iHead);
                if dist < d(iHead)
                    d(iHead) = dist;
                    pred(iHead) = iTail;
                end
                if dist <= currBest
                    currBest = dist;
                    head = iHead;
                end
            end
        end
    end    
%     set(dsp.mob_hVertex(S),'markersize',6,'markerfacecolor','w')
%     disp(head)
    
    % Add the last node with minimum distance to S and update the list of 
    % active nodes and arcs
    S = [S head];
    notS(notS == head) = [];
    inS(head) = true;
    newArcs = find(A(head,:));
    newArcs = newArcs(~ismember(newArcs,S));
    if ~isempty(newArcs)
        arcs{head} = newArcs;
        actS = [actS head];
    end
end

