function varargout = ssesPrototype(varargin)
% SSESPROTOTYPE GUI application for interactive development and assessnemt of ESS designs
%   Refer to SSES users manual for detailed descriptions and help
%
% Author: Joseph Shaw, jshaw@spa.com, jshaw3@gmu.edu
%
% SSES project team:
%   Helen Anderson
%   Steven Beres
%   Joseph Shaw
%   Timpthy Valadez
%
% Sponsor: Dr. K. C. Chang
%
% Faculty Advisor: Dr. Thomas Speller
%
% Copyright 2009, SSES Applications. All rights reserved

% Last Modified by GUIDE v2.5 06-May-2009 10:39:16

%% Begin initialization code - DO NOT EDIT
warning('off','all')
gui_Singleton = 0;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @ssesPrototype_OpeningFcn, ...
                   'gui_OutputFcn',  @ssesPrototype_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
warning('on','all')
% End initialization code - DO NOT EDIT

%% Outputs from this function are returned to the command line.
function varargout = ssesPrototype_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

%% Initialize SSES application
function ssesPrototype_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output = hObject;
guidata(hObject, handles);

% Generate SSES core data structures
% Site data
site.label = '';                % Site label
site.xGrid = [0 100];           % X axis tick cooridnates
site.xUnits = 'meters';         % X units
site.xMesh = [0 100;0 100];     % X coordinate mesh grid data 
site.xLimit = [0 100];          % X axis limits
site.xLabel = [];               % X axis display label
site.yGrid = [0 100];           % X axis tick cooridnates
site.yUnits = 'meters';         % Y units
site.yMesh = [0 0;100 100];     % Y coordinate mesh grid data
site.yLimit = [0 100];          % Y axis limits
site.yLabel = [];               % Y axis display label
site.zUnits = 'meters';         % Z units
site.zElev = [0 0;0 0];         % Elevation mesh data
site.zLimit = [0 10];           % X axis limits
site.zLabel = [];               % Z axis display label
site.xGridLines = [];           % X axis grid lines
site.yGridLines = [];           % Y axis grid lines
site.zGridLines = [];           % Z axis grid lines
site.textureMaps = [];          % Image structure of indexed images with associated colormap 
site.startTM = [];              % Label of image to display at startup
site.terrainData = [];          % Not used -- keep for compatibility with older data
site.mobilityData = [];         % Not used -- keep for compatibility with older data
site.structureData = [];        % Not used -- keep for compatibility with older data
site.featureData = [];          % Not used -- keep for compatibility with older data

terrainList = struct('type','Clear','RGB',[1 1 1],'width',1,'style','-','characteristics',[]);

% Terrain network data
ter.hMasterFigure = handles.MasterFigure; % Handle of master figure
ter.vertexXYZ = [];             % Terrain mesh vertex position
ter.vertex = struct();          % Terrain mesh vertex amplifying data
ter.nVertex = 0;                % Current number of terrain mesh vertices
ter.deletedVertices = [];       % List of deleted vertices for reuse
ter.adj = sparse([]);           % Terrain mesh adjacency matrix
ter.edge2vert = zeros(0,2);     % List of vertices connected by each edge 
ter.edge2face = zeros(0,2);     % List of tiles adjacent to each edge
ter.edge = struct();            % Terrain mesh edge amplifying data
ter.nEdge = 0;                  % Current number of terrain mesh edges
ter.faceTerIndex = [];          % Terrain mesh terrain type index
ter.face = terrainList(1);      % Terrain mesh face amplifying data
ter.facet2vert = {};            % List of vertices that form each facet 
ter.face2vert = [];             % nFace x 3 array of vertices that form each face
ter.faceCenterXYZ = [];         % Coordinates of face centroid
ter.nFace = 0;                  % Current number of terrain facets
ter.deletedFacets = [];         % List of deleted facets for reuse
ter.altOffset = 0.1;              % Offset between actual DEM and terrain facet alt
ter.terrainList = terrainList;  % Structure of terrain types
ter.currTerrainIndex = 1;       % Index of currently selected terrain type
ter.hTerSel = [];               % Handle to terrain select pushbutton
ter.meshEditMode = 0;           % Terrain mesh editing mode 
ter.currVertex = [];            % Currently active terrain mesh vertex
ter.currEdge = [];              % Currently active terrain mesh edge
ter.currTile = [];              % Currently active terrain mesh facet
ter.newVertexXYZ = [];          % Position of prospective new vertices
ter.hNewVertex = [];            % Graphic handles to new vertices
ter.selectedEdge = [];          % Currently selected edge
ter.hSelectedEdge = [];         % Graphic handle to selected edge
ter.dragVertexSel = [];         % Current vertex for dragging
ter.dragVertexRestoreXYZ = [];  % Saved location of vertex prior to dragging

% Mobility network data
mob.nEdge = [];                 % Number of mobility map edges
mob.edge2face = [];             % list of 
mob.eDist = sparse([]);         % Mobility map Euclidean distance
mob.edgeCost = [];              % Mobility map edge cost
mob.adjCost = sparse([]);       % Threat specific mobility edge cost (adjacency matrix)
% mob.p2p_source = [];            % Point-to-point shortest path route source
% mob.p2p_sink = [];              % Point-to-point shortest path route sink
% mob.p2p_time =[];               % Point-to-point shortest path route movement time
% mob.p2p_pred = [];              % Point-to-point shortest path route path predecessor
% mob.best_source = [];           % Best shortest path route source over all paths
% mob.best_sink = [];             % Best shortest path route sink over all paths
% mob.best_time =[];              % Best shortest path route movement time over all paths
% mob.best_pred = [];             % Best shortest path route path predecessor over all paths

% Obstacle data
obs.hMasterFigure = handles.MasterFigure; % Handle of master figure
obs.obstacleList = [];          % List of available obstacle types
obs.edgeObstacles = {};         % Cell array of emplaced edge obstacles
obs.faceObstacles = {};         % Cell array of emplaced face obstacles -- not currently used

% Sensor data
sensorList.model = '';          % Sensor model
sensorList.class = [];          % Sensor class {'line' or 'volume'}
sensorList.type = [];           % Sensor type
sensorList.sensorLabel = [];    % Discrete sensor label
sensorList.emission = [];       % Sensor emission mechanism {'active' or 'passive'}
sensorList.PdMode = [];         % Sensor Pd calculation mode -- currently only constant Pd is implemented 
sensorList.Pd_const = [];       % Constant Pd value
sensorList.Pd_lookup = [];      % Pd lookup table value -- not currently implemented -- may need to convert to cell
sensorList.Pd_model = [];       % Pd calculation function handle
sensorList.effRange = [];       % Sensor maximum effective range
sensorList.Xlocation = [];      % Sensor X location -- single value for volume sensors 2 element vector for line sensors
sensorList.Ylocation = [];      % Sensor Y location -- single value for volume sensors 2 element vector for line sensors
sensorList.Zlocation = [];      % Sensor Z location -- single value for volume sensors 2 element vector for line sensors
sensorList.mountHeight = 0;     % Sensor mounting height -- zero for line sensors
sensorList.AZlimit = [];        % Volume sensor azimuth left and right limits - not used for line sensors
sensorList.ELlimit = [];        % Volume sensor elevation lower and upper limits - not used for line sensors
sensorList.AZmax = [];          % Volume sensor maximum allowed azimuth extent
sensorList.ELmax = [];          % Volume sensor maximum allowed elevation extent
sensorList.unitCost = [];       % Sensor unit cost
sensorList.elementCost = [];    % Sensor element cost -- same as unit cost for vol sensors, function of run length for line sensors
sensorList.supportCost = [];    % Sensor support cost -- manually input
sensorList.totalCost = [];      % Sensor total cost
sensorList.edgeRGB = [0 0 0];   % Specifies sensor display color
sensorList.edgeWidth = 2;       % Specifies sensor display line width
sensorList.edgeStyle = '-';     % Specifies sensor display line style
sensorList.faceRGB = [1 1 1];   % Volume sensor face color -- not used for line sensors
sensorList.faceAlpha = 0.7;     % Volume sensor face density -- not used for line sensors

ess.hMasterFigure = handles.MasterFigure; % Handle of master figure
ess.sensorList = sensorList;        % List of available sensor types
ess.currentSensor = 0;              % Pointer to current sensor
ess.currentSensorParam = sensorList;% Current sensor template
ess.lineSensors = {};               % Cell array of emplaced line (edge) sensors
ess.volumeSensors = {};             % Cell array of emplaced volume (face) sensors
ess.EdgePd = [];                    % Probability of threat detection when crossing edge
ess.FacePd = [];                    % Probability of threat detection when transiting face
ess.editMode = 0;                   % Index of sensor editing mode 0 = none, 1 = add, 2 = delete, 3 = edit
ess.cumCost = 0;                    % Cumulative cost of sensors and support gear
ess.sensorCounter = 0;              % Counter of sensors assigned
ess.currPdValid = 0;                % Flag indicating whether current Pd calculation is valid
ess.savedPlanData = {};             % Data for saved ESS plans     
ess.savedPlanList = {};             % List of labels for saved ESS plans

% Display data
% Green to red progression colormap
g2rRGB =[0.38          0.7         0.16
        0.498            1            0
       0.5961       0.9843       0.5961
       0.8784            1            1
       0.6902       0.8784        0.902
        0.902        0.902       0.9804
       0.9608       0.8706        0.702
            1            1            0
            1         0.89         0.01
            1        0.549            0
            1       0.2706            0
         0.83          0.1         0.12];
    
dsp.hMasterFigure = handles.MasterFigure;
dsp.hVertex = [];               % Terrain mesh vertex graphic handles
dsp.hEdge = [];                 % Terrain mesh edge graphic handles
dsp.hFace = [];                 % Terrain mesh facet graphic handles
dsp.curTerrainHue = ter.terrainList(1).RGB; % Plot color of current terrain type
dsp.mob_hVertex = [];           % List of handles to mobility map vertices / face centers
dsp.mob_hEdge = [];             % List of handles to mobility map edges
dsp.curTerrainAlpha = 0.5;      % Terrain map transparency
dsp.curMobNetAlpha = 1;         % Mobility network transparency value
dsp.curTextureMap = [];         % Handle to current texture map
dsp.curImageMapAlpha = 1;       % Image map transparency
dsp.g2rRGB = g2rRGB;            % Utility display colormap
dsp.hEdgeObstacles = [];        % List of handles to edge obstacles
dsp.hFaceObstacles = [];        % List of handles to face obstacles -- not currently implemented
dsp.nLineSensors = 0;           % Number of line sensors emplaced -- not currently used
dsp.nVolumeSensors = 0;         % Number of line sensors emplaced -- not currently used
dsp.hLineSensors = [];          % List of handles to line sensors
dsp.hVolSensorPos = [];         % List of handles to volume sensor location markers
dsp.hVolSensorCoverage = [];    % List of handles to volume sensor coverage patches
dsp.defendedPoint = [];         % Face index of defende point (intruder goal)
dsp.hDefendedPoint = [];        % Handle to defended point marker
dsp.intruderSource = [];        % Face index of intruder starting point 
dsp.hIntruderSource = [];       % Handle to intruder starting point marker
dsp.spMoveTime = [];            % Intruder quickest path node data (min movement time)
dsp.spPred = [];                % Intruder quickest path predecessor tree data
dsp.minPd = [];                 % Intruder minimum Pd path node data (probability of detection)
dsp.minPdPred = [];             % Intruder minimum Pd path predecessor tree data
dsp.minPdRouteTime = [];        % Minimum Pd route final transit time to goal 
dsp.minRxTpd = [];              % Intruder minimum reactiont time path node data (probability of detection)
dsp.minRxTwrt = [];             % Intruder minimum reaction time path node data (weighted reaction time)
dsp.minRxTpred = [];            % Intruder minimum reaction time path predecessor tree data
dsp.minRxT = [];                % Minimum reaction time path final expected reaction time
dsp.hDisplyOptionsPB = [];      % Handle of (hijacked) display options pushbutton control
dsp.hEditTerrainPB = [];        % Handle of (hijacked) terrain editing pushbutton control
dsp.hEditSensorsPB = [];        % Handle of (hijacked) sensor editing pushbutton control

% Update Terrain selection buttons in terrain editor panel
ter = updateTerrainSelectDisplay(ter,handles);

util = [];  % *** DEBUG -- placeholder, util no longer used?

% "Hijack" fileopen and filesave uipushtools and direct them to project functions
pth = findall(handles.MasterFigure,'type','uipushtool');

nfh = findobj(pth,'tooltipstring','New Figure');
set(nfh,'ClickedCallback','ssesPrototype(''initializeSSESproject'',gcbo,[],guidata(gcbo))',...
    'Tag','SSES.ProjectNew','ToolTipString','Initialize SSES project interactively')

ofh = findobj(pth,'tooltipstring','Open File');
set(ofh,'ClickedCallback','ssesPrototype(''openSSESproject'',gcbo,[],guidata(gcbo))',...
    'Tag','SSES.ProjectOpen','ToolTipString','Open SSES Project')

sfh = findobj(pth,'tooltipstring','Save Figure');
set(sfh,'ClickedCallback','ssesPrototype(''saveSSESproject'',gcbo,[],guidata(gcbo))',...
    'Tag','SSES.ProjectSave','ToolTipString','Save SSES Project')

tth = findall(handles.MasterFigure,'type','uitoggletool');
efh = findobj(tth,'tooltipstring','Edit Plot');
load iconCDATA; %jRGB; 
set(efh,'ClickedCallback','ssesPrototype(''importSSESprojectData'',gcbo,[],guidata(gcbo))',...
    'Tag','SSES.ImportData','ToolTipString','Import SSES Project Data','separator','off','cdata',cdataIM)

dfh = findobj(tth,'tooltipstring','Data Cursor');
ifh = findobj(tth,'tooltipstring','Insert Colorbar');
set(dfh,'ClickedCallback','ssesPrototype(''setSSESdisplayOptions'',gcbo,[],guidata(gcbo))',...
    'Tag','SSES.DisplayOptions','ToolTipString','SSES display options','separator','on','cdata',get(ifh,'cdata'))
dsp.hDisplyOptionsPB = dfh;

set(ifh,'ClickedCallback','ssesPrototype(''enableSSESterrainEdit'',gcbo,[],guidata(gcbo))',...
    'Tag','SSES.DisplayOptions','ToolTipString','Edit Terrain','separator','off','cdata',cdataTE)
dsp.hEditTerrainPB = ifh;

lfh = findobj(tth,'tooltipstring','Insert Legend');
set(lfh,'ClickedCallback','ssesPrototype(''enableSSESeditSensors'',gcbo,[],guidata(gcbo))',...
    'Tag','SSES.EditSensors','ToolTipString','Edit Sensors','separator','off')
dsp.hEditSensorsPB = lfh;

% Save data structures in SSES figure application data
setappdata(handles.MasterFigure,'siteData',site)
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'obstacleData',obs)
setappdata(handles.MasterFigure,'threatData',[])
setappdata(handles.MasterFigure,'mobilityData',mob)
setappdata(handles.MasterFigure,'displayData',dsp)
setappdata(handles.MasterFigure,'structureData',[])
setappdata(handles.MasterFigure,'featureData',[])
setappdata(handles.MasterFigure,'sensorData',ess)
setappdata(handles.MasterFigure,'utilityData',util)

%% Executes on button press in selectTerrainMeshFromNpts.
function selectTerrainMeshFromNpts_Callback(hObject, eventdata, handles) %#ok<DEFNU>
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');
if ter.meshEditMode == 1
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(handles.selectTerrainMeshFromNpts,'BackgroundColor',bgcol)
    ter.meshEditMode = 0;
    set(handles.MainAxes,'ButtonDownFcn',[])
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)

    ter.meshEditMode = 1;
    set(handles.selectTerrainMeshFromNpts,'BackgroundColor',[1 1 1])
    set(handles.MainAxes,'HitTest','on','ButtonDownFcn','ssesPrototype(''addMeshFromNpoints'',gcbo,[],guidata(gcbo))')
    show_XY_position('on',handles)
    show_L1L2_length('off',handles)
    
    % If the terrain map contains no vertices provide option to start
    % terrain mesh with a regularly spaced grid of vertices
    if ter.nVertex == 0
        button = questdlg('Do you want to start with a regular terrain grid?   ',...
            'New site initialization','Yes','No','No');
        if strcmpi(button,'Yes')
            prompt = {'X or uniform grid spacing    ','Y grid spacing (omit for uniform grid    .'};
            dlg_title = 'Select terrain grid spacing';
            num_lines = 1;
            def = {'50',''};
            gs = inputdlg(prompt,dlg_title,num_lines,def);
            if ~isempty(gs)
                gsn = str2double(gs);
                gsn = gsn(~isnan(gsn));
                if numel(gsn) > 0
                    xstep = abs(gsn(1));
                end
                if numel(gsn) > 1
                    ystep = abs(gsn(2));
                else
                    ystep = round(xstep*sind(60));
                end
                site = getappdata(handles.MasterFigure,'siteData');

                xVo = site.xLimit(1):xstep:site.xLimit(2);
                if xVo(end) ~= site.xLimit(2)
                    xVo = [xVo site.xLimit(2)];
                end
                xVe = [site.xLimit(1) (site.xLimit(1)+xstep/2):xstep:site.xLimit(2)];
                if xVe(end) ~= site.xLimit(2)
                    xVe = [xVe site.xLimit(2)];
                end
                yV = site.yLimit(1):ystep:site.yLimit(2);
                if yV(end) ~= site.yLimit(2)
                    yV = [yV site.yLimit(2)];
                end
                nVertex = 0;
                for iR = 1:numel(yV)
                    if logical(mod(iR,2))
                        thisXV = xVo;
                    else
                        thisXV = xVe;
                    end
                    for iC = 1:numel(thisXV)
                        nVertex = nVertex+1;
                        x = thisXV(iC);
                        y = yV(iR);
                        z = interp2(site.xMesh,site.yMesh,site.zElev,x,y);
                        ter.newVertexXYZ(nVertex,1:3) = [x y z];
                        ter.hNewVertex(nVertex) = plot3(x,y,z,'o','markerfacecolor',[1 0 0],...
                        'markerEdgeColor',[1 0 0],'MarkerSize',3,...
                        'ButtonDownFcn','ssesPrototype(''addMeshFromNpoints'',gcbo,[],guidata(gcbo))');
                    end
                end
            end
        end        
    end
    
    set(handles.MasterFigure,'WindowButtonMotionFcn','ssesPrototype(''updateCursorPos'',gcbo,1,guidata(gcbo))')
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp);

%% Executes on button press in selectTerrainMeshFromEdgeAndPoint.
function selectTerrainMeshFromEdgeAndPoint_Callback(hObject, eventdata, handles)
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nEdge == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 2
    set(dsp.hVertex,'HitTest','off')
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(handles.selectTerrainMeshFromEdgeAndPoint,'BackgroundColor',bgcol)
    ter.meshEditMode = 0;
    set(handles.MainAxes,'ButtonDownFcn',[])
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)

    ter.meshEditMode = 2;
    set(handles.selectTerrainMeshFromEdgeAndPoint,'BackgroundColor',[1 1 1])
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','on','ButtonDownFcn',...
        'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
    set(dsp.hFace,'HitTest','off')
    set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
    show_XY_position('on',handles)
    show_L1L2_length('on',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn','ssesPrototype(''updateCursorPos'',gcbo,2,guidata(gcbo))')
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Executes on button press in selectAddInteriorVertex.
function selectAddInteriorVertex_Callback(hObject, eventdata, handles)
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nFace == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 3
    set(dsp.hVertex,'HitTest','off')
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(handles.selectAddInteriorVertex,'BackgroundColor',bgcol)
    set(handles.MainAxes,'ButtonDownFcn',[])
    ter.meshEditMode = 0;
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)

    ter.meshEditMode = 3;
    set(handles.selectAddInteriorVertex,'BackgroundColor',[1 1 1])
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hFace,'HitTest','on','ButtonDownFcn',...
        'ssesPrototype(''addInteriorVertex'',gcbo,[],guidata(gcbo))')
    set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
    show_XY_position('on',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn','ssesPrototype(''updateCursorPos'',gcbo,1,guidata(gcbo))')
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Executes on button press in selectDragVertex.
function selectDragVertex_Callback(hObject, eventdata, handles)
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nVertex == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 4
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(handles.selectDragVertex,'BackgroundColor',bgcol)
    ter.meshEditMode = 0;
    set(handles.MainAxes,'ButtonDownFcn',[])
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)
    ter.meshEditMode = 4;
    set(handles.selectDragVertex,'BackgroundColor',[1 1 1])
    set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hFace,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hVertex,'HitTest','on','ButtonDownFcn','ssesPrototype(''dragVertex'',gcbo,[],guidata(gcbo))')
    show_XY_position('on',handles)
    show_L1L2_length('off',handles)
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Executes on button press in selectDeleteVertex.
function selectDeleteVertex_Callback(hObject, eventdata, handles)
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nVertex == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 5
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hFace,'HitTest','off','ButtonDownFcn',[])
    set(handles.selectDeleteVertex,'BackgroundColor',bgcol)
    set(handles.MainAxes,'ButtonDownFcn',[])
    ter.meshEditMode = 0; 
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)
    ter.meshEditMode = 5;
    set(handles.selectDeleteVertex,'BackgroundColor',[1 1 1])
    set(dsp.hVertex,'HitTest','on','ButtonDownFcn',...
        'ssesPrototype(''deleteVertex'',gcbo,[],guidata(gcbo))')
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hFace,'HitTest','off','ButtonDownFcn',[])
    set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Executes on button press in selectDeleteEdge.
function selectDeleteEdge_Callback(hObject, eventdata, handles)
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nEdge == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 6
    set(dsp.hVertex,'HitTest','off')
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(handles.selectDeleteEdge,'BackgroundColor',bgcol)
    set(handles.MainAxes,'ButtonDownFcn',[])
    ter.meshEditMode = 0;
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)
    ter.meshEditMode = 6;
    set(handles.selectDeleteEdge,'BackgroundColor',[1 1 1])
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','on','ButtonDownFcn',...
        'ssesPrototype(''deleteEdge'',gcbo,[],guidata(gcbo))')
    set(dsp.hFace,'HitTest','off','ButtonDownFcn',[])
    set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Executes on button press in selectPaintTerrain.
function selectPaintTerrain_Callback(hObject, eventdata, handles)
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nFace == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 7
    set(dsp.hVertex,'HitTest','off')
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(handles.selectPaintTerrain,'BackgroundColor',bgcol)
    set(handles.MainAxes,'ButtonDownFcn',[])
    ter.meshEditMode = 0;
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
else
    deselectTerrainPushbuttons(ter,handles)
    ter.meshEditMode = 7;
    set(handles.selectPaintTerrain,'BackgroundColor',[1 1 1])
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hFace,'HitTest','on','ButtonDownFcn',...
        'ssesPrototype(''paintTerrainType'',gcbo,[],guidata(gcbo))')
    set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
    show_XY_position('on',handles)
    show_L1L2_length('off',handles)
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Enable obstacle addition
function ObstacleEditingAdd_Callback(hObject, eventdata, handles) %#ok<DEFNU>
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nEdge == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 10
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(dsp.hEdgeObstacles,'HitTest','off')
    set(handles.ObstacleEditingAdd,'BackgroundColor',bgcol)
    ter.meshEditMode = 0;
    set(handles.MainAxes,'ButtonDownFcn',[])
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)
    ter.meshEditMode = 10;
    set(handles.ObstacleEditingAdd,'BackgroundColor',[1 1 1])
    set(dsp.hEdge,'HitTest','on','ButtonDownFcn','ssesPrototype(''addObstacles'',gcbo,[],guidata(gcbo))')
    show_XY_position('on',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn','ssesPrototype(''updateCursorPos'',gcbo,1,guidata(gcbo))')
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp);

%% Enable obstacle deletion
function ObstacleEditingDelete_Callback(hObject, eventdata, handles) %#ok<DEFNU>
bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nEdge == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 11
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(dsp.hEdgeObstacles,'HitTest','on')
    set(handles.selectTerrainMeshFromNpts,'BackgroundColor',bgcol)
    ter.meshEditMode = 0;
    set(handles.MainAxes,'ButtonDownFcn',[])
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)
    ter.meshEditMode = 11;
    set(handles.ObstacleEditingDelete,'BackgroundColor',[1 1 1])
    set(dsp.hEdgeObstacles,'HitTest','on','ButtonDownFcn',...
        'ssesPrototype(''deleteObstacles'',gcbo,[],guidata(gcbo))')
    show_XY_position('on',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn','ssesPrototype(''updateCursorPos'',gcbo,1,guidata(gcbo))')
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp);

%% Enable obstacle editing
function ObstacleEditingEdit_Callback(hObject, eventdata, handles) %#ok<DEFNU>

bgcol = [0.92549 0.913725 0.847059];
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ter.nEdge == 0
    beepInvalidAction
    return
end
if ter.meshEditMode == 12
    set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hEdge,'HitTest','off')
    set(dsp.hFace,'HitTest','off')
    set(handles.selectTerrainMeshFromNpts,'BackgroundColor',bgcol)
    ter.meshEditMode = 0;
    set(handles.MainAxes,'ButtonDownFcn',[])
    show_XY_position('off',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn',[])
else
    deselectTerrainPushbuttons(ter,handles)
    ter.meshEditMode = 12;
    set(handles.ObstacleEditingEdit,'BackgroundColor',[1 1 1])
    set(handles.MainAxes,'HitTest','on','ButtonDownFcn','ssesPrototype(''editObstacles'',gcbo,[],guidata(gcbo))')
    show_XY_position('on',handles)
    show_L1L2_length('off',handles)
    set(handles.MasterFigure,'WindowButtonMotionFcn','ssesPrototype(''updateCursorPos'',gcbo,1,guidata(gcbo))')
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp);

%% Selects terrain type from radio button menu
function selectTerrainType_Callback(hObject, eventdata, handles)
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

terMask = ter.hTerSel == gcbo;
set(ter.hTerSel(~terMask),'value',0)
ter.currTerrainIndex = find(terMask);
dsp.curTerrainHue = ter.terrainList(ter.currTerrainIndex).RGB;

setappdata(handles.MasterFigure,'terrainData',ter);
setappdata(handles.MasterFigure,'displayData',dsp)

%% Create new terrain network from n vertices
function addMeshFromNpoints(hObject, eventdata, handles)
% Makes new terrain mesh from set of 3 or more points
ter = getappdata(handles.MasterFigure,'terrainData');
mob = getappdata(handles.MasterFigure,'mobilityData');
dsp = getappdata(handles.MasterFigure,'displayData');

site = getappdata(handles.MasterFigure,'siteData');

if hObject == handles.MainAxes
    % Attempt to add additional point to new mesh
    cp = get(gca,'CurrentPoint');
    
    % Ensure new point is not within any current facet
    for iTile = 1:numel(dsp.hFace)
        vertX = get(dsp.hFace(iTile),'xdata');
        vertX = [vertX; vertX(1)];
        vertY = get(dsp.hFace(iTile),'ydata');
        vertY = [vertY; vertY(1)];
        [inP, onP] = inpolygon(cp(1,1),cp(1,2),vertX,vertY);
        if inP || onP
            beep
            return
        end
    end 
    
    % New point is not within any facet so plot the vertex and add to the pending list
    cp(:,3) = interp2(site.xMesh,site.yMesh,site.zElev,cp(2,1),cp(2,2));
    nextNewVert = size(ter.newVertexXYZ,1)+1;
    ter.newVertexXYZ(nextNewVert,1:3) = cp(2,:);
    ter.hNewVertex(nextNewVert) = plot3(cp(2,1),cp(2,2),cp(2,3)+ter.altOffset,'o','markerfacecolor',[1 0 0],...
        'markerEdgeColor',[1 0 0],'MarkerSize',3,...
        'ButtonDownFcn','ssesPrototype(''addMeshFromNpoints'',gcbo,[],guidata(gcbo))');
    setappdata(handles.MasterFigure,'terrainData',ter)
    
else
    % Calling object is a previously defined point so attempt to complete the new mesh
    if numel(ter.hNewVertex) < 3
        % At least 3 points are needed to form a mesh -- do not allow completion
        beep
        return
    else
        % We have at least 3 points so generate the mesh
        
        % Use delaunay trangulation to generate the mesh
        tri = delaunay(ter.newVertexXYZ(:,1),ter.newVertexXYZ(:,2));
        tri4 = [tri tri(:,1)];

        curNumVert = ter.nVertex; % size(ter.vertexXYZ,1);
        curNumEdge = ter.nEdge;
        newVertexList = [];
        
        for iTile = 1:size(tri,1)
            % For each triangle create individual edges and facet
            [ijk, sIndx] = sort(tri(iTile,:));
            
            edgeX = ter.newVertexXYZ(tri4(iTile,sIndx),1);
            edgeY = ter.newVertexXYZ(tri4(iTile,sIndx),2);
            edgeZ = ter.newVertexXYZ(tri4(iTile,sIndx),3);            
            
            % Check whether the triangle contains any existing vertices or intersects any existing edges
            if curNumVert > 0
                [inP, onP] = inpolygon(ter.vertexXYZ(1:curNumVert,1),ter.vertexXYZ(1:curNumVert,2),edgeX,edgeY);
                if ~any(inP) && ~any(onP)
                   edgesIntersect = checkEdgeIntersect(ter,1:curNumEdge,edgeX,edgeY);
                else
                    edgesIntersect = 0;
                end
            else
                inP = 0;
                onP = 0;
                edgesIntersect = 0;
            end
            if ~any(inP) && ~any(onP) && ~edgesIntersect;
                % OK -- facet does not contain or intersect any existing vertices or edges 
                
                % Add new vertices to terrain structure 
                addVertRel = ijk(~ismember(ijk,newVertexList));
                newVertexList = [newVertexList addVertRel];
                ter.nVertex = ter.nVertex + numel(addVertRel);
                ter.vertexXYZ(addVertRel+curNumVert,1:3) = ter.newVertexXYZ(addVertRel,:);
                dsp.hVertex(addVertRel+curNumVert) = ter.hNewVertex(addVertRel);
                
                thisFaceVertAbs = ijk + curNumVert;                
                [ter,mob,dsp] = addFace(ter,mob,dsp,thisFaceVertAbs);

                % Add first edge
                v = sort(ijk(1:2)+curNumVert);
                if any(size(ter.adj) < v) || ter.adj(v(1),v(2)) == 0
                    % If the vertices are not already connected add a new edge
                    ter.nEdge = ter.nEdge+1;                        % Increment edge count
                    ter.adj(v(1),v(2)) = ter.nEdge;                 % Add to adjacency matrix
                    ter.adj(v(2),v(1)) = ter.nEdge;                 % Add to adjacency matrix
                    ter.edge2vert(ter.nEdge,1:2) = v;               % Link edge to vertices
                    ter.edge2face(ter.nEdge,1) = ter.nFace;       % Link edge to first terrain facet
                    dsp.hEdge(ter.nEdge) = plot3(edgeX([1 2]),edgeY([1 2]),edgeZ([1 2])+ter.altOffset,'color',[1 0 0]);
                else
                    % If the vertices are already connected just link the new region to the edge
                    ter.edge2face(ter.adj(v(1),v(2)),2) = ter.nFace;       % Link edge to second terrain facet
                end
                
                % Add second edge
                v = sort(ijk(2:3)+curNumVert);
                if any(size(ter.adj) < v) || ter.adj(v(1),v(2)) == 0
                    % If the vertices are not already connected add a new edge
                    ter.nEdge = ter.nEdge+1;                        % Increment edge count
                    ter.adj(v(1),v(2)) = ter.nEdge;                 % Add to adjacency matrix
                    ter.adj(v(2),v(1)) = ter.nEdge;                 % Add to adjacency matrix
                    ter.edge2vert(ter.nEdge,1:2) = v;               % Link edge to vertices                    
                    ter.edge2face(ter.nEdge,1) = ter.nFace;       % Link edge to first terrain facet
                    dsp.hEdge(ter.nEdge) = plot3(edgeX([2 3]),edgeY([2 3]),edgeZ([2 3])+ter.altOffset,'color',[1 0 0]);
                else
                    % If the vertices are already connected just link the new region to the edge
                    ter.edge2face(ter.adj(v(1),v(2)),2) = ter.nFace;       % Link edge to second terrain facet
                end
                
                % Add third edge
                v = sort(ijk([3 1])+curNumVert);
                if any(size(ter.adj) < v) || ter.adj(v(1),v(2)) == 0
                    % If the vertices are not already connected add a new edge
                    ter.nEdge = ter.nEdge+1;                        % Increment edge count
                    ter.adj(v(1),v(2)) = ter.nEdge;                 % Add to adjacency matrix
                    ter.adj(v(2),v(1)) = ter.nEdge;                 % Add to adjacency matrix
                    ter.edge2vert(ter.nEdge,1:2) = v;               % Link edge to vertices                    
                    ter.edge2face(ter.nEdge,1) = ter.nFace;       % Link edge to first terrain facet
                    dsp.hEdge(ter.nEdge) = plot3(edgeX([3 1]),edgeY([3 1]),edgeZ([3 1])+ter.altOffset,'color',[1 0 0]);
                else
                    % If the vertices are already connected just link the new region to the edge
                    ter.edge2face(ter.adj(v(1),v(2)),2) = ter.nFace;       % Link edge to second terrain facet
                end
            end
        end
    
        % Clean up new vertex list terrain vertex list
        set(ter.hNewVertex,'HitTest','off','ButtonDownFcn',[]);
        
        % Delete any vertices that were disallowed
        ter.hNewVertex(newVertexList) = [];
        delete(ter.hNewVertex);
        
        % Reset new vertex list
        ter.newVertexXYZ = [];
        ter.hNewVertex = [];
    end    
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'mobilityData',mob)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Extend terrain network using current edge and new or existing vertex
function addMeshFromEdgeAndPoint(hObject, eventdata, handles)
% Makes new terrain mesh face from a current edge and a new or existing vertex

ter = getappdata(handles.MasterFigure,'terrainData');
mob = getappdata(handles.MasterFigure,'mobilityData');

dsp = getappdata(handles.MasterFigure,'displayData');

site = getappdata(handles.MasterFigure,'siteData');

% Determine whether / which edge is selected
edgeIndex = find(hObject == dsp.hEdge);
if ~isempty(edgeIndex)
    % Object is a terrain map edge 
    if isempty(ter.selectedEdge)
        % No edge is currently selected
        
        if ~all(ter.edge2face(edgeIndex,:))
            % If and the clicked edge is not an interior edge select it 
            ter.selectedEdge = edgeIndex;
            set(dsp.hEdge(ter.selectedEdge),'linewidth',3)
            
            % Disable selection of other edges
            eList = 1:ter.nEdge;
            eList(eList == edgeIndex) = [];
            set(dsp.hEdge(eList),'HitTest','off')
            
            % Enable selection of main axis points and existing vertices
            set(handles.MainAxes,'HitTest','on','ButtonDownFcn',...
                'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
            set(dsp.hVertex,'HitTest','on','ButtonDownFcn',...
                    'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
        else
            % Do not allow seellection of interior edges
            beep
            return
        end
        
    else
        % If an edge was already selected deselect the previously selected edge
        % This should no longer be possible since hit test is now disabled for all other edges                        
        if edgeIndex == ter.selectedEdge
            % Re-clicking the current edge releases it
            set(dsp.hEdge(ter.selectedEdge),'linewidth',.5)
            ter.selectedEdge = [];
            set(dsp.hEdge,'HitTest','on','ButtonDownFcn',...
                    'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
            set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
            set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
        else
            % Clicking a new edge selects it
            % This should no longer be possible since hit test is now disabled for all other edges
            beep; disp('Unexpected execution in add mesh from edge and point'); keyboard
            ter.selected = edgeIndex;
            set(dsp.hEdge(ter.selectedEdge),'linewidth',3)
            set(handles.MainAxes,'HitTest','on','ButtonDownFcn',...
                'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
            set(dsp.hVertex,'HitTest','on','ButtonDownFcn',...
                'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
        end
    end
    
else
    % Returned object was not an edge -- it must be a current vertex or new point
    % This should only be possible if an edge was previously selected
    if ~isempty(ter.selectedEdge)
        
        % Get current vertices for selected edge
        currVert = ter.edge2vert(ter.selectedEdge,1:2);
        
        if hObject == handles.MainAxes
            % Make new facet using new vertex
            % For this case we must add 1 new vertex, 1 new face, and 2 new edges
            
            % Get new vertex point
            cp = get(gca,'CurrentPoint');
            xPos = cp(1,1);
            yPos = cp(1,2);
            
            % Ensure new point is not within any current face
            for iFace = 1:numel(dsp.hFace)
                vList = [ter.face2vert(iFace,:) ter.face2vert(iFace,1)];
                vX = ter.vertexXYZ(vList,1);
                vY = ter.vertexXYZ(vList,2);
                [inP, onP] = inpolygon(xPos,yPos,vX,vY);
                if inP || onP
                    beep
                    return
                end
            end
            
            % Ensure the new region will not enclose any current vertices 
            vList = 1:ter.nVertex;
            vList(ismember(vList,currVert)) = [];
            vX = [xPos; ter.vertexXYZ(currVert,1); xPos]; 
            vY = [yPos; ter.vertexXYZ(currVert,2); yPos]; 
            [inP, onP] = inpolygon(ter.vertexXYZ(vList,1),ter.vertexXYZ(vList,2),vX,vY);
            if any(inP) || any(onP)
                beep
                return
            end

            % Ensure the new edges will not intersect any current edges
            eList = 1:ter.nEdge;           
            eList(eList == ter.selectedEdge) = [];
            
            e1x = [ter.vertexXYZ(currVert(1),1); xPos];
            e1y = [ter.vertexXYZ(currVert(1),2); yPos];
            if checkEdgeIntersect(ter,eList,e1x,e1y)
                beep
                return
            else
                e2x = [ter.vertexXYZ(currVert(2),1); xPos];
                e2y = [ter.vertexXYZ(currVert(2),2); yPos];
                if checkEdgeIntersect(ter,eList,e2x,e2y)
                    beep
                    return
                end
            end
            
            % New point is acceptable -- add new vertex, face and edges
            
            % Find elevation for new point
            zPos = interp2(site.xMesh,site.yMesh,site.zElev,xPos,yPos);
            
            % Add vertex to list
            ter.nVertex = ter.nVertex + 1;
            ter.vertexXYZ(ter.nVertex,1:3) = [xPos yPos zPos];
            
            % Add face
            vList = sort([currVert ter.nVertex]);
            [ter,mob,dsp] = addFace(ter,mob,dsp,vList);
            
            % Link current edge to new face
            ter.edge2face(ter.selectedEdge,2) = ter.nFace;
            
            % Add first new edge
            [ter,dsp] = addEdge(ter,dsp, [currVert(1) ter.nVertex]);      % Add edge from start of curr edge to new vertex
            ter.edge2face(ter.nEdge,1) = ter.nFace;             % Link new edge to new face
            
            % Add second new edge
            [ter,dsp] = addEdge(ter,dsp, [currVert(2) ter.nVertex]);      % Add edge from end of curr edge to new vertex
            ter.edge2face(ter.nEdge,1) = ter.nFace;             % Link new edge to new face            

            % Plot new vertex
            dsp.hVertex(ter.nVertex) = plot3(cp(2,1),cp(2,2),zPos+ter.altOffset,'o','markerfacecolor',[1 0 0],...
                'markerEdgeColor',[1 0 0],'MarkerSize',3,'HitTest','off','ButtonDownFcn',[]);
            
            % Release current edge
            set(dsp.hEdge(ter.selectedEdge),'linewidth',.5)
            ter.selectedEdge = [];
            set(dsp.hEdge,'HitTest','on','ButtonDownFcn',...
                'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
            set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
            set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
            
         else
            newVert = find(hObject == dsp.hVertex);
            if ~isempty(currVert)
                % Make new facet using existing vertex
                xPos = ter.vertexXYZ(newVert,1);
                yPos = ter.vertexXYZ(newVert,2);
                
                % By definition the selected vertex is not within any existing faces
                
                % Ensure the new region will not enclose any existing vertices 
                vList = 1:ter.nVertex;
                vList(ismember(vList,[currVert newVert])) = [];
                vX = [xPos; ter.vertexXYZ(currVert,1); xPos]; 
                vY = [yPos; ter.vertexXYZ(currVert,2); yPos]; 
                [inP, onP] = inpolygon(ter.vertexXYZ(vList,1),ter.vertexXYZ(vList,2),vX,vY);
                if any(inP) || any(onP)
                    beep
                    return
                end

                % Determine whether the new vertex is adjacent to the current edge vertices
                % If adjacent, do not check this edge for edge intersection and do not re-add it  
                eStartEdgeVert = sort([currVert(1) newVert]);
                eEndEdgeVert = sort([currVert(2) newVert]);
                adjSize = size(ter.adj);
                if adjSize(1) >= eStartEdgeVert(1) 
                    eStartAdj = ter.adj(eStartEdgeVert(1),eStartEdgeVert(2));
                else
                    eStartAdj = 0;
                end
                if adjSize(1) >= eEndEdgeVert(1) 
                    eEndAdj = ter.adj(eEndEdgeVert(1),eEndEdgeVert(2));
                else
                    eEndAdj = 0;
                end
                
                if eStartAdj && eEndAdj
                    % If current segment start and end vertices are both adjacent to the new vertex
                    % It is possible that the face already exists -- if so do not recreate it
                    possFace = unique([ter.edge2face(eStartAdj,:) ter.edge2face(eEndAdj,:)]);
                    possFace(possFace==0) = [];
                    for iFace = possFace(:)'
                        if all(ismember(ter.face2vert(iFace,:),[currVert newVert]))
                            % If all new vertices match an vertices for an existing face it already exists
                            beep
                            return
                        end
                    end
                end
                
                if ~eStartAdj && ~eEndAdj
                    % If neither segment start or end vertices are adjacent we also need to verify
                    % that the new edges will not intersect any existing edge -- it is enough to
                    % check exterior edges
                    
                    eList = findExteriorEdges(ter,ter.selectedEdge);
                    eX = [ter.vertexXYZ(currVert(1),1); xPos; ter.vertexXYZ(currVert(2),1)];
                    eY = [ter.vertexXYZ(currVert(1),2); yPos; ter.vertexXYZ(currVert(2),2)];
                    if checkEdgeIntersect(ter,eList,eX,eY)
                        beep
                        return
                    end
                end     
                
                % The new face is acceptable -- add face and edge(s)
                % Add new face
                vList = sort([currVert newVert]);
                [ter,mob,dsp] = addFace(ter,mob,dsp,vList);

                % Link current edge to new face
                ter.edge2face(ter.selectedEdge,2) = ter.nFace;

                % Add/link first new edge
                if ~eStartAdj
                    [ter,dsp] = addEdge(ter,dsp, [currVert(1) newVert]);      % Add edge from start of curr edge to new vertex
                    ter.edge2face(ter.nEdge,1) = ter.nFace;         % Link new edge to new face
                else
                    ter.edge2face(eStartAdj,2) = ter.nFace;         % If edge is adjacent just link it to the face
                end

                % Add / link second new edge
                if ~eEndAdj
                    [ter,dsp] = addEdge(ter,dsp, [currVert(2) newVert]);      % Add edge from end of curr edge to new vertex
                    ter.edge2face(ter.nEdge,1) = ter.nFace;         % Link new edge to new face                
                else
                    ter.edge2face(eEndAdj,2) = ter.nFace;           % If edge is adjacent just link it to the face
                end                   

                % Release current edge
                set(dsp.hEdge(ter.selectedEdge),'linewidth',.5)
                ter.selectedEdge = [];
                set(handles.MainAxes,'HitTest','off','ButtonDownFcn',[])
                set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
                set(dsp.hEdge,'HitTest','on','ButtonDownFcn',...
                    'ssesPrototype(''addMeshFromEdgeAndPoint'',gcbo,[],guidata(gcbo))')
            
            else
                error('Callback object should have been either the main axes or an existing vertex')
            end
            
        end
    else
        beep
    end
end
setappdata(handles.MasterFigure,'terrainData',ter);
setappdata(handles.MasterFigure,'mobilityData',mob)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Add Interior Vertex
function addInteriorVertex(hObject, eventdata, handles)
% Makes new terrain mesh from set of 3 or more points
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

cp = get(gca,'CurrentPoint');
xPos = cp(1,1);
yPos = cp(1,2);

cbFace = find(dsp.hFace == hObject);
cbFaceVert = [ter.face2vert(cbFace,:) ter.face2vert(cbFace,1)];
[in, on] = inpolygon(xPos,yPos,ter.vertexXYZ(cbFaceVert,1),ter.vertexXYZ(cbFaceVert,2));
if in || on
    % Since the current point is in the face it is correct
    correctFace = cbFace;
else
    % If they current point is not on the face check other faces in order of increasing distance
    % from centroid to current point
    dist = sqrt((ter.faceCenterXYZ(:,1)-xPos).^2 + (ter.faceCenterXYZ(:,2)-yPos).^2);
    [s,dList] = sort(dist);
    dList(dList==cbFace) = [];
    correctFace = [];
    for iFace = dList(:)'
        cbFaceVert = [ter.face2vert(iFace,:) ter.face2vert(iFace,1)];
        [in, on] = inpolygon(xPos,yPos,ter.vertexXYZ(cbFaceVert,1),ter.vertexXYZ(cbFaceVert,2));
        if in || on
            correctFace = iFace;
           break 
        end
    end
end

if ~isempty(correctFace)
    % We found the correct face so add the new vertex and subdivide the existing face
    site = getappdata(handles.MasterFigure,'siteData');

    % Add new vertex to list
    zPos = interp2(site.xMesh,site.yMesh,site.zElev,xPos,yPos);
    ter.nVertex = ter.nVertex + 1;
    ter.vertexXYZ(ter.nVertex,1:3) = [xPos yPos zPos];
    dsp.hVertex(ter.nVertex) = plot3(cp(2,1),cp(2,2),zPos+ter.altOffset,'o','markerfacecolor',[1 0 0],...
                'markerEdgeColor',[1 0 0],'MarkerSize',3,'HitTest','off','ButtonDownFcn',[]);
    
    % Find vertices of current face
    cfv = ter.face2vert(correctFace,:);
    
    % Find adjacent faces
    adjFace = sort(find(sum(ismember(ter.face2vert,cfv),2)==2));
    
    if ~isempty(adjFace)
        % Find common vertices for the lowest number adjacent face
        sharedVert = intersect(cfv, ter.face2vert(adjFace(1),:));
        otherVert = setdiff(cfv,sharedVert);
    else
        sharedVert = cfv(1:2);
        otherVert = cfv(3);
    end
    
    % Update face data for the current face -- retain edge with lowest vertex indices
    face1vert = [sharedVert ter.nVertex];
    
    ter.facet2vert{correctFace} = face1vert;  % *** deprecated
    ter.face2vert(correctFace,1:3) = face1vert;                             % Link vertices
    ter.faceCenterXYZ(correctFace,1:3) = sum(ter.vertexXYZ(face1vert,:))/3; % Compute centroid

    vlc = [face1vert(:); face1vert(1)];                                     % Closed list of vertices
    fX = ter.vertexXYZ(vlc,1);                                              % Patch X data
    fY = ter.vertexXYZ(vlc,2);                                              % Patch Y data
    fZ = ter.vertexXYZ(vlc,3);                                              % Patch Z data
    set(dsp.hFace(correctFace),'xdata',fX,'ydata',fY,'zdata',fZ+ter.altOffset);
    
    % Update mobility network vertex (face center point) and edge
    set(dsp.mob_hVertex(correctFace),'xdata',ter.faceCenterXYZ(correctFace,1),...
        'ydata',ter.faceCenterXYZ(correctFace,2),'zdata',ter.faceCenterXYZ(correctFace,3)+ter.altOffset);
    if ~isempty(adjFace)
        keepMobEdge = find(all(ismember(ter.edge2face,[correctFace adjFace(1)]),2),1);
        set(dsp.mob_hEdge(keepMobEdge),'xdata',ter.faceCenterXYZ([correctFace adjFace(1)],1),...
            'ydata',ter.faceCenterXYZ([correctFace adjFace(1)],2),'zdata',...
            ter.faceCenterXYZ([correctFace adjFace(1)],3)+ter.altOffset)    
        ter = updateMobNetDist(ter,keepMobEdge);
    end
    
    % Delete other mobility network edges -- they will be recreated if necessary with the new faces
    killMobEdge = find(all(ismember(ter.edge2face,[correctFace; adjFace(2:end)]),2));
    delete(dsp.mob_hEdge(killMobEdge))
    dsp.mob_hEdge(killMobEdge) = 0;
    mob = getappdata(handles.MasterFigure,'mobilityData');
    for iKillEdge = killMobEdge(:)'
       mob.eDist(ter.edge2face(iKillEdge,1),ter.edge2face(iKillEdge,2)) = 0;
       mob.eDist(ter.edge2face(iKillEdge,2),ter.edge2face(iKillEdge,1)) = 0;
    end
    setappdata(handles.MasterFigure,'mobilityData',mob);
    
    % Connect each existing vertex with new center vertex
    [ter,dsp] = addEdge(ter,dsp, [sharedVert(1) ter.nVertex]);
    ter.edge2face(ter.nEdge,1:2) = [correctFace ter.nFace+1];
    
    [ter,dsp] = addEdge(ter,dsp, [sharedVert(2) ter.nVertex]);
    ter.edge2face(ter.nEdge,1:2) = [correctFace ter.nFace+2];
    
    [ter,dsp] = addEdge(ter,dsp, [otherVert ter.nVertex]);
    ter.edge2face(ter.nEdge,1:2) = [ter.nFace+1 ter.nFace+2];
    
    % Add first new face and update terrain type and color to match current face
    face2vert = [sharedVert(1) otherVert ter.nVertex];
    [ter,mob,dsp] = addFace(ter,mob,dsp,face2vert);
    ter.face(ter.nFace) = ter.face(correctFace);
    adjZ = get(dsp.hFace(ter.nFace),'zdata');
    set(dsp.hFace(ter.nFace),'faceColor',ter.face(correctFace).RGB,'zdata',adjZ-.01,...
        'ButtonDownFcn','ssesPrototype(''addInteriorVertex'',gcbo,[],guidata(gcbo))',...
        'HitTest','on')
    e2v = sort([sharedVert(1), otherVert]);
    edge2 = ter.adj(e2v(1),e2v(2));
    ter.edge2face(edge2,(ter.edge2face(edge2,:)==correctFace)) = ter.nFace;
    
    % Add second new face and update terrain type and color to match current face
    face3vert = [sharedVert(2) otherVert ter.nVertex];
    [ter,mob,dsp] = addFace(ter,mob,dsp,face3vert);
    ter.face(ter.nFace) = ter.face(correctFace);
    adjZ = get(dsp.hFace(ter.nFace),'zdata');
    set(dsp.hFace(ter.nFace),'faceColor',ter.face(correctFace).RGB,'zdata',adjZ-.01,...
        'ButtonDownFcn','ssesPrototype(''addInteriorVertex'',gcbo,[],guidata(gcbo))',...
        'HitTest','on')
    e3v = sort([sharedVert(2), otherVert]);
    edge3 = ter.adj(e3v(1),e3v(2));
    ter.edge2face(edge3,(ter.edge2face(edge3,:)==correctFace)) = ter.nFace;
    
else
    beep
    return
end
setappdata(handles.MasterFigure,'terrainData',ter);
setappdata(handles.MasterFigure,'displayData',dsp)

%% Drag Vertex
function dragVertex(hObject, eventdata, handles)
% Drags vertex to new position
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if hObject == handles.MasterFigure
    % if callback object is master figure then perform dragging operation
    
    cp = get(gca,'CurrentPoint');
    xPos = cp(1,1);
    yPos = cp(1,2);
    updateCursorPos(handles.MasterFigure,1,handles)
    
    % Do not update if current point is not within the plot area
    xl = get(handles.MainAxes,'XLim');
    if xPos < xl(1) || xl(2) < xPos
        return
    end
    yl = get(handles.MainAxes,'YLim');
    if yPos < yl(1) || yl(2) < yPos
        return
    end
    
    if ter.dragVertexIsInterior
        % If the vertex is interior or exterior the new location is only acceptable if
        % it is within one of the adjacent facets
        isOK = false;
        for iFace = ter.dragVertexAdjFacet'
            fvList = ter.facet2vert{iFace};
            fvList = [fvList fvList(1)];
            isOK = inpolygon(xPos,yPos,ter.vertexXYZ(fvList,1),ter.vertexXYZ(fvList,2));
            if isOK
                break
            end
        end
        if ~isOK
            return
        end
        
        checkEdges = find(any( ismember(ter.edge2face,ter.dragVertexAdjFacet),2));
        checkEdges(ismember(checkEdges,ter.dragVertexAdjEdge)) = [];
        % Check for intersection between adjacent and exterior 
        for iEdge = ter.dragVertexAdjEdge'
            thisFixedVert = ter.edge2vert(iEdge,:);
            thisFixedVert(thisFixedVert == ter.dragVertexSel) = [];

            edgeX = [ter.vertexXYZ(thisFixedVert,1) cp(1,1)];
            edgeY = [ter.vertexXYZ(thisFixedVert,2) cp(1,2)];
            if checkEdgeIntersect(ter,checkEdges,edgeX,edgeY);
                return
            end
        end

    else
        % For an exterior vertex the new location is acceptable if the point is not inside
        % any non-adjacent face and the resulting new edges do not cross any 
        % other exterior edge or interior edges of adjacent faces
        
        % Ensure new point is not within any current face
        checkFace = find(all(~ismember(ter.face2vert,ter.dragVertexSel),2));
        for iFace = checkFace(:)'
            vList = [ter.face2vert(iFace,:) ter.face2vert(iFace,1)];
            vX = ter.vertexXYZ(vList,1);
            vY = ter.vertexXYZ(vList,2);
            [inP, onP] = inpolygon(xPos,yPos,vX,vY);
            if inP || onP
                return
            end
        end

        checkEdges = find(all(~ismember(ter.edge2vert,ter.dragVertexSel),2));
        % Check for intersection between adjacent and exterior 
        for iEdge = ter.dragVertexAdjEdge'
            thisFixedVert = ter.edge2vert(iEdge,:);
            thisFixedVert(thisFixedVert == ter.dragVertexSel) = [];

            edgeX = [ter.vertexXYZ(thisFixedVert,1) cp(1,1)];
            edgeY = [ter.vertexXYZ(thisFixedVert,2) cp(1,2)];
            if checkEdgeIntersect(ter,checkEdges,edgeX,edgeY);
                return
            end
        end        
    end
    
    % New vertex location is acceptable -- accept new location and update edges and facets
    site = getappdata(handles.MasterFigure,'siteData');
    cp(:,3) = interp2(site.xMesh,site.yMesh,site.zElev,cp(2,1),cp(2,2));
    ter.vertexXYZ(ter.dragVertexSel,1:3) = cp(1,1:3);
    
    % *** Update faces
    for iFace = ter.dragVertexAdjFacet'
        faceVert = ter.facet2vert{iFace};
        faceVert  = [faceVert faceVert(1)];
        
        faceXYZ = ter.vertexXYZ(faceVert,1:3);
        set(dsp.hFace(iFace),'xdata',faceXYZ(:,1),'ydata',faceXYZ(:,2),...
            'zdata',faceXYZ(:,3)+ter.altOffset)
        
        ter.faceCenterXYZ(iFace,1:3) = sum(ter.vertexXYZ(faceVert(1:3),:))/3;    % Compute centroid
        set(dsp.mob_hVertex(iFace),'xdata',ter.faceCenterXYZ(iFace,1),...
            'ydata',ter.faceCenterXYZ(iFace,2),'zdata',...
            ter.faceCenterXYZ(iFace,3)+ter.altOffset);
        
        possMobEdges = find(any(ter.edge2face == iFace,2));
        % If number of mobility edges < the largest edge under consideration expand the handle list
        if numel(dsp.mob_hEdge) < max(possMobEdges(:))
            dsp.mob_hEdge(max(possMobEdges(:))) = 0;
        end
        possMobEdges = possMobEdges(dsp.mob_hEdge(possMobEdges) ~= 0);
        for iMobEdge = possMobEdges'
            centerXYZ = ter.faceCenterXYZ(ter.edge2face(iMobEdge,:),1:3);
            set(dsp.mob_hEdge(iMobEdge),'xdata',centerXYZ(:,1),'ydata',centerXYZ(:,2),...
                'zdata',centerXYZ(:,3)+ter.altOffset)
            ter = updateMobNetDist(ter,iMobEdge);
        end 
    end

    % *** Update edges
    for iEdge = ter.dragVertexAdjEdge'
        edgeXYZ = ter.vertexXYZ(ter.edge2vert(iEdge,:),1:3);
        set(dsp.hEdge(iEdge),'xdata',edgeXYZ(:,1),'ydata',edgeXYZ(:,2),...
            'zdata',edgeXYZ(:,3)+ter.altOffset)       
    end
    set(dsp.hVertex(ter.dragVertexSel),'xdata',cp(1,1),'ydata',cp(1,2))   

else
    % If callback object is a vertex handle select the vertex and initialize vertex dragging
    thisVertex = find(hObject == dsp.hVertex);
    if ~isempty(thisVertex)
        ter.dragVertexSel = thisVertex;
         
        % Find edges and facets that are adjacent to this vertex
        ter.dragVertexAdjEdge = find(any(ter.edge2vert == thisVertex,2));        
        adjFacetList = ter.edge2face(ter.dragVertexAdjEdge,:);
        ter.dragVertexAdjFacet = unique(adjFacetList);
        ter.dragVertexAdjFacet(ter.dragVertexAdjFacet == 0) = [];
        
        % Vertex is "interior" if all adjacent edges are adjacent to two facets        
        ter.dragVertexIsInterior = all(all(adjFacetList,2));
        
        if ter.dragVertexIsInterior
            mc = [0 1 0];
        else
            mc = [0 0 1];
        end

        ter.dragVertexRestoreXYZ = ter.vertexXYZ(thisVertex,:);
        set(dsp.hVertex(thisVertex),'markersize',4,'markerFaceColor',mc,'markerEdgeColor',mc)

        set(handles.MasterFigure,'WindowButtonMotionFcn','ssesPrototype(''dragVertex'',gcbo,[],guidata(gcbo))',...
            'WindowButtonUpFcn','ssesPrototype(''releaseDraggedVertex'',gcbo,[],guidata(gcbo))');
    else
        beep
    end
end
setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)
    
%% Release Drag Vertex
function releaseDraggedVertex(hObject, eventdata, handles)
ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Release vertex and return to normal appearance
set(handles.MasterFigure,'WindowButtonMotionFcn',[],'WindowButtonUpFcn',[]);
set(dsp.hVertex(ter.dragVertexSel),'markersize',3,'markerFaceColor',[1 0 0],'markerEdgeColor',[1 0 0]);

% Clear drag vertex temp data
ter.dragVertexSel = [];
ter.dragVertexRestoreXYZ = [];
ter.dragVertexAdjEdge = [];
ter.dragVertexAdjFacet = [];
ter.dragVertexIsInterior = [];

setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Paint Terrain Type
function paintTerrainType(hObject, eventdata, handles)
% Change terrain face type and color to currently selected terrain type / color

ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

cp = get(gca,'CurrentPoint');
xPos = cp(1,1);
yPos = cp(1,2);

correctFace = findCorrectFace(hObject,handles,xPos,yPos);
ter.faceTerIndex(correctFace) = ter.currTerrainIndex;
ter.face(correctFace) = ter.terrainList(ter.currTerrainIndex);
set(dsp.hFace(correctFace),'FaceColor',ter.face(correctFace).RGB)

setappdata(handles.MasterFigure,'terrainData',ter)
setappdata(handles.MasterFigure,'displayData',dsp)

%% Add Face
function [ter,mob,dsp] = addFace(ter,mob,dsp,vertList)
% Add new terrain mesh face

if numel(vertList) ~= 3
    error('SSES:AddFace','New edge must comprise exactly three vertices')
end

ter.nFace = ter.nFace + 1;                                              % Increment face count
ter.facet2vert{ter.nFace} = vertList;  % *** deprecated
ter.face2vert(ter.nFace,1:3) = vertList;                                % Link vertices
ter.faceCenterXYZ(ter.nFace,1:3) = sum(ter.vertexXYZ(vertList,:))/3;    % Compute centroid

ter.faceTerIndex = ter.currTerrainIndex;
ter.face(ter.nFace) = ter.terrainList(ter.currTerrainIndex);            % Copy terrain data

vlc = [vertList(:);vertList(1)];                                        % Closed list of vertices
fX = ter.vertexXYZ(vlc,1);                                              % Patch X data
fY = ter.vertexXYZ(vlc,2);                                              % Patch Y data
fZ = ter.vertexXYZ(vlc,3);                                              % Patch Z data

% Plot face and add handle to face handle vector
dsp.hFace(ter.nFace) = patch(fX,fY,fZ+ter.altOffset-.02,fZ,'edgecolor','none',...
    'FaceColor',dsp.curTerrainHue,'facealpha',dsp.curTerrainAlpha,...
    'ButtonDownFcn',[],'HitTest','off');

dsp.mob_hVertex(ter.nFace) = plot3(ter.faceCenterXYZ(ter.nFace,1),ter.faceCenterXYZ(ter.nFace,2),...
    ter.faceCenterXYZ(ter.nFace,3)+ter.altOffset+.1,'bo','MarkerSize',2,'MarkerFaceColor',[0 0 1],'MarkerEdgeColor',[0 0 1],...
    'HitTest','off');

% If there is an adjacent face add the mobility network edge if it does not already exist
% Get the list of face vertices and find any adjacent faces by comparing vertices
vList = ismember(ter.face2vert(:,1:3),ter.face2vert(ter.nFace,1:3));
adjFaceList = find(sum(vList,2)==2);
if ~ isempty(adjFaceList)
    
    % 
    for adjFace = adjFaceList'
        % Find the shared vertices
        adjVert = ter.face2vert(adjFace,vList(adjFace,:));
        
        % Find the corresponding terrain map edge
        adjEdge = find(all(ismember(ter.edge2vert,adjVert),2));
        
        % If there is currently no mobility edge connecting these faces add one now
        if numel(dsp.mob_hEdge) < adjEdge || dsp.mob_hEdge(adjEdge) == 0
            newMobFace = sort([adjFace ter.nFace]);
            dsp.mob_hEdge(adjEdge) = plot3(ter.faceCenterXYZ(newMobFace,1),...
                ter.faceCenterXYZ(newMobFace,2),ter.faceCenterXYZ(newMobFace,3)+ter.altOffset,...
                'color',[0 0 1],'linewidth',1,'HitTest','off');

            ter = updateMobNetDist(ter,adjEdge);
            ter.mob_edge2face(adjEdge,1:2) = [adjFace ter.nFace];
            
        end
        
    end
end

%% Add Edge
function [ter,dsp] = addEdge(ter,dsp,vertList)
% Add new terrain mesh edge

if numel(vertList) ~= 2
    error('SSES:AddEdge','New edge must comprise exactly two vertices')
end

ter.nEdge = ter.nEdge + 1;                                  % Increment edge count
vertList = sort(vertList);                                  % Always list vertices in ascending order
ter.adj(vertList(1),vertList(2)) = ter.nEdge;               % Add to adjacency matrix
ter.adj(vertList(2),vertList(1)) = ter.nEdge;               % Add to adjacency matrix
ter.edge2vert(ter.nEdge,1:2) = vertList;                    % Link edge to vertices
eX = ter.vertexXYZ(vertList,1);
eY = ter.vertexXYZ(vertList,2);
eZ = ter.vertexXYZ(vertList,3)-.01;

% Plot edge and add handle to edge handle vector
dsp.hEdge(ter.nEdge) = plot3(eX,eY,eZ+ter.altOffset,'color',[1 0 0],'ButtonDownFcn',[],'HitTest','off');

%% Update Mobility Network Distances
function ter = updateMobNetDist(ter,adjEdge)
site = getappdata(ter.hMasterFigure,'siteData');

% Find adjacent faces
adjFace = ter.edge2face(adjEdge,:);
% If we are adding a new exterior face edge2face data may not yet be updated
% For this case find the adjacent face by examining face vertices
if ~all(adjFace)
    adjFace = find(sum(ismember(ter.face2vert,ter.edge2vert(adjEdge,:)),2)==2);
end
afcXYZ = ter.faceCenterXYZ(adjFace,1:3);
adjEdgeVert = ter.edge2vert(adjEdge,:);
aevXYZ = ter.vertexXYZ(adjEdgeVert,:);


[segInt,pX,pY] = lineSegInt(afcXYZ(:,1),afcXYZ(:,2),aevXYZ(:,1),aevXYZ(:,2));
pZ = interp2(site.xMesh,site.yMesh,site.zElev,pX,pY);

df1e = norm(afcXYZ(1,:) - [pX pY pZ]);
df2e = norm(afcXYZ(2,:) - [pX pY pZ]);

ter.mob_eDist(adjFace(1),adjFace(2)) = df1e;  %"half distance" from face 2 to face 1
ter.mob_eDist(adjFace(2),adjFace(1)) = df2e;  %"half distance" from face 1 to face 2

%% Find exterior edges
function extEdges = findExteriorEdges(ter,excludeEdge,extVertex,extFace)
% Finds "exterior" edges, excluding any edges specified by excludeEdge
% Include as "exterior" nominally interior edges that are adjacent to faces which are explicitly
% specified by extFace or adjacent to vertices specified by extVertex 

if nargin < 2
    excludeEdge = [];
    excludeFace = [];
elseif nargin >= 3
    excludeFace = find(any(ismember(ter.face2vert,extVertex),2));
    addExEdge = find(any(ismember(ter.edge2vert,extVertex),2));
    excludeEdge = unique([excludeEdge(:); addExEdge(:)]);
    if nargin == 4
        excludeFace = unique([excludeFace; extFace(:)]);
    end
else
    excludeFace = [];
end

% Find the faces adjacent to each edge -- an edge is exterior if it is only bordered by one face
edge2FaceList = ter.edge2face;
edge2FaceList(ismember(edge2FaceList,excludeFace)) = 0;

% Exclude selected edges
extEdges = find(~all(edge2FaceList,2));
extEdges = extEdges(~ismember(extEdges,excludeEdge));


%% Find the "correct" terrain network face corresponding to X-Y position
function correctFace = findCorrectFace(hObject,handles,xPos,yPos)

ter = getappdata(handles.MasterFigure,'terrainData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ~isempty(hObject) && ishandle(hObject)
    cbFace = find(dsp.hFace == hObject);
    cbFaceVert = [ter.face2vert(cbFace,:) ter.face2vert(cbFace,1)];
    [in, on] = inpolygon(xPos,yPos,ter.vertexXYZ(cbFaceVert,1),ter.vertexXYZ(cbFaceVert,2));
else
    in = 0;
    on = 0;
end
if in || on
    % Since the current point is in the face it is correct
    correctFace = cbFace;
else
    % If they current point is not on the face check other faces in order of increasing distance
    % from centroid to current point
    dist = sqrt((ter.faceCenterXYZ(:,1)-xPos).^2 + (ter.faceCenterXYZ(:,2)-yPos).^2);
    [s,dList] = sort(dist);
    if ~isempty(hObject)
        dList(dList==cbFace) = [];
    end
    correctFace = [];
    for iFace = dList(:)'
        cbFaceVert = [ter.face2vert(iFace,:) ter.face2vert(iFace,1)];
        [in, on] = inpolygon(xPos,yPos,ter.vertexXYZ(cbFaceVert,1),ter.vertexXYZ(cbFaceVert,2));
        if in || on
            correctFace = iFace;
           break 
        end
    end
end

%% Add obstacles to site
function addObstacles(hObject, eventdata, handles)
% Makes new terrain mesh face from a current edge and a new or existing vertex

obs = getappdata(handles.MasterFigure,'obstacleData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Determine whether / which edge is selected
edgeIndex = find(hObject == dsp.hEdge);
if ~isempty(edgeIndex)
    currObsIndx = get(handles.ObstacleEditingPopup,'value');
    obs.edgeObstacles{edgeIndex} = obs.obstacleList(currObsIndx);
    currObs = obs.obstacleList(currObsIndx);
    xd = get(dsp.hEdge(edgeIndex),'xdata');
    yd = get(dsp.hEdge(edgeIndex),'ydata');
    zd = get(dsp.hEdge(edgeIndex),'zdata');
    dsp.hEdgeObstacles(edgeIndex) = plot3(xd,yd,zd+.01,'LineStyle',currObs.style,...
        'color',currObs.RGB,'linewidth',currObs.width,'HitTest','off');
else
    beep
end
setappdata(handles.MasterFigure,'obstacleData',obs);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Delete obstacles from site
function deleteObstacles(hObject, eventdata, handles)
% Makes new terrain mesh face from a current edge and a new or existing vertex

obs = getappdata(handles.MasterFigure,'obstacleData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Determine whether / which edge is selected
edgeIndex = find(hObject == dsp.hEdgeObstacles);
if ~isempty(edgeIndex)
    obs.edgeObstacles{edgeIndex} = [];
    delete(dsp.hEdgeObstacles(edgeIndex))
    dsp.hEdgeObstacles(edgeIndex) = 0;
else
    beep
end
setappdata(handles.MasterFigure,'obstacleData',obs);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Check for edge intersection -- utility function
function edgesIntersect = checkEdgeIntersect(ter,checkEdgeList,edgeX,edgeY)
% CHECKEDGEINTERSECT tests whether an edge or any of three edges intersect any existing edges

checkEdgeList = checkEdgeList(:)';
edgeX = edgeX(:);
edgeY = edgeY(:);
epTol = 1;
edgesIntersect = 0;
for iCurrEdge = checkEdgeList
    xcurr = ter.vertexXYZ(ter.edge2vert(iCurrEdge,:),1);
    ycurr = ter.vertexXYZ(ter.edge2vert(iCurrEdge,:),2);
   
    % Check first new edge
    edgesIntersect = lineSegInt(xcurr,ycurr,edgeX([1 2]),edgeY([1 2]),epTol);
    if edgesIntersect
        return
    elseif size(edgeX,1) > 2
        % Check second new edge
        edgesIntersect = lineSegInt(xcurr,ycurr,edgeX([2 3]),edgeY([2 3]),epTol);
        if edgesIntersect
            return
        else
            edgesIntersect = lineSegInt(xcurr,ycurr,edgeX([3 1]),edgeY([3 1]),epTol);
            if edgesIntersect
                return
            end
        end
    end
end

%% Open SSES project .mat file and load data and display
function openSSESproject(hObject, eventdata, handles)

% Get project file and path
[projFN, projPath] = uigetfile('*SSES.mat');
if ischar(projFN)
    load(fullfile(projPath,projFN))
else
    return
end

if ~exist('site','var')
    beep
    warning('Matlab:SSESprototype','Project file does not contain required data')
    return
end

set(handles.MasterFigure,'Pointer','watch')
set(handles.MainAxes,'xlim',site.xLimit,'ylim',site.yLimit,'ZlimMode','auto',... %'zlim',site.zLimit,
    'xtick',site.xGridLines,'ytick',site.yGridLines,'ztick',site.zGridLines,...    
    'xGrid','on','yGrid','on','zGrid','on','box','on')
xlabel(site.xLabel)
ylabel(site.yLabel)
zlabel(site.zLabel)

if exist('dsp','var')
    % If saved display data is available update current display data
    fn = fieldnames(dsp); %#ok<NODEF>
    currDsp = getappdata(handles.MasterFigure,'displayData');
    cfn = fieldnames(currDsp);
    tabu = {'hDisplyOptionsPB', 'hEditTerrainPB', 'hEditSensorsPB'};
    fn = {fn{~ismember(fn,tabu)}}';
    for iField = 1:numel(fn)
        if strmatch(fn{iField}, cfn, 'exact')
            currDsp.(fn{iField}) = dsp.(fn{iField});
        end
    end
    dsp = currDsp;
    setappdata(handles.MasterFigure,'displayData',dsp) %#ok<NODEF>
    % If current texturemap (image) data exists then delete it
    if isfield(dsp,'curTextrueMap') && ishandle(dsp.curTextureMap)
        delete(dsp.curTextureMap);
    end    
end

if exist('site','var')
    setappdata(handles.MasterFigure,'siteData',site)
    % If new texture (image) data is specified the draw it now 
    if isfield(site.textureMaps,'label')
        tmLabels = {site.textureMaps.label}';
        tmIndx = strmatch(site.startTM, tmLabels);
        set(handles.TextureMapDispSel,'string',[tmLabels;get(handles.TextureMapDispSel,'string')])
    else
        tmIndx = [];
    end
    zmin = min(site.zElev(:));
    if ~isempty(tmIndx)
        dsp.curTextureMap = surf(site.xMesh,site.yMesh,site.zElev*0+zmin,'FaceColor','texturemap','edgecolor','none',...
            'cdata',flipud(site.textureMaps(tmIndx).textureImage),'cdatamapping','direct',...
            'hittest','off');
        colormap(site.textureMaps(tmIndx).textureCmap)
        
    else
        dsp.curTextureMap = surf(site.xGrid,site.yGrid,site.zElev*0+zmin,'FaceColor',[1 1 1],'edgecolor','none',...
            'cdata',site.zElev);
    end
    set(handles.Display2dImagePB,'string','3D Imagery')
    drawnow
end

if exist('ter','var')
    currTer = getappdata(handles.MasterFigure,'terrainData');
    % Where fields exist in newly loade "ter" copy the data to the 
    % current terrain data -- new ter may not contain all default data
    fn = fieldnames(ter); %#ok<NODEF>
    cfn = fieldnames(currTer);
    
    % If saved data has legacy format terrain list convert the data
    if strmatch('terrainList', fn, 'exact') && isfield(ter.terrainList,'hue')
        terrainList = repmat(struct('type',[],'RGB',[],'width',[],'style',[],...
            'characteristics',[]),size(ter.terrainList));
        for iTL = 1:numel(ter.terrainList)
            terrainList(iTL).type = ter.terrainList(iTL).type;
            terrainList(iTL).RGB = ter.terrainList(iTL).hue;
        end
        ter.terrainList = terrainList;
    end
        
    for iField = 1:numel(fn)
        if strmatch(fn{iField}, cfn, 'exact')
            if ~isempty(strmatch(fn{iField}, 'face', 'exact')) && isfield(ter.face,'hue')
                for iFace = 1:numel(ter.face)
                    currTer.face(iFace).type = ter.face(iFace).type;
                    currTer.face(iFace).RGB = ter.face(iFace).hue;
                end
            else
                currTer.(fn{iField}) = ter.(fn{iField});
            end
        else
            missmatch.(fn{iField}) = ter.(fn{iField});
        end
    end
    ter = currTer;
    ter.hMasterFigure = gcf;
    
    % Set terrain select pushbuttons
    ter = updateTerrainSelectDisplay(ter,handles);
    drawnow
    
    % Set obstacle options
    if ~isempty(obs.obstacleList)
        set(handles.ObstacleEditingPopup,'string',{obs.obstacleList.type}') %#ok<NODEF>
    end
    
    % Redraw terrain map faces and face centers
    for iFace = 1:ter.nFace
        vList = ter.facet2vert{iFace};
        X = ter.vertexXYZ(vList,1);
        Y = ter.vertexXYZ(vList,2);
        Z = ter.vertexXYZ(vList,3);
        fc = ter.face(iFace).RGB; 
        dsp.hFace(iFace) = patch(X,Y,Z+ter.altOffset,Z,'FaceColor',fc,'facealpha',...
            dsp.curTerrainAlpha,'edgecolor','none','HitTest','off','ButtonDownFcn',[]);
        dsp.mob_hVertex(iFace) = plot3(ter.faceCenterXYZ(iFace,1),...
            ter.faceCenterXYZ(iFace,2),ter.faceCenterXYZ(iFace,3)+ter.altOffset+.1,...
            'bo','MarkerSize',2,'MarkerFaceColor',[0 0 1],'MarkerEdgeColor',[0 0 1],...
            'HitTest','off');
    end
    drawnow
    
    % Redraw terrain map edges
    for iEdge = 1:ter.nEdge
        vList = ter.edge2vert(iEdge,1:2);
        X = ter.vertexXYZ(vList,1);
        Y = ter.vertexXYZ(vList,2);
        Z = ter.vertexXYZ(vList,3);
        dsp.hEdge(iEdge) = plot3(X,Y,Z+ter.altOffset,'color',[1 0 0],'linewidth',.5,'HitTest','on','ButtonDownFcn',[]);
    end
    drawnow

    % Redraw terrain map vertices
    for iVert = 1:ter.nVertex
        X = ter.vertexXYZ(iVert,1);
        Y = ter.vertexXYZ(iVert,2);
        Z = ter.vertexXYZ(iVert,3);
        dsp.hVertex(iVert) = plot3(X,Y,Z+ter.altOffset,'o','markerfacecolor',[1 0 0],'markerEdgeColor',[1 0 0],...
            'MarkerSize',3,'HitTest','off','ButtonDownFcn',[]);
    end
    drawnow
    
    % Redraw mobility network edges
    for iMobEdge = find(all(ter.edge2face,2))'
        adjFaces = ter.edge2face(iMobEdge,:);
        dsp.mob_hEdge(iMobEdge) = plot3(ter.faceCenterXYZ(adjFaces,1),...
            ter.faceCenterXYZ(adjFaces,2),ter.faceCenterXYZ(adjFaces,3)+ter.altOffset,...
            'color',[0 0 1],'linewidth',1,'HitTest','off');
        ter = updateMobNetDist(ter,iMobEdge);
    end
    drawnow
    
    setappdata(handles.MasterFigure,'terrainData',ter)
    setappdata(handles.MasterFigure,'displayData',dsp)
end

if exist('obs','var') && ~isempty(obs)
    fn = fieldnames(obs); %#ok<NODEF>
    currObs = getappdata(handles.MasterFigure,'obstacleData');
    cfn = fieldnames(currObs);
    for iField = 1:numel(fn)
        if strmatch(fn{iField}, cfn, 'exact')
            currObs.(fn{iField}) = obs.(fn{iField});
        end
    end
    obs = currObs;
    
    % Redraw obstacles
    for iObs = 1:numel(obs.edgeObstacles)
        if ~isempty(obs.edgeObstacles{iObs})
            currObs = obs.edgeObstacles{iObs};
            xd = get(dsp.hEdge(iObs),'xdata');
            yd = get(dsp.hEdge(iObs),'ydata');
            zd = get(dsp.hEdge(iObs),'zdata');
            dsp.hEdgeObstacles(iObs) = plot3(xd,yd,zd+.01,'LineStyle',currObs.style,...
                'color',currObs.RGB,'linewidth',currObs.width,'HitTest','off');
        end
    end
    if numel(obs.edgeObstacles) < numel(dsp.hEdge)
        obs.edgeObstacles{numel(dsp.hEdge)} = [];
        dsp.hEdgeObstacles(numel(dsp.hEdge)) = 0;
    end
    setappdata(handles.MasterFigure,'obstacleData',obs) %#ok<NODEF>
    setappdata(handles.MasterFigure,'displayData',dsp)
end
if exist('threat','var') && ~isempty(threat)
    setappdata(handles.MasterFigure,'threatData',threat)
    set(handles.selectThreat,'string',{threat.threatDB.threat}',...
        'value',1)
    setappdata(handles.MasterFigure,'threatData',threat);
end
if exist('mob','var') && ~isempty(mob)
    setappdata(handles.MasterFigure,'mobilityData',mob)
end
if exist('str','var') && ~isempty(str)
    setappdata(handles.MasterFigure,'structureData',str)
end
if exist('feat','var') && ~isempty(feat)
    setappdata(handles.MasterFigure,'featureData',feat)
end
if exist('ess','var') && ~isempty(ess) %#ok<NODEF>
    % If saved display data is available update current display data
    fn = fieldnames(ess); %#ok<NODEF>
    currESS = getappdata(handles.MasterFigure,'sensorData');
    cfn = fieldnames(currESS);
    for iField = 1:numel(fn)
        if strmatch(fn{iField}, cfn, 'exact')
            currESS.(fn{iField}) = ess.(fn{iField});
        end
    end
    ess = currESS;
    % Reset but do not display sensor editing parameters
    ess.editMode = 1;
    ess.currentSensor = 0;
    ess.currentSensorParam = ess.sensorList(1);
    updateSensorEditData(ess,handles)
    ess.editMode = 0;
    updateSensorEditData(ess,handles)
    
    % Redraw line and volume sensors
    lsIndex = find(~cellfun(@isempty, ess.lineSensors));
    if ~isempty(lsIndex)
        for iLS = lsIndex
            thisSensor = ess.lineSensors{iLS};
            dsp.hLineSensors(iLS) = plot3(...
                thisSensor.Xlocation,thisSensor.Ylocation,thisSensor.Zlocation,...
                'LineStyle',thisSensor.edgeStyle,'color',thisSensor.edgeRGB,...
                'linewidth',thisSensor.edgeWidth,'HitTest','off');
        end
    end
    
    vsIndex = find(~cellfun(@isempty, ess.volumeSensors));
    if ~isempty(vsIndex)
        for iVS = vsIndex

            ess.currentSensorParam = ess.volumeSensors{iVS};
            dsp.hVolSensorCoverage(iVS) = makeVolSensorPatch(ess,handles);
            
            dsp.hVolSensorPos(iVS) = plot3(ess.currentSensorParam.Xlocation,...
                ess.currentSensorParam.Ylocation,ess.currentSensorParam.Zlocation,'k-^',...
                'marker','^','markersize',6','markerFaceColor','k','markerEdgeColor','k',...
                'linewidth',2,'lineStyle',ess.currentSensorParam.edgeStyle,'HitTest','off');
        end
        updateSensorEditData(ess,handles)
    end
    drawnow
    
    % Redraw defended point and intruder sink
    if ~isempty(dsp.defendedPoint) && dsp.defendedPoint ~= 0
        dsp.hDefendedPoint = plot3(ter.faceCenterXYZ(dsp.defendedPoint,1),...
            ter.faceCenterXYZ(dsp.defendedPoint,2),...
            ter.faceCenterXYZ(dsp.defendedPoint,3)+ter.altOffset + .1,...
            'marker','o','markeredgecolor','g','markerfacecolor','g',...
            'markersize',8,'visible','on');
    end    
    if ~isempty(dsp.intruderSource) && dsp.intruderSource ~= 0
        dsp.hIntruderSource = plot3(ter.faceCenterXYZ(dsp.intruderSource,1),...
            ter.faceCenterXYZ(dsp.intruderSource,2),...
            ter.faceCenterXYZ(dsp.intruderSource,3)+ter.altOffset + .1,...
            'marker','d','markeredgecolor','r','markerfacecolor','r',...
            'markersize',8,'visible','on');
    end
    
    set(handles.SensorEditingSelectSensor,'string',{ess.sensorList.model}')
    setappdata(handles.MasterFigure,'sensorData',ess)
    setappdata(handles.MasterFigure,'displayData',dsp)
end
if exist('util','var') && ~isempty(util)
    setappdata(handles.MasterFigure,'utilityData',util)
end
set(handles.MasterFigure,'Pointer','arrow')

%% Initialize SSES project based on user interaction
function initializeSSESproject(hObject, eventdata, handles)

site = getappdata(handles.MasterFigure,'siteData');
hasTexture = false;
hasElevation = false;

% Project label
str = inputdlg({'Enter site label   .'},'Site Label',1,{'New site'});
if isempty(str)
    beep
    return
end
site.label = str;

% Get x and y coordinate limits
str = inputdlg({'Enter site west coordinate   .','Enter site east coordinate',...
    'Enter site south coordinate','Enter site north coordinate'},'Enter Site Limits',1,{'0','','0',''});
mlim = str2double(str);
if ~any(isnan(mlim(1:2))) && mlim(1) < mlim(2)
    site.xLimit = mlim(1:2)';
else
    beep
    site.xLimit = [0 100];
end
if ~any(isnan(mlim(3:4))) && mlim(3) < mlim(4)
    site.yLimit = mlim(3:4)';
else
    beep
    site.yLimit = [0 100];
end
set(handles.MainAxes,'xlim',site.xLimit,'ylim',site.yLimit)

% Get map units
str = inputdlg({'Enter X units   .','Enter Y units','Enter Z units'},...
    'Enter units',1,{'meters','meters','meters'});
if ~isempty(str)
    site.xUnits = str{1};
    site.yUnits = str{2};
    site.zUnits = str{3};
else
    site.xUnits = 'meters';
    site.yUnits = 'meters';
    site.zUnits = 'meters';
end

% Get axis labels
str = inputdlg({'Enter X axis label   .','Enter Y axis label','Enter Z axis label'},...
    'Enter Axis Labels',1,{'East (m)','North (m)','Elevation (m)'});
if ~isempty(str)
    site.xLabel = str{1};
else
    beep
    site.xLabel = 'East (m)';
end
if numel(str) > 1
    site.yLabel = str{2};
else
    beep
    site.yLabel = 'North (m)';
end
if numel(str) > 2
    site.zLabel = str{3};
else
    beep
    site.zLabel = 'Elevation (m)';
end
xlabel(site.xLabel)
ylabel(site.yLabel)
zlabel(site.zLabel)

% Set axis ticks
xt = get(handles.MainAxes,'xtick');
yt = get(handles.MainAxes,'ytick');
str = inputdlg({'Enter X axis spacing   .','Enter Y axis spacing',},'Enter Axis spacing'...
    ,1,{int2str(xt(2)-xt(1)),int2str(yt(2)-yt(1))});
if ~isempty(str)
    sn = str2double(str);
    if ~isnan(sn(1))
        xt = site.xLimit(1):sn(1):site.xLimit(2);
        if xt(end) ~= site.xLimit(2)
            xt = [xt site.xLimit(2)];
        end
    end
    if numel(sn) > 1 && ~isnan(sn(2))
        yt = site.yLimit(1):sn(1):site.yLimit(2);
        if yt(end) ~= site.yLimit(2)
            yt = [yt site.yLimit(2)];
        end
    end
end
set(handles.MainAxes,'xtick',xt,'ytick',yt)
site.xGridLines = xt;
site.yGridLines = yt;

% Load site imagery
button = questdlg('Do you want to load site imagery?    .','Load site imagery','Yes','No','Yes');
iTM = 1;
while strcmpi(button,'Yes')
    % Get project file and path
    [textureFN, texturePath] = uigetfile('*.*','Select image file');
    
    if ischar(textureFN)
        try
            RGB = imread(fullfile(texturePath,textureFN));
            
            iptbOK = exist('rgb2ind','file');
            if iptbOK == 2
                [I,map] = rgb2ind(RGB,256);
            else
                hw = warndlg({'MATLAB Image Processing Toolbox not found            .',...
                    'Converting image to grayscale texturemap'});
                uiwait(hw)
                RGBd = double(RGB);
                I = RGBd(:,:,1)*0.30 + RGBd(:,:,2)*0.59 + RGBd(:,:,3)*0.11;
                figure('visible','off')
                map = colormap(gray(256));
                close(gcf)
            end
            tm(iTM).label =  textureFN;
            tm(iTM).textureImage = I;
            tm(iTM).textureCmap = map;
            hasTexture = true;
            iTM = iTM + 1;
        catch
            hasTexture = false;
        end 
    else
        beep  
    end
    button = questdlg('Do you want to load another image?    .','Load site imagery','Yes','No','Yes');
end
% if no texturemap was loaded generate default (blank) texturemap
if exist('tm','var')
    site.textureMaps = tm;
    site.startTM = tm(1).label;
else
    I = ones(2,2);
    map = colormap;
    tm.label = 'none';
    tm.textureImage = I;
    tm.textureCmap = map;
    site.textureMaps = tm;
    site.startTM = tm.label;
end         

h = image(tm(1).textureImage);
set(handles.MainAxes,'ydir','normal')
set(gcf,'colormap',tm(1).textureCmap)

% Load elevation data
button = questdlg('Do you want to load site elevation data?    .','Load elevation data','Yes','No','Yes');
if strcmpi(button,'Yes')
    % Get project file and path
    [elFN, elPath] = uigetfile('*.mat','Select elevation data file');
    
    if ischar(elFN)
        load(fullfile(elPath,elFN));
        if exist('xMesh','var') && exist('yMesh','var') && exist('zElev','var') && ...
                all(size(xMesh) == size(yMesh)) && all(size(xMesh) == size(zElev))
            hasElevation = true;
            
            xl = [min(xMesh(:)) max(xMesh(:))];
            yl = [min(yMesh(:)) max(yMesh(:))];
            if any(xl ~= site.xLimit) || any(yl ~= site.yLimit)
                hw = warndlg({'Elevation data does not match site coordinates       .',...
                    'Image data will probably be distorted'});
                uiwait(hw)
            
                % If necessary expand elevation grid data to cover the full site 
                if site.xLimit(1) < xMesh(1)
                   xMesh = [xMesh(:,1)*0+site.xLimit(1) xMesh];
                   yMesh = [yMesh(:,1) yMesh];
                   zElev = [zElev(:,1) zElev];                
                end
                if site.xLimit(2) > xMesh(end)
                   xMesh = [xMesh xMesh(:,end)*0+site.xLimit(2)];
                   yMesh = [yMesh yMesh(:,end)];
                   zElev = [zElev zElev(:,end)];                
                end
                if site.yLimit(1) < yMesh(1)
                   xMesh = [xMesh(1,:); xMesh];
                   yMesh = [yMesh(1,:)*0+site.yLimit(1); yMesh];
                   zElev = [zElev(1,:); zElev];                
                end
                if site.yLimit(2) > yMesh(end)
                   xMesh = [xMesh; xMesh(end,:)];
                   yMesh = [yMesh; yMesh(end,:)*0+site.yLimit(2)];
                   zElev = [zElev; zElev(end,:)];                
                end
                
            end
            site.xMesh = xMesh;
            site.yMesh = yMesh;
            site.zElev = zElev;
            site.xGrid = xMesh(1,:);
            site.yGrid = yMesh(:,1);            
        else
            hw = warndlg({'Elevation data was not properly formatted            .',...
                    'Elevation data file must contain "xMesh" and "yMesh" and',...
                    '"zElev" variables in meshgrid format',...
                    'Elevation data not used'});
            uiwait(hw)
            site.xMesh = [site.xLimit; site.xLimit];
            site.yMesh = [site.yLimit' site.xLimit'];
        end
            
    else
        beep
        site.xMesh = [site.xLimit;site.xLimit];
        site.yMesh = [site.yLimit' site.xLimit'];
    end
else
    site.xMesh = [site.xLimit;site.xLimit];
    site.yMesh = [site.yLimit' site.xLimit'];
end

% If image and elevation data were provided draw texturemap
if hasTexture && hasElevation
    delete(h)
    surf(site.xMesh,site.yMesh,site.zElev,'FaceColor','texturemap','edgecolor','none',...
        'cdata',flipud(site.textureMaps(1).textureImage),'cdatamapping','direct',...
        'hittest','off');
    colormap(site.textureMaps(1).textureCmap)
end
set(handles.MainAxes,'ZLimMode','auto')
site.zGridLines = get(handles.MainAxes,'ztick');
site.zLimit = get(handles.MainAxes,'zlim');

setappdata(handles.MasterFigure,'siteData',site);

%% saveSSESproject
function saveSSESproject(hObject, eventdata, handles)
%Saves SSES project data to .mat file

[filename,pathname] = uiputfile('*SSES.mat');

site = getappdata(handles.MasterFigure,'siteData'); %#ok<NASGU>
ter = getappdata(handles.MasterFigure,'terrainData'); %#ok<NASGU>
dsp = getappdata(handles.MasterFigure,'displayData'); %#ok<NASGU>
obs = getappdata(handles.MasterFigure,'obstacleData'); %#ok<NASGU>
threat = getappdata(handles.MasterFigure,'threatData'); %#ok<NASGU>
mob = getappdata(handles.MasterFigure,'mobilityData'); %#ok<NASGU>
str = getappdata(handles.MasterFigure,'structureData'); %#ok<NASGU>
feat = getappdata(handles.MasterFigure,'featureData'); %#ok<NASGU>
ess = getappdata(handles.MasterFigure,'sensorData'); %#ok<NASGU>
util = getappdata(handles.MasterFigure,'utilityData'); %#ok<NASGU>

if ~ischar(filename)
    beep
    warning('Matlab:ssesPrototype','Invalid filename -- project not saved')
    return
end

sIndx = findstr(filename,'SSES.mat');
if ~isempty(sIndx)
    if sIndx+7 == numel(filename)
        try
            save(fullfile(pathname,filename),'site','ter','dsp','obs','threat','mob','str','feat','ess','util')
        catch
            beep
            warning('Matlab:SSESprototype','Unable to save project to intended file - saved in recoverySSES.mat')
            save(fullfile(pathname,'recoverySSES.mat'),'site','ter','dsp','obs','threat','mob','str','feat','ess','util')
        end
    end    
else
    sIndx = findstr(filename,'.mat');
    if sIndx+3 == numel(filename)
        filename = [filename(1:sIndx-1) 'SSES' '.mat'];
        try
            save(fullfile(pathname,filename),'site','ter','dsp','obs','threat','mob','str','feat','ess','util')
        catch
            beep
            warning('Matlab:SSESprototype','Unable to save project to intended file - saved in recoverySSES.mat')
            save(fullfile(pathname,'recoverySSES.mat'),'site','ter','dsp','obs','threat','mob','str','feat','ess','util')
        end
    else
        beep
        warning('Matlab:ssesPrototype','Invalid filename -- project not saved')
    end
end

%% import SSES project data from external Excel workbooks
function importSSESprojectData(hObject, eventdata, handles)
% Enables external data import interface panel

set(handles.ImportDataPanel,'visible','on')
uic = findobj(handles.ImportDataPanel,'type','uicontrol');
set(uic,'visible','on','HitTest','on')
set(hObject,'State','off')

%% Complete import SSES project data
function ImportDataDone_Callback(hObject, eventdata, handles)
% Disable and hide external data import interface panel

set(handles.ImportDataPanel,'visible','off')
ccc = get(handles.ImportDataPanel,'children');
set(ccc,'visible','off','HitTest','off')

%% TerrainMapAlphaSlider
% --- Executes on slider movement
function TerrainMapAlphaSlider_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
dsp.curTerrainAlpha = get(hObject,'Value');
if dsp.curTerrainAlpha ~=0
    set(dsp.hFace,'FaceAlpha',dsp.curTerrainAlpha,'visible','on')
else
    set(dsp.hFace,'FaceAlpha',dsp.curTerrainAlpha,'visible','off')
end
setappdata(handles.MasterFigure,'displayData',dsp)

% --- Executes during slider creation, after setting all properties.
function TerrainMapAlphaSlider_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

%% ImageryAlphaSlider
% --- Executes on slider movement
function ImageryAlphaSlider_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
dsp.curImageMapAlpha = get(hObject,'Value');
if dsp.curImageMapAlpha ~=0
    set(dsp.curTextureMap,'FaceAlpha',dsp.curImageMapAlpha,'visible','on')
else
    set(dsp.curTextureMap,'FaceAlpha',dsp.curImageMapAlpha,'visible','off')
end
    
setappdata(handles.MasterFigure,'displayData',dsp)

% --- Executes during object creation, after setting all properties.
function ImageryAlphaSlider_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

%% Show XY Position
function show_XY_position(state,handles)
set(handles.xTextLab,'visible',state)
set(handles.yTextLab,'visible',state)
set(handles.xText,'visible',state,'string','')
set(handles.yText,'visible',state,'string','')

%% Show L1L2 Length
function show_L1L2_length(state,handles)
set(handles.L1TextLab,'visible',state)
set(handles.L2TextLab,'visible',state)
set(handles.L1Text,'visible',state,'string','')
set(handles.L2Text,'visible',state,'string','')

%% Update XY position
function updateCursorPos(obj,opt,handles)
cp = get(handles.MainAxes,'CurrentPoint');
xl = get(handles.MainAxes,'xlim');
yl = get(handles.MainAxes,'ylim');

if xl(1) <= cp(1,1) && cp(1,1) <= xl(2) && yl(1) <= cp(1,2) && cp(1,2) <= yl(2)
    set(handles.xText,'string',num2str(cp(1,1),'%6.1f'))
    set(handles.yText,'string',num2str(cp(1,2),'%6.1f'))

    if opt == 2
        ter = getappdata(handles.MasterFigure,'terrainData');
        if ~isempty(ter.selectedEdge)
            
            vXYZ = ter.vertexXYZ(ter.edge2vert(ter.selectedEdge,:),1:3);
            L1 = norm(vXYZ(1,1:2)-cp(1,1:2));
            L2 = norm(vXYZ(2,1:2)-cp(1,1:2));
            L = sort([L1 L2]);
            set(handles.L1Text,'string',num2str(L(1),'%6.1f'))
            set(handles.L2Text,'string',num2str(L(2),'%6.1f'))
        else
            set(handles.L1Text,'string','')
            set(handles.L2Text,'string','')
        end
    end
    
else
    set(handles.xText,'string','')
    set(handles.yText,'string','')
    if opt == 2
        set(handles.L1Text,'string','')
        set(handles.L2Text,'string','')
    end
end

%% Select threat
function selectThreat_Callback(hObject, eventdata, handles)
v = get(hObject,'Value');
str = get(hObject,'string');
threat = getappdata(handles.MasterFigure,'threatData');
threat.currentThreatType = str{v};
threat.currentThreatIndex = v;
setappdata(handles.MasterFigure,'threatData',threat);

% --- Executes during object creation, after setting all properties.
function selectThreat_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

%% Import Threat Data
function ImportThreatData_Callback(hObject, eventdata, handles)

% Get / check database file name and path
[projFN, projPath] = uigetfile('*.xls*','Select Threat Database');
if ~ischar(projFN) || ~ischar(projPath)
    beep
    return
end    
fullPathName = fullfile(projPath,projFN);

if ~exist(fullPathName,'file')
    beep
    return
end

set(handles.ImportDataWaitbar,'visible','on')
axes(handles.ImportDataWaitbar)
hWB = patch([0 0 0 0 0],[0 0 1 1 0],'b');
drawnow

% Read database file
[fileType, threatList] = xlsfinfo(fullPathName); 

if ~strcmpi(fileType,'Microsoft Excel Spreadsheet')
    warning('Matlab:SSESprototype','The specified file is not a valid Excel worksheet')
    beep
    return
end

tdb = struct(...
    'threat',[],...
    'terrainType',{},...
    'movementRateKPH',[],...
    'movementRateMSEC',[],...
    'signatures',{});
threatDB = repmat(tdb,numel(threatList),1);

for iSheet = 1:numel(threatList)
    [num,txt] = xlsread(fullPathName, threatList{iSheet});
    
    threatDB(iSheet).threat = threatList{iSheet};
    
    set(hWB,'xdata',[0 .2 .2 0 0])
    
    labelList = {txt{:,1}}';
    mrHeader = strmatch('Movement Rates',labelList);
    if ~isempty(mrHeader)
        terTypeList = {labelList{(mrHeader+1):end}}';
        firstBlank = strmatch('',terTypeList, 'exact');
        if isempty(firstBlank)
            firstBlank = numel(terTypeList+1);
        end
        rngStr = ['B' int2str(mrHeader+1) ':B' int2str(mrHeader+firstBlank(1))];
        moveRate = xlsread(fullPathName, threatList{iSheet},rngStr);
        threatDB(iSheet).terrainType = {terTypeList{1:(firstBlank(1)-1)}}';
        speedFormat = txt{mrHeader,2};
        if ~isempty(strfind(speedFormat,'kmh'))
            threatDB(iSheet).movementRateKPH = moveRate;
            threatDB(iSheet).movementRateMSEC = moveRate*1000/3600;
        elseif ~isempty(strfind(speedFormat,'msec'))
            threatDB(iSheet).movementRateKPH = moveRate*3600/1000;
            threatDB(iSheet).movementRateMSEC = moveRate;
        elseif ~isempty(strfind(speedFormat,'mph'))
            threatDB(iSheet).movementRateKPH = moveRate*1.6093;
            threatDB(iSheet).movementRateMSEC = moveRate*1609.3/3600;
        else
            warning('Matlab:SSESprototype',['Speed format not recognized for threat type "' threatList{iSheet} '" zero speed assigned'])
            threatDB(iSheet).movementRateKPH = moveRate*0;
            threatDB(iSheet).movementRateMSEC = moveRate*0;
        end
    end
        
    btHeader = strmatch('Breach Times',labelList);
    if ~isempty(btHeader)
        obsTypeList = {labelList{(btHeader+1):end}}';
        firstBlank = strmatch('',obsTypeList, 'exact');
        if isempty(firstBlank)
            firstBlank = numel(terTypeList+1);
        end
        rngStr = ['B' int2str(btHeader+1) ':B' int2str(btHeader+firstBlank(1))];
        breachTime = xlsread(fullPathName, threatList{iSheet},rngStr);
        breachTime(isnan(breachTime)) = inf;
        threatDB(iSheet).ObstacleType = {obsTypeList{1:(firstBlank(1)-1)}}';
        threatDB(iSheet).breachTimeSec = breachTime;
    end
    
    s = struct;
    sigHeader = strmatch('signatures',lower(labelList));
    if ~isempty(sigHeader)
        sigTypeList = {labelList{(sigHeader+1):end}}';
        firstBlank = strmatch('',sigTypeList, 'exact');
        if isempty(firstBlank)
            firstBlank = numel(sigTypeList);
        end
        rngStr = ['B' int2str(sigHeader+1) ':B' int2str(sigHeader+firstBlank(1))];
        sigData = xlsread(fullPathName, threatList{iSheet},rngStr);
        for iSD = 1:numel(sigData)
            s.(sigTypeList{iSD}) = sigData(iSD);
        end
        threatDB(iSheet).signatures = s;
    end
    
    fc = .2 + .8*iSheet/numel(threatList);
    set(hWB,'xdata',[0 fc fc 0 0])
    drawnow
end

threat.currentThreatType = threatDB(1).threat;
threat.currentThreatIndex = 1;
threat.threatDB = threatDB;
threat.sourceFaceList = {};

set(handles.selectThreat,'string',{threat.threatDB.threat}','value',1)
setappdata(handles.MasterFigure,'threatData',threat);

delete(hWB)
set(handles.ImportDataWaitbar,'visible','off')

%% Import Terrain Data
function ImportTerrainData_Callback(hObject, eventdata, handles)

% Get / check database file name and path
[projFN, projPath] = uigetfile('*.xls*','Select Terrain / Obstacle Database');
if ~ischar(projFN) || ~ischar(projPath)
    beep
    return
end    
fullPathName = fullfile(projPath,projFN);

if ~exist(fullPathName,'file')
    beep
    return
end

set(handles.ImportDataWaitbar,'visible','on')
axes(handles.ImportDataWaitbar)
hWB = patch([0 0 0 0 0],[0 0 1 1 0],'b');
drawnow

% Read database file
[fileType, terrainList] = xlsfinfo(fullPathName); 

if ~strcmpi(fileType,'Microsoft Excel Spreadsheet')
    warning('Matlab:SSESprototype','The specified file is not a valid Excel worksheet')
    beep
    return
end

tList = struct('type','','RGB',[1 1 1],'width',1,'style','-','characteristics',[]);
oList = struct('type','','RGB',[1 1 1],'width',1,'style','-','characteristics',[]);
nTl = 0;
nOl = 0;
set(hWB,'xdata',[0 .2 .2 0 0])
drawnow

for iSheet = 1:numel(terrainList)
    [num,txt] = xlsread(fullPathName, terrainList{iSheet});
    
    cIndex = strmatch('Category', txt, 'exact');
    if isempty(cIndex)
         warning('Matlab:SSESprototype','Entry is missing "Catgory" flag -- ignored')
         continue
    end
    category = txt{cIndex,2};
    
    switch lower(category)
        case 'terrain'
            nTl = nTl + 1;
            tIndex = strmatch('Type', txt, 'exact');
            if ~isempty(tIndex)
                 tList(nTl).type = txt{tIndex,2};
            else
                tList(nTl).type = ['Terrain' int2str(nTl)];
            end
            rIndex = strmatch('RGB', txt, 'exact');
            if ~isempty(rIndex)
                rowStr = int2str(rIndex); 
                rngStr = ['B' rowStr ':D' rowStr];
                tList(nTl).RGB = xlsread(fullPathName, terrainList{iSheet},rngStr);
            else
                tList(nTl).RGB =[1 1 1];
            end
            
            % Put code to read terrain characteristics here
            
        case 'obstacle'
            nOl = nOl + 1;
            tIndex = strmatch('Type', txt, 'exact');
            if ~isempty(tIndex)
                 oList(nOl).type = txt{tIndex,2};
            else
                oList(nOl).type = ['Obstacle' int2str(nOl)];
            end
            rIndex = strmatch('RGB', txt, 'exact');
            if ~isempty(rIndex)
                rowStr = int2str(rIndex); 
                rngStr = ['B' rowStr ':D' rowStr];
                oList(nOl).RGB = xlsread(fullPathName, terrainList{iSheet},rngStr);
            else
                oList(nOl).RGB =[1 1 1];
            end
            wIndex = strmatch('Width', txt, 'exact');
            if ~isempty(wIndex)
                oList(nOl).width = xlsread(fullPathName, terrainList{iSheet},['B' int2str(wIndex)]);
            else
                oList(nOl).width = 1;
            end
            sIndex = strmatch('Style', txt, 'exact');
            if ~isempty(sIndex)
                oList(nOl).style = txt{sIndex,2};
            else
                oList(nOl).style = '-';
            end
        otherwise
            warning('Matlab:SSESprototype','Terrain catgory not recognized -- ignored')
    end
    
    fc = .2 + .8*iSheet/numel(terrainList);
    set(hWB,'xdata',[0 fc fc 0 0])
    drawnow
end
ter = getappdata(handles.MasterFigure,'terrainData');
ter.terrainList = tList;
ter = updateTerrainSelectDisplay(ter,handles);
set(ter.hTerSel,'visible','on')
setappdata(handles.MasterFigure,'terrainData',ter)

obs = getappdata(handles.MasterFigure,'obstacleData');
obs.curObs = [];
obs.obstacleList = oList;
if ~isempty(obs) && isfield(obs,'obstacleList') && isfield(obs.obstacleList,'type')
    set(handles.ObstacleEditingPopup,'string',{obs.obstacleList.type}')
end
setappdata(handles.MasterFigure,'obstacleData',obs)

delete(hWB)
set(handles.ImportDataWaitbar,'visible','off')

%% Import Sensor Data
function ImportSensorData_Callback(hObject, eventdata, handles)

ess = getappdata(handles.MasterFigure,'sensorData');

% Get / check database file name and path
[projFN, projPath] = uigetfile('*.xls*','Select Terrain / Obstacle Database');
if ~ischar(projFN) || ~ischar(projPath)
    beep
    return
end    
fullPathName = fullfile(projPath,projFN);

if ~exist(fullPathName,'file')
    beep
    return
end

set(handles.ImportDataWaitbar,'visible','on')
axes(handles.ImportDataWaitbar)
hWB = patch([0 0 0 0 0],[0 0 1 1 0],'b');
drawnow

s = struct(...
    'model','',...
    'class','',...
    'type','',...
    'sensorLabel','',...
    'emission','',...
    'PdMode','Pd_const',...
    'Pd_const',0,...
    'Pd_lookup',[],...
    'Pd_model',[],...
    'effRange',0,...
    'Xlocation',[],...
    'Ylocation',[],...
    'Zlocation',[],...
    'mountHeight',0,...
    'AZlimit',[],...
    'ELlimit',[],...
    'AZmax',0,...
    'ELmax',0,...
    'unitCost',0,...
    'elementCost',0,...
    'supportCost',0,...
    'totalCost',0,...
    'edgeRGB',[.5 .5 .5],...
    'edgeWidth',0.5,...
    'edgeStyle','-',...
    'faceRGB',[1 1 1],...
    'faceAlpha',0.3);
fn = fieldnames(s);

[typ, sheets] = xlsfinfo(fullPathName);
sl = repmat(s,1,numel(sheets));

% Read each sheet
for iSheet = 1:numel(sheets)
    [num, txt] = xlsread(fullPathName, sheets{iSheet});

    % Parse sheet data
    for i = 2:size(txt,1)
        
        % Discard data that does not match sensorList field names
        ind = strmatch(txt{i,1}, fn, 'exact');
        if ~isempty(ind)
           if ~strcmpi(txt{i,2},'')
               sl(iSheet).(fn{ind}) = txt{i,2};
           elseif ~isnan(num(i,2))
               n = num(i,2);
               j = 3;
               while j <= size(num,2) && ~isnan(num(i,j))
                   n = [n num(i,j)];
                   j = j+1;
               end
               sl(iSheet).(fn{ind}) = n;
           end
        end
    end
    fc = iSheet/numel(sheets);
    set(hWB,'xdata',[0 fc fc 0 0])
    drawnow
end
ess.sensorList = sl;
set(handles.SensorEditingSelectSensor,'string',sheets)

setappdata(handles.MasterFigure,'sensorData',ess);
delete(hWB)
set(handles.ImportDataWaitbar,'visible','off')

%% Import Environment Data -- not currently implemented
function ImportEnvironmentData_Callback(hObject, eventdata, handles)
% For future expansion -- not currently implemented
beep

%% Select Environment -- not currently implemented.
function selectEnvironment_Callback(hObject, eventdata, handles)
beep

% --- Executes during object creation, after setting all properties.
function selectEnvironment_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

%% Update terrain select display
function ter = updateTerrainSelectDisplay(ter,handles)
terrainList = ter.terrainList;
nTerType = numel(terrainList);
hPB = zeros(16,1);
for iTerSel = 1:16
    hPB(iTerSel) = eval(['handles.TerrainSel' int2str(iTerSel)]);
    if iTerSel <= nTerType
        set(hPB(iTerSel),'BackgroundColor',terrainList(iTerSel).RGB,...
            'String',terrainList(iTerSel).type,'visible','off','HitTest','on')
        if sum(terrainList(iTerSel).RGB) <= 1.1
            set(hPB(iTerSel),'ForegroundColor',[1 1 1])
        end
    else
        set(hPB(iTerSel),'Visible','off','HitTest','off')
    end
end
ter.hTerSel = hPB;
set(hPB(1),'value',1)

%% Update movement cost (time)
function updateMovementCost_Callback(hObject, eventdata, handles)

ter = getappdata(handles.MasterFigure,'terrainData');
mob = getappdata(handles.MasterFigure,'mobilityData');
dsp = getappdata(handles.MasterFigure,'displayData');
tht = getappdata(handles.MasterFigure,'threatData');
obs = getappdata(handles.MasterFigure,'obstacleData');

saveBGC = get(handles.updateMovementCost,'BackgroundColor');
savePTR = get(handles.MasterFigure,'Pointer');
set(handles.updateMovementCost,'BackgroundColor','y','SelectionHighlight','off');
set(handles.MasterFigure,'Pointer','watch')
drawnow

% Get current threat index and reinitialize cost data
cti = tht.currentThreatIndex;
mobEdgeList = find(all(mob.edge2face,2))';
mob.edgeCost = zeros(ter.nEdge,1);
mob.adjCost = spalloc(ter.nFace,ter.nFace,nnz(mob.eDist));

if numel(obs.edgeObstacles) < numel(dsp.hEdge)
    obs.edgeObstacles{numel(dsp.hEdge)} = {};
    setappdata(handles.MasterFigure,'obstacleData',obs);
end

for iME = mobEdgeList
    F = mob.edge2face(iME,1:2);
    fd12 = mob.eDist(F(1),F(2));
    mr1 = tht.threatDB(cti).movementRateMSEC(ter.faceTerIndex(F(1)));
    fd21 = mob.eDist(F(2),F(1));
    mr2 = tht.threatDB(cti).movementRateMSEC(ter.faceTerIndex(F(2)));
    
    if ~isempty(obs) && ~isempty(obs.edgeObstacles{iME})
        % Calculate additional movement cost due to obstacles
        obsIndex = strmatch(obs.edgeObstacles{iME}.type,tht.threatDB(cti).ObstacleType);
        if ~isempty(obsIndex)
            obsDelay = tht.threatDB(cti).breachTimeSec(obsIndex);
        else
            obsDelay = 0;
        end
    else
        obsDelay = 0;
    end
    
    if mr1 == 0 || mr2 == 0 || isnan(obsDelay)
        mob.edgeCost(iME) = inf;
    else
        mob.edgeCost(iME) = fd12/mr1 + fd21/mr2 + obsDelay;
    end
    mob.adjCost(F(1),F(2)) = mob.edgeCost(iME);
    mob.adjCost(F(2),F(1)) = mob.edgeCost(iME);
end

setappdata(handles.MasterFigure,'terrainData',ter);
setappdata(handles.MasterFigure,'mobilityData',mob);
setappdata(handles.MasterFigure,'displayData',dsp);

% If mobility edge display option is cost then update now 
val = get(handles.MobNetEdgeDispSel,'Value');
if val == 4 || val == 5
    MobNetEdgeDispSel_Callback([], [], handles)
end

set(handles.updateMovementCost,'BackgroundColor',saveBGC);
set(handles.MasterFigure,'Pointer',savePTR)
drawnow

%% Update probability of detection
function updatePd_Callback(hObject, eventdata, handles)

ter = getappdata(handles.MasterFigure,'terrainData');
mob = getappdata(handles.MasterFigure,'mobilityData');
dsp = getappdata(handles.MasterFigure,'displayData');
tht = getappdata(handles.MasterFigure,'threatData');
ess = getappdata(handles.MasterFigure,'sensorData');

saveBGC = get(handles.updatePd,'BackgroundColor');
savePTR = get(handles.MasterFigure,'Pointer');
set(handles.updatePd,'BackgroundColor','y','SelectionHighlight','off');
set(handles.MasterFigure,'Pointer','watch')
drawnow

% Get current threat index and initialize Pd data
cti = tht.currentThreatIndex;
thisThreat = tht.threatDB(cti);
ess.EdgePd = zeros(ter.nEdge,1);
ess.FacePd = zeros(ter.nFace,1);
ess.adjCost = spalloc(ter.nFace,ter.nFace,nnz(mob.eDist));

% Get list of mobility edges and line sensors
% mobEdgeList = find(all(mob.edge2face,2))';
lsList = find(~cellfun(@isempty,ess.lineSensors));
if numel(ess.lineSensors) < numel(dsp.hEdge)
    ess.lineSensors{numel(dsp.hEdge)} = [];
    setappdata(handles.MasterFigure,'sensorData',ess);
end

for iLS = lsList
    ess.EdgePd(iLS) = ess.lineSensors{iLS}.Pd_const; % **** DEBUG replace this with more sophisticated check
end

vsList = find(~cellfun(@isempty,ess.volumeSensors));
for iVS = vsList
    thisVS = ess.volumeSensors{iVS};
    tfc = ter.faceCenterXYZ;
    
    % Find terrain faces where detectability may exist
    % Within range
    effRng = thisVS.effRange;
    r = sqrt((tfc(:,1)-thisVS.Xlocation).^2 + (tfc(:,2)-thisVS.Ylocation).^2);
    eligFace = find(r<=effRng);
    
    % Within azimuth limits
    thisVS.AZlimit = mod(thisVS.AZlimit,360);
    if thisVS.AZmax < 360
        az = mod(90-(360/(2*pi))*(cart2pol(tfc(eligFace,1)-thisVS.Xlocation, ...
            tfc(eligFace,2)-thisVS.Ylocation)),360);
        if thisVS.AZlimit(1) <= thisVS.AZlimit(2)
            % "Nominal" case
            inAZ = (thisVS.AZlimit(1) <= az & az <= thisVS.AZlimit(2));
        else
            % "Zero crossing" case
            inAZ = (thisVS.AZlimit(1) <= az | az <= thisVS.AZlimit(2));
        end
        eligFace = eligFace(inAZ);
    end
    
    % Within elevation limits
    if isfield(thisThreat.signatures,'Height')
        tgtHgt = thisThreat.signatures.Height;
    else
        tgtHgt = 0;
    end
    dZ = tfc(eligFace,3) - thisVS.Zlocation;
    elTop = atand((dZ+tgtHgt)./r(eligFace));
    elBottom = atand(dZ./r(eligFace));
    inEL = thisVS.ELlimit(1) <= elTop & elBottom <= thisVS.ELlimit(2);
    eligFace = eligFace(inEL);
    
    % Clear line-of-sight
    blocked = false(size(eligFace));
    for iFace = 1:numel(eligFace)
        blocked(iFace) = checkLOS(thisVS,eligFace(iFace),tgtHgt,ter,dsp,handles);
    end
    eligFace = eligFace(~blocked);
    
    % Put Pd calculations here
    ess.FacePd(eligFace) = 1 - (1 - ess.FacePd(eligFace)).*(1 - thisVS.Pd_const);
end
ess.currPdValid = 1;
setappdata(handles.MasterFigure,'sensorData',ess);

% If mobility edge display option is cost then update now
if ~isempty(hObject)
    val = get(handles.TerrainPartitionFaceDispSel,'Value');
    if val == 4 
        updateTerrainMapWithFacePd(dsp)
    elseif val == 5
        updateTerrainMapWithPathPd(dsp)
    end
end

set(handles.updatePd,'BackgroundColor',saveBGC);
set(handles.MasterFigure,'Pointer',savePTR)
drawnow

%% Calulate Minimum Time Path
function calcMinTimePath_Callback(hObject, eventdata, handles)

saveBGC = get(handles.calcMinTimePath,'BackgroundColor');
savePTR = get(handles.MasterFigure,'Pointer');
set(handles.calcMinTimePath,'BackgroundColor','y','SelectionHighlight','off');
set(handles.MasterFigure,'Pointer','watch')
drawnow

% Update mobility map before calculating path
updateMovementCost_Callback(hObject, eventdata, handles)

ter = getappdata(handles.MasterFigure,'terrainData');
mob = getappdata(handles.MasterFigure,'mobilityData');
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');

% If necessary update Pd
if ~ess.currPdValid || isfield(ess,'adjCost')
    updatePd_Callback(hObject, eventdata, handles)
    ess = getappdata(handles.MasterFigure,'sensorData');
end 

if isempty(dsp.defendedPoint) || isempty(dsp.intruderSource)
    beep
    return
end

if dsp.intruderSource ~= dsp.defendedPoint
    
    % Check for existance of matlab_bgl library 'dijkstra_sp' function
    % If available use it since its MEX implementaiton is much faster than the simple MATLAB implementation
    % If matlab_bgl is not installed use local dijkstras implementation
    if exist('dijkstra_sp','file') == 2
        tic; [dsp.spMoveTime, dsp.spPred] = dijkstra_sp(mob.adjCost,dsp.intruderSource); toc
    else
        tic;[dsp.spMoveTime, dsp.spPred] = dijkstra_local(mob.adjCost,dsp.intruderSource);toc
    end
    
    % Plot quickest path by tracing predecessors and build forward path
    currFace = dsp.defendedPoint;
    is2dp = currFace;
    while currFace ~= dsp.intruderSource
        set(dsp.mob_hEdge(all(ismember(mob.edge2face,[currFace dsp.spPred(currFace)]),2)),'color','k','linewidth',3)
        currFace = dsp.spPred(currFace);
        is2dp = [currFace is2dp];
    end
end

% Map arcs to ajdacent faces (nodes)
face2edgeMap = spalloc(ter.nFace,ter.nFace,2*ter.nEdge);
for i = 1:ter.nEdge
    if ter.edge2face(i,1) ~= 0 && ter.edge2face(i,2) ~=0
        face2edgeMap(ter.edge2face(i,1),ter.edge2face(i,2)) = i;
        face2edgeMap(ter.edge2face(i,2),ter.edge2face(i,1)) = i;
    end
end

% Calculate time-to-reach target from each point on the quickest path
time2tgt = -(dsp.spMoveTime(is2dp) - dsp.spMoveTime(dsp.defendedPoint));

newPd = 0;
Rt = 0;
for i = 2:numel(is2dp)
    % Calculate new cumulative Pd, incremental Pd, and weighted average reaction time
    % i.e. time required to reach objective after detection
    iHead = is2dp(i);
    iTail = is2dp(i-1);
    prevPd = newPd;
    newPd = 1-(1-newPd)*(1-ess.EdgePd(face2edgeMap(iTail,iHead)))*(1-ess.FacePd(iHead));
    incPd = newPd - prevPd;
    if  incPd > 0;
        Rt = Rt + incPd*time2tgt(i); 
    end
end

set(handles.MOEminRoutePdVal,'string',num2str(newPd,'%4.3f'),'BackgroundColor','w')
set(handles.MOEminPdRouteTimeVal,'string',num2str(dsp.spMoveTime(dsp.defendedPoint),'%8.0f'),...
    'BackgroundColor',[0.5020    1.0000    0.5020])
set(handles.MOEminRouteReactVal,'string',num2str(Rt,'%8.0f'),'BackgroundColor','w')

% Check whether terrain map needs to be updated
s = get(handles.TerrainPartitionFaceDispSel,'String');
val = s{get(handles.TerrainPartitionFaceDispSel,'Value')};
switch val
    case 'Actual Move Time'
        updateTerrainMapWithActualMoveTime(dsp)
        
    case 'Relative Move Time'
        updateTerrainMapWithRelativeMoveTime(dsp)
end

setappdata(handles.MasterFigure,'terrainData',ter);
setappdata(handles.MasterFigure,'mobilityData',mob);
setappdata(handles.MasterFigure,'displayData',dsp);

set(handles.calcMinTimePath,'BackgroundColor',saveBGC);
set(handles.MasterFigure,'Pointer',savePTR)
drawnow

%% Find minimum Pd path
function MinPdPath_Callback(hObject, eventdata, handles)

% Change pushbutton color and cursor to indicate that processing is ongoing
saveBGC = get(handles.MinPdPath,'BackgroundColor');
savePTR = get(handles.MasterFigure,'Pointer');
set(handles.MinPdPath,'BackgroundColor','y','SelectionHighlight','off');
set(handles.MasterFigure,'Pointer','watch')
drawnow

% Add logical check to see if movement cost needs to be updated
updateMovementCost_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');
mob = getappdata(handles.MasterFigure,'mobilityData');
ter = getappdata(handles.MasterFigure,'terrainData');

% If necessary update Pd
if ~ess.currPdValid || isfield(ess,'adjCost')
    updatePd_Callback(hObject, eventdata, handles)
    ess = getappdata(handles.MasterFigure,'sensorData');
end  

% Initialize algorithm data structures
nVert = size(ess.adjCost,1);        % Number of vertices
Pd = inf(nVert,1);                  % Probability of detection for each vertex
pL = inf(nVert,1);                  % Path length for min Pd path for each vertex
pred = nan(nVert,1);                % Predecessor tree for min Pd path
inS = false(nVert,1);               % Logical vector showing nodes in S (fixed distance)
arcs = cell(nVert,1);               % List of arcs emenating from each vertex

% Set starting vertex
if ~isempty(dsp.intruderSource) && dsp.intruderSource ~= 0
    u = dsp.intruderSource;
else
    u = 1;
end

% Find time-to-go to the objective from each vertex 
if ~isempty(dsp.defendedPoint)
    if exist('dijkstra_sp','file') == 2
        moveTime2target = dijkstra_sp(mob.adjCost,dsp.defendedPoint);
    else
        moveTime2target = dijkstra_local(mob.adjCost,dsp.defendedPoint);
    end
else
    moveTime2target = zeros(size(mob.adjCost,1));
end

% Initialize Pd and predecessor tree
Pd(u) = 0;
pL(u) = 0;
pred(u) = 0;

S = u;
notS = 1:nVert; notS(u) = [];
inS(u) = 1;
actS = u;
arcs{u} = find(mob.adjCost(u,:));

% Find edges from each vertex
face2edgeMap = spalloc(ter.nFace,ter.nFace,2*ter.nEdge);
for i = 1:ter.nEdge
    if ter.edge2face(i,1) ~= 0 && ter.edge2face(i,2) ~=0
        face2edgeMap(ter.edge2face(i,1),ter.edge2face(i,2)) = i;
        face2edgeMap(ter.edge2face(i,2),ter.edge2face(i,1)) = i;
    end
end

% Main algorithm
while ~isempty(notS)
    elig = [];
    % Check all edges from vertices in S to vertices in notS
    for iTail = actS
        for iHead = arcs{iTail}
            
            % If the new head is already in S skip it and delete the arc
            if inS(iHead) %isnan(pred(iHead))
                arcs{iTail} = arcs{iTail}(arcs{iTail} ~= iHead);
                % If all arcs have been deleted delete the tail from the 
                % list of active nodes                
                if isempty(arcs{iTail})
                    actS(actS == iTail) = [];
                end
            else
                elig = [elig iHead];
                newPd = 1-(1-Pd(iTail))*(1-ess.EdgePd(face2edgeMap(iTail,iHead)))*(1-ess.FacePd(iHead));
                newPL = pL(iTail) + mob.edgeCost(face2edgeMap(iTail,iHead));
                
                % If new Pd is lower or new Pd is equal to current but path length is
                % less than or equal to update distance labels and predecessor
                if newPd < Pd(iHead) || (newPd == Pd(iHead) && newPL < pL(iHead))
                    Pd(iHead) = newPd;
                    pL(iHead) = newPL;
                    pred(iHead) = iTail;
                end
            end
        end
    end
    
    % Find a vertex with best Pd, break ties based on path length, then time-to-go to target
    elig = unique(elig);
    minPd = min(Pd(elig));
    elig = elig(Pd(elig)==minPd);
    if numel(elig) > 1
        minpL = min(pL(elig));
        elig = elig(pL(elig)==minpL);
        if numel(elig) > 1
            [minT2T, indx] = min(moveTime2target(elig));
            head = elig(indx);           
        else
            head = elig;
        end
    else
        head = elig;
    end
    
    % Move best vertex into S and add its eligible arcs
    S = [S head];
    notS(notS == head) = [];
    inS(head) = true;
    newArcs = find(mob.adjCost(head,:));
    newArcs = newArcs(~ismember(newArcs,S));
    if ~isempty(newArcs)
        arcs{head} = newArcs;
        actS = [actS head];
    end
end
dsp.minPd = Pd;
dsp.minPdPred = pred;

% Plot min Pd path by back tracing predecessors and build forward path 
if dsp.intruderSource ~= dsp.defendedPoint
    currFace = dsp.defendedPoint;
    is2dp = currFace;
    while currFace ~= dsp.intruderSource
        set(dsp.mob_hEdge(face2edgeMap(currFace,dsp.minPdPred(currFace))),'color','y','linewidth',3)
        currFace = dsp.minPdPred(currFace);
        is2dp = [currFace is2dp];
    end
else
    is2dp = [];
end

% Calculate expected reaction time
Rt = 0;
for i = 2:numel(is2dp)
    incPd = Pd(is2dp(i)) - Pd(is2dp(i-1));
    if  incPd > 0;
        Rt = Rt + incPd*moveTime2target(i); 
    end
end
dsp.minRxT = Rt;

dsp.minPdRouteTime = pL(dsp.defendedPoint);
set(handles.MOEminRoutePdVal,'string',num2str(dsp.minPd(dsp.defendedPoint),'%4.3f'),...
    'BackgroundColor',[0.5020    1.0000    0.5020])
set(handles.MOEminPdRouteTimeVal,'string',num2str(dsp.minPdRouteTime,'%8.0f'),'BackgroundColor','w')
set(handles.MOEminRouteReactVal,'string',num2str(Rt,'%8.0f'),'BackgroundColor','w')
setappdata(handles.MasterFigure,'displayData',dsp)

set(handles.MinPdPath,'BackgroundColor',saveBGC);
set(handles.MasterFigure,'Pointer',savePTR)
drawnow

%% Find minimum reaction time path
function MinReactionTimePath_Callback(hObject, eventdata, handles)

% Change pushbutton color and cursor to indicate that processing is ongoing
saveBGC = get(handles.MinReactionTimePath,'BackgroundColor');
savePTR = get(handles.MasterFigure,'Pointer');
set(handles.MinReactionTimePath,'BackgroundColor','y','SelectionHighlight','off');
set(handles.MasterFigure,'Pointer','watch')
drawnow

% Add logical check to see if movement cost needs to be updated
updateMovementCost_Callback(hObject, eventdata, handles);

dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');
mob = getappdata(handles.MasterFigure,'mobilityData');
ter = getappdata(handles.MasterFigure,'terrainData');

% If necessary update Pd
if ~ess.currPdValid || isfield(ess,'adjCost')
    updatePd_Callback(hObject, eventdata, handles)
    ess = getappdata(handles.MasterFigure,'sensorData');
end

% Initialize algorithm data structures
nVert = size(ess.adjCost,1);
Rt = inf(nVert,1);
Pd = zeros(nVert,1);
pL = inf(nVert,1);
pred = nan(nVert,1);
arcs = cell(nVert,1);
inS = false(nVert,1);

% Set starting vertex
if ~isempty(dsp.intruderSource) && dsp.intruderSource ~= 0
    u = dsp.intruderSource;
else
    u = 1;
end
% Find time-to-go to the objective from each vertex to 
if ~isempty(dsp.defendedPoint)
    if exist('dijkstra_sp','file') == 2
        moveTime2target = dijkstra_sp(mob.adjCost,dsp.defendedPoint);
    else
        moveTime2target = dijkstra_local(mob.adjCost,dsp.defendedPoint);
    end
else
    moveTime2target = zeros(size(mob.adjCost,1));
end

% Initialize Pd and predecessor tree
Rt(u) = 0;          % Expected Pd weighted reaction time
Pd(u) = 0;          % Probability of detection along min reaction time path
pL(u) = 0;          % Path length of min reaction time path
pred(u) = 0;        % Predecessor tree

% Initialize starting vertex
S = u;
notS = 1:nVert; notS(u) = [];
actS = u;
inS(u) = 1;

% Extract arcs to save time 
arcs{u} = find(mob.adjCost(u,:));

% Map arcs to ajdacent faces (nodes)
face2edgeMap = spalloc(ter.nFace,ter.nFace,2*ter.nEdge);
for i = 1:ter.nEdge
    if ter.edge2face(i,1) ~= 0 && ter.edge2face(i,2) ~=0
        face2edgeMap(ter.edge2face(i,1),ter.edge2face(i,2)) = i;
        face2edgeMap(ter.edge2face(i,2),ter.edge2face(i,1)) = i;
    end
end

% Main algorithm -- Modified Dijkstas shortest path
while ~isempty(notS)
    elig = [];
    for iTail = actS
        for iHead = arcs{iTail}
            
            % If the new head is already in S skip it and delete the arc
            if inS(iHead) %isnan(pred(iHead))
                arcs{iTail} = arcs{iTail}(arcs{iTail} ~= iHead);
                % If all arcs have been deleted delete the tail from the list of active nodes                
                if isempty(arcs{iTail})
                    actS(actS == iTail) = [];
                end
            else
                elig = [elig iHead];    
                
                % Calculate new cumulative Pd, incremental Pd, and weighted average reaction time
                % i.e. time required to reach objective after detection
                newPd = 1-(1-Pd(iTail))*(1-ess.EdgePd(face2edgeMap(iTail,iHead)))*(1-ess.FacePd(iHead));
                incPd = newPd - Pd(iTail);
                newPL = pL(iTail) + mob.edgeCost(face2edgeMap(iTail,iHead));
                if  incPd == 0;
                    newRt = Rt(iTail);
                else
                    newRt = Rt(iTail) + incPd*moveTime2target(iHead); 
                end
                
                if (newRt < Rt(iHead)) || (newRt == Rt(iHead) && newPd<Pd(iHead)) || ...
                        (newRt==Rt(iHead) && newPd==Pd(iHead) && newPL <= pL(iHead)) 
                    Rt(iHead) = newRt;
                    Pd(iHead) = newPd;
                    pL(iHead) = newPL;
                    pred(iHead) = iTail;
                end
            end
        end
    end
    
    % Find a vertex with lowest reaction time, break ties based on minimum probability 
    % of detection,then minimum path length, then minimum time-to-go to target
    minRt = min(Rt(elig));
    elig = elig(Rt(elig)==minRt);
    if numel(elig) > 1
        minPd = min(Pd(elig));
        elig = elig(Pd(elig)==minPd);
        if numel(elig) > 1
            minpL = min(pL(elig));
            elig = elig(pL(elig)==minpL);
            if numel(elig) > 1           
                [minT2T, indx] = min(moveTime2target(elig));
                head = elig(indx);           
            else
                head = elig;
            end
        else
            head = elig;
        end
    else
        head = elig;
    end
    
    % Move best vertex into S and add its eligible arcs
    S = [S head];
    notS(notS == head) = [];
    inS(head) = 1;
    newArcs = find(mob.adjCost(head,:));
    newArcs = newArcs(~ismember(newArcs,S));
    if ~isempty(newArcs)
        arcs{head} = newArcs;
        actS = [actS head];
    end
end

% record metrics and path
dsp.minRxTwrt = Rt;             % Intruder minimum reaction time path node data (weighted reaction time)
dsp.minRxTpd = Pd;              % Intruder minimum reactiont time path node data (probability of detection)
dsp.minRxTpred = pred;          % Intruder minimum reaction time path predecessor tree data

dsp.minRxT = Rt(dsp.defendedPoint);  % Minimum reaction time path final expected reaction time

% Backtrace minimum reaction time path
minRxTmoveCost = 0;
if dsp.intruderSource ~= dsp.defendedPoint
    currFace = dsp.defendedPoint;
    while currFace ~= dsp.intruderSource
        set(dsp.mob_hEdge(face2edgeMap(currFace,dsp.minRxTpred(currFace))),'color','g','linewidth',3)
        minRxTmoveCost = minRxTmoveCost + mob.edgeCost(face2edgeMap(currFace,dsp.minRxTpred(currFace)));
        currFace = dsp.minRxTpred(currFace);
    end
end

% Update measures of effectiveness
dsp.minPdRouteTime = minRxTmoveCost;
set(handles.MOEminRoutePdVal,'string',num2str(dsp.minRxTpd(dsp.defendedPoint),...
    '%4.3f'),'BackgroundColor','w')
set(handles.MOEminPdRouteTimeVal,'string',num2str(dsp.minPdRouteTime,'%8.0f'),...
    'BackgroundColor','w')
set(handles.MOEminRouteReactVal,'string',num2str(dsp.minRxT,'%8.0f'),...
    'BackgroundColor',[0.5020    1.0000    0.5020])
setappdata(handles.MasterFigure,'displayData',dsp)

set(handles.MinReactionTimePath,'BackgroundColor',saveBGC);
set(handles.MasterFigure,'Pointer',savePTR)
drawnow

%% Activate and show terrain editor control panel
function enableSSESterrainEdit(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ter = getappdata(handles.MasterFigure,'terrainData');
vis = get(handles.EditTerrainPanel,'visible');
bgcol = [0.92549 0.913725 0.847059];
switch vis
    case 'on'
        % "Clean up" terrain editor data to ensure clear restart
        ter.meshEditMode = 0;
        ter.currVertex = [];
        ter.currEdge = [];
        ter.currTile = [];
        ter.newVertexXYZ = [];
        ter.hNewVertex = [];
        ter.selectedEdge = [];
        ter.hSelectedEdge = [];
        ter.dragVertexSel = [];
        
        set(dsp.hVertex,'HitTest','off','ButtonDownFcn',[])
        set(dsp.hEdge,'HitTest','off')
        set(dsp.hFace,'HitTest','off')
        set(handles.selectTerrainMeshFromNpts,'BackgroundColor',bgcol)
        set(handles.MainAxes,'ButtonDownFcn',[])
        show_XY_position('off',handles)
        show_L1L2_length('off',handles)
        set(handles.MasterFigure,'WindowButtonMotionFcn',[])
        
        set(handles.selectTerrainMeshFromNpts,'BackgroundColor',bgcol)
        set(handles.selectTerrainMeshFromEdgeAndPoint,'BackgroundColor',bgcol)
        set(handles.selectAddInteriorVertex,'BackgroundColor',bgcol)
        set(handles.selectDragVertex,'BackgroundColor',bgcol)
        set(handles.selectDeleteVertex,'BackgroundColor',bgcol)
        set(handles.selectDeleteEdge,'BackgroundColor',bgcol)
        set(handles.selectPaintTerrain,'BackgroundColor',bgcol)
        set(handles.EditTerrainPanel,'visible','off');
        set(handles.MOEpanel,'visible','on')
        setappdata(handles.MasterFigure,'terrainData',ter)
        
    case 'off'
        % Deselect and hide other panels
        set(handles.EditSensorPanel,'visible','off');
        set(dsp.hEditSensorsPB,'state','off')
        set(handles.MOEpanel,'visible','off')
        
        set(handles.EditDisplayOptions,'visible','off');
        set(dsp.hDisplyOptionsPB,'state','off')
        
        % Activates display option selection panel        
        set(handles.EditTerrainPanel,'visible','on');
        set(ter.hTerSel(1:numel(ter.terrainList)),'visible','on')
        set(handles.EditTerrainPanel,'visible','on');
        set(handles.MOEpanel,'visible','off')
        
        % Disable sensor editing to prevent inadvertent modification
        ess = getappdata(handles.MasterFigure,'sensorData');
        ess.editMode = 0;
        setappdata(handles.MasterFigure,'sensorData',ess);
end

%% Activate and show sensor editor control panel
function enableSSESeditSensors(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
vis = get(handles.EditSensorPanel,'visible');
switch vis
    case 'on'
        set(handles.EditSensorPanel,'visible','off');
        set(dsp.hEditSensorsPB,'state','off')
    case 'off'
        % Deselect and hide other panels
        set(handles.EditTerrainPanel,'visible','off');
        set(dsp.hEditTerrainPB,'state','off')
        
        set(handles.EditDisplayOptions,'visible','off');
        set(dsp.hDisplyOptionsPB,'state','off')
        
        % Activates display option selection panel
        set(handles.EditSensorPanel,'visible','on');        
        set(handles.MOEpanel,'visible','on')
        updateMOEdata(handles)
end

%% Activate and show display option control panel
function setSSESdisplayOptions(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
vis = get(handles.EditDisplayOptions,'visible');
switch vis
    case 'on'
        set(handles.EditDisplayOptions,'visible','off');
        set(dsp.hDisplyOptionsPB,'selected','off')
        
    case 'off'
        % Deselect and hide other panels
        set(handles.EditTerrainPanel,'visible','off');
        set(dsp.hEditTerrainPB,'state','off')
        
        set(handles.EditSensorPanel,'visible','off');
        set(dsp.hEditSensorsPB,'state','off')

        % Activates display option selection panel
        set(handles.EditDisplayOptions,'Visible','on','HitTest','on')
        set(handles.MOEpanel,'visible','on')
        updateMOEdata(handles)
        
        % Disable sensor editing to prevent inadvertent modification
        ess = getappdata(handles.MasterFigure,'sensorData');
        ess.editMode = 0;
        setappdata(handles.MasterFigure,'sensorData',ess)
end

%% Select Terrain Partition Vertex Display
function TerrainPartitionVerticesDispSel_Callback(hObject, eventdata, handles)
% Select display option for terrain partition network edges
s = get(hObject,'String');
val = s{get(hObject,'Value')};
dsp = getappdata(handles.MasterFigure,'displayData');
switch val
    case 'Red'
        set(dsp.hVertex,'Visible','on','MarkerFaceColor','r','MarkerEdgeColor','r','markersize',4)    
    case 'None'
        set(dsp.hVertex,'Visible','off')
end
setappdata(handles.MasterFigure,'displayData',dsp);

%% Select Terrain Partition Edge Display
function TerrainPartitionEdgeDispSel_Callback(hObject, eventdata, handles)
s = get(hObject,'String');
val = s{get(hObject,'Value')};
dsp = getappdata(handles.MasterFigure,'displayData');
switch val
    case 'Red'
        set(dsp.hEdge,'Visible','on','color','r','linewidth',1)    
    case 'None'
        set(dsp.hEdge,'Visible','off')
end
setappdata(handles.MasterFigure,'displayData',dsp);

%% Select Terrain Partition Face Display
function TerrainPartitionFaceDispSel_Callback(hObject, eventdata, handles)
s = get(handles.TerrainPartitionFaceDispSel,'String');
val = s{get(handles.TerrainPartitionFaceDispSel,'Value')};
dsp = getappdata(handles.MasterFigure,'displayData');
switch val
    case 'Terrain Type'
        ter = getappdata(handles.MasterFigure,'terrainData'); 
        for iFace = 1:numel(dsp.hFace)
            set(dsp.hFace(iFace),'Visible','on','FaceColor',ter.face(iFace).RGB)
        end
    case 'Actual Move Time'
        updateTerrainMapWithActualMoveTime(dsp)
    case 'Relative Move Time'
        updateTerrainMapWithRelativeMoveTime(dsp)
    case 'Face Pd'
        updateTerrainMapWithFacePd(dsp)
    case 'Path Pd'
        updateTerrainMapWithPathPd(dsp)
end

%% Select Mobility Network Vertex Display
function MobNetVertexDispSel_Callback(hObject, eventdata, handles)
s = get(hObject,'String');
val = s{get(hObject,'Value')};
dsp = getappdata(handles.MasterFigure,'displayData');
switch val
    case 'Blue'
        set(dsp.mob_hVertex,'Visible','on','MarkerFaceColor','b','MarkerEdgeColor','b',...
            'marker','o','markersize',3,'visible','on')    
    case 'None'
        set(dsp.mob_hVertex,'Visible','off')
end
setappdata(handles.MasterFigure,'displayData',dsp);

%% Select Mobility Network Edge Display
function MobNetEdgeDispSel_Callback(hObject, eventdata, handles)
s = get(handles.MobNetEdgeDispSel,'String');
val = s{get(handles.MobNetEdgeDispSel,'Value')};
dsp = getappdata(handles.MasterFigure,'displayData');
mob = getappdata(handles.MasterFigure,'mobilityData');
sf = strfind(val,'-');
if ~isempty(sf)
    opt = val(sf+1:end);
    val = val(1:sf-1);
else
    opt = [];
end
    
switch val
    case 'Blue'
        set(dsp.mob_hEdge(mob.edgeCost ~=0),'Visible','on','color','b','linewidth',1) 
    
    case 'Time'        
        mec = mob.edgeCost;
        [t,index] = sort(mec);
        zap = (t==0);
        t = t(~zap); 
        index = index(~zap);
        
        % Time-actual
        switch opt
            case 'actual'
                timeVect = [10 20 30 40 50 100 200 300 400 500 1000];
                for iCost = 1:(numel(timeVect)-1)
                    ltCutoff = (t <= timeVect(iCost));
                    set(dsp.mob_hEdge(index(ltCutoff)),'Visible','on',...
                        'color',dsp.g2rRGB(iCost,1:3),'linewidth',1) 
                    t = t(~ltCutoff);
                    index = index(~ltCutoff);            
                end
                set(dsp.mob_hEdge(index),'Visible','on',...
                    'color',dsp.g2rRGB(end,1:3),'linewidth',1)

            % Time-relative
            case 'relative'
                inc = floor(numel(t)/12);
                done = 0;
                for iCost = 1:11
                    top = floor(inc*iCost);
                    set(dsp.mob_hEdge(index(done+1:top)),'Visible','on',...
                        'color',dsp.g2rRGB(iCost,1:3),'linewidth',1)
                    done = top;            
                end
                set(dsp.mob_hEdge(index(done+1:end)),'Visible','on',...
                    'color',dsp.g2rRGB(end,1:3),'linewidth',1)
        end
        
    case 'None'
        set(dsp.mob_hEdge,'Visible','off')
end
setappdata(handles.MasterFigure,'displayData',dsp);

%% Select Texturemap image to display
function TextureMapDispSel_Callback(hObject, eventdata, handles)
s = get(hObject,'string');
val = s{get(hObject,'value')};
indx = strmatch(val,s,'exact');
if ~isempty(indx)
    dsp = getappdata(handles.MasterFigure,'displayData');
    site = getappdata(handles.MasterFigure,'siteData');
    set(dsp.curTextureMap,'cdata',flipud(site.textureMaps(indx).textureImage));
    colormap(site.textureMaps(indx).textureCmap)
end

%% Select sensor type
function SensorEditingSelectSensor_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
s = get(hObject,'string');
val = s{get(hObject,'value')};
modelIndex = strmatch(val,{ess.sensorList.model},'exact');
if ess.editMode == 1
    if ~isempty(modelIndex)
       ess.currentSensorParam = ess.sensorList(modelIndex);
       ess.currentSensorParam.totalCost = ess.currentSensorParam.unitCost;
       ess.currentSensor = 0;
       updateSensorEditData(ess,handles)    
    else
        beep
    end
else
    if ess.currentSensor ~= 0
       oldval = strmatch(ess.currentSensorParam.model,s);
       if ~isempty(oldval)
           set(hObject,'value',oldval)
       end
    end
    beep 
end
setappdata(handles.MasterFigure,'sensorData',ess);

%% Edit sensor X(1) location
function SensorEditingSensorLocationEditX1_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')
    val = str2double(get(handles.SensorEditingSensorLocationEditX1,'string'));
    ess.currentSensorParam.Xlocation(1) = val;
    
    site = getappdata(handles.MasterFigure,'siteData');
    gndHt = interp2(site.xMesh,site.yMesh,site.zElev,...
        ess.currentSensorParam.Xlocation(1),ess.currentSensorParam.Ylocation(1));
    mtHt = ess.currentSensorParam.mountHeight;
    ess.currentSensorParam.Zlocation(1) = gndHt + mtHt;
    set(handles.SensorEditingSensorLocationEditZ1,'string',num2str(gndHt+mtHt,'%4.2f'))
    
    cs = abs(ess.currentSensor);
    ess.volumeSensors{cs}.Xlocation(1) = val;
    ess.volumeSensors{cs}.Zlocation(1) = ess.currentSensorParam.Zlocation(1);
    delete(dsp.hVolSensorCoverage(cs))
    dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);
    set(dsp.hVolSensorPos(cs),'xdata',val,'zdata',ess.currentSensorParam.Zlocation(1))
else
    if ess.currentSensor ~= 0
        set(handles.SensorEditingSensorLocationEditX1,'string',ess.currentSensorParam.Xlocation(1))
    else
        set(handles.SensorEditingSensorLocationEditX1,'string','')
    end
    beep
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit sensor X(2) position (dummy)
function SensorEditingSensorLocationEditX2_Callback(hObject, eventdata, handles)
% X2 is not editable -- X2 is only displayed when line sensor is selected 
% line sensor coordinates are determined by the position of the associated terrain edge
beep
if numel(ess.currentSensorParam.Xlocation) > 1
    set(handles.SensorEditingSensorLocationEditX2,'string',ess.currentSensorParam.Xlocation(2))
else
     set(handles.SensorEditingSensorLocationEditX2,'string','')
end

%% Edit sensor Y(1) position
function SensorEditingSensorLocationEditY1_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')
    val = str2double(get(handles.SensorEditingSensorLocationEditY1,'string'));
    ess.currentSensorParam.Ylocation(1) = val;
    
    site = getappdata(handles.MasterFigure,'siteData');
    gndHt = interp2(site.xMesh,site.yMesh,site.zElev,...
        ess.currentSensorParam.Xlocation(1),ess.currentSensorParam.Ylocation(1));
    mtHt = ess.currentSensorParam.mountHeight;
    ess.currentSensorParam.Zlocation(1) = gndHt + mtHt;
    set(handles.SensorEditingSensorLocationEditZ1,'string',num2str(gndHt+mtHt,'%4.2f'))
    
    cs = abs(ess.currentSensor);
    ess.volumeSensors{cs}.Ylocation(1) = val;
    ess.volumeSensors{cs}.Zlocation(1) = ess.currentSensorParam.Zlocation(1);
    delete(dsp.hVolSensorCoverage(cs))
    dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);
    set(dsp.hVolSensorPos(cs),'ydata',val,'zdata',ess.currentSensorParam.Zlocation(1))    
else
    if ess.currentSensor ~= 0
        set(handles.SensorEditingSensorLocationEditY1,'string',ess.currentSensorParam.Ylocation(1))
    else
        set(handles.SensorEditingSensorLocationEditY1,'string','')
    end
    beep
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit sensor Y(2) position (dummy)
function SensorEditingSensorLocationEditY2_Callback(hObject, eventdata, handles)
% Y2 is not editable -- Y2 is only displayed when line sensor is selected 
% line sensor coordinates are determined by the position of the associated terrain edge
beep
if numel(ess.currentSensorParam.Ylocation) > 1
    set(handles.SensorEditingSensorLocationEditY2,'string',ess.currentSensorParam.Ylocation(2))
else
     set(handles.SensorEditingSensorLocationEditY2,'string','')
end

%% Edit sensor Z(1) position
function SensorEditingSensorLocationEditZ1_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing if there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')
    val = str2double(get(handles.SensorEditingSensorLocationEditZ1,'string'));
    ess.currentSensorParam.Zlocation(1) = val;
    cs = abs(ess.currentSensor);
    ess.volumeSensors{cs}.Zlocation(1) = val;
    delete(dsp.hVolSensorCoverage(cs))
    dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);
    set(dsp.hVolSensorPos(cs),'ydata',val)
    site = getappdata(handles.MasterFigure,'siteData');
    gndHt = interp2(site.xMesh,site.yMesh,site.zElev,...
        ess.currentSensorParam.Xlocation,ess.currentSensorParam.Ylocation);
    mountHt = val - gndHt;
    ess.currentSensorParam.mountHeight = mountHt;
    ess.volumeSensors{cs}.mountHeight = mountHt;
    set(handles.SensorEditingMountingHeightEdit,'string',num2str(mountHt,'%4.2f'))
    
else
    if ess.currentSensor ~= 0
        set(handles.SensorEditingSensorLocationEditZ1,'string',ess.currentSensorParam.Zlocation(1))
    else
        set(handles.SensorEditingSensorLocationEditZ1,'string','')
    end
    beep
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit sensor Z(2) position (dummy)
function SensorEditingSensorLocationEditZ2_Callback(hObject, eventdata, handles)
% Z2 is not editable -- Z2 is only displayed when line sensor is selected 
% line sensor coordinates are determined by the position of the associated terrain edge
beep
set(handles.SensorEditingSensorLocationEditZ2,'string',ess.currentSensorParam.Zlocation(2))
if numel(ess.currentSensorParam.Zlocation) > 1
    set(handles.SensorEditingSensorLocationEditZ2,'string',ess.currentSensorParam.Zlocation(2))
else
    set(handles.SensorEditingSensorLocationEditZ2,'string','')
end

%% Change sensor editing mode to "Add"
function SensorEditingAddSensorPB_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');
bgcol = [0.92549 0.913725 0.847059];

if ess.editMode == 1
    % Current sensor editing mode is add -- disable add sensor
    ess.editMode = 0;    
    set(handles.MainAxes,'HitTest','off')
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    setSensorEditingGenericTitles(handles,'off')
    setSensorEditingVolumeTitles(handles,'off')
    setSensorEditingGenericData(handles,'off')
    set(handles.SensorEditingAddSensorPB,'BackgroundColor',bgcol);
else
    % Current sensor editing mode is not add -- set add mode
    ess.editMode = 1;
    % Show labels
    setSensorEditingGenericTitles(handles,'on')    
    setSensorEditingGenericData(handles,'on')
    
    % Set callbacks based on current sensor type
    set(dsp.hLineSensors(dsp.hLineSensors ~= 0),'HitTest','off')
    set(dsp.hVolSensorPos,'HitTest','off')
    set(dsp.hVolSensorCoverage,'HitTest','off')
    val = lower(get(handles.SensorEditingSensorClassDsipText,'String'));
    switch val
        case {'line',[]}
            set(dsp.hEdge,'HitTest','on','ButtonDownFcn',...
                'ssesPrototype(''addLineSensor'',gcbo,[],guidata(gcbo))')
            set(handles.MainAxes,'HitTest','off')
        case {'volume','area'}
            set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
            set(handles.MainAxes,'HitTest','on','ButtonDownFcn',...
                'ssesPrototype(''addVolumeSensor'',gcbo,[],guidata(gcbo))')
            setSensorEditingVolumeTitles(handles,'on')
    end    
    % Toggle pushbuttons
    set(handles.SensorEditingAddSensorPB,'BackgroundColor','w');
    set(handles.SensorEditingDeletePB,'BackgroundColor',bgcol);
    set(handles.SensorEditingEditPB,'BackgroundColor',bgcol);
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Change sensor editing mode to "Delete"
function SensorEditingDeletePB_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');
bgcol = [0.92549 0.913725 0.847059];

if ess.editMode == 2
    % Current sensor editing mode is delete -- disable delete sensor
    ess.editMode = 0;
    set(hObject,'BackgroundColor',bgcol);
    set(handles.MainAxes,'HitTest','off')
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hLineSensors(dsp.hLineSensors ~=0),'HitTest','off')
    set(dsp.hVolSensorPos,'HitTest','off')
    set(dsp.hVolSensorCoverage,'HitTest','off')
else
    % Current sensor editing mode is not deletion -- set delete mode
    ess.editMode = 2;
    set(handles.SensorEditingAddSensorPB,'BackgroundColor',bgcol);
    set(handles.SensorEditingDeletePB,'BackgroundColor','w');
    set(handles.SensorEditingEditPB,'BackgroundColor',bgcol);
    
    set(handles.MainAxes,'HitTest','off')
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hLineSensors(dsp.hLineSensors ~= 0),'HitTest','on',...
        'ButtonDownFcn','ssesPrototype(''deleteSensor'',gcbo,[],guidata(gcbo))') 
    set(dsp.hVolSensorPos,'HitTest','on',...
        'ButtonDownFcn','ssesPrototype(''deleteSensor'',gcbo,[],guidata(gcbo))')
    setSensorEditingGenericTitles(handles,'off')
    setSensorEditingGenericData(handles,'off')
    setSensorEditingVolumeTitles(handles,'off') 
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Change sensor editing mode to "Edit"
function SensorEditingEditPB_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');
bgcol = [0.92549 0.913725 0.847059];

if ess.editMode == 3
    % Current sensor editing mode is edit -- disable edit sensor
    ess.editMode = 0;
    set(handles.SensorEditingEditPB,'BackgroundColor',bgcol);
    set(handles.MainAxes,'HitTest','off')
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hLineSensors(dsp.hLineSensors ~=0),'HitTest','off')
    set(dsp.hVolSensorPos(dsp.hVolSensorPos ~= 0),'HitTest','off')
    set(dsp.hVolSensorCoverage(dsp.hVolSensorCoverage ~=0),'HitTest','off')
else
    % Current sensor editing mode is not edit mode -- set edit mode    
    set(handles.SensorEditingAddSensorPB,'BackgroundColor',bgcol);
    set(handles.SensorEditingDeletePB,'BackgroundColor',bgcol);
    set(handles.SensorEditingEditPB,'BackgroundColor','w');
    
    set(handles.MainAxes,'HitTest','off')
    set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
    set(dsp.hLineSensors(dsp.hLineSensors ~= 0),'HitTest','on',...
        'ButtonDownFcn','ssesPrototype(''editSensor'',gcbo,[],guidata(gcbo))') 
    set(dsp.hVolSensorPos,'HitTest','on',...
        'ButtonDownFcn','ssesPrototype(''editSensor'',gcbo,[],guidata(gcbo))')
    % If current editing mode was delete then show generic labels now
    % Otherwise maintain current editing display
    if ess.editMode == 2
        setSensorEditingGenericTitles(handles,'on')
        setSensorEditingGenericData(handles,'off')
        setSensorEditingVolumeTitles(handles,'off')
    end
    ess.editMode = 3;
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit sensor azimuth left (counter clockwise) limit
function SensorEditingSensorAzLedit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing if there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')
    ess.currPdValid = 0;
    val = mod(str2double(get(handles.SensorEditingSensorAzLedit,'string')),360);
    set(handles.SensorEditingSensorAzLedit,'string',num2str(val,'%4.2f'))
    
    ess.currentSensorParam.AZlimit(1) = val;
    if ess.currentSensorParam.AZmax < 360
        if ess.currentSensorParam.AZlimit(2) > ess.currentSensorParam.AZlimit(1)
            if ess.currentSensorParam.AZlimit(2) - ess.currentSensorParam.AZlimit(1) > ...
                    ess.currentSensorParam.AZmax
                rAzLim = ess.currentSensorParam.AZlimit(1) + ess.currentSensorParam.AZmax;
                ess.currentSensorParam.AZlimit(2) = rAzLim;
                set(handles.SensorEditingSensorAzRedit,'string',num2str(mod(rAzLim,360),'%4.2f'))
            end
        else
            if (360-ess.currentSensorParam.AZlimit(1))+ess.currentSensorParam.AZlimit(2) > ...
                    ess.currentSensorParam.AZmax
                rAzLim = ess.currentSensorParam.AZmax - (360-ess.currentSensorParam.AZlimit(1));
                ess.currentSensorParam.AZlimit(2) = mod(rAzLim,360);
                set(handles.SensorEditingSensorAzRedit,'string',num2str(mod(rAzLim,360),'%4.2f'))                
            end
        end
        
    end
    
    cs = abs(ess.currentSensor);
    ess.volumeSensors{cs}.AZlimit = ess.currentSensorParam.AZlimit;
    
    if dsp.hVolSensorCoverage(cs) ~=0 && ishandle(dsp.hVolSensorCoverage(cs))
        delete(dsp.hVolSensorCoverage(cs))
    end
    dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);
    
else
    if ess.currentSensor ~= 0
        set(handles.SensorEditingSensorAzLedit,'string',...
            num2str(ess.currentSensorParam.AZlimit(1),'%4.2f'))
    else
        set(handles.SensorEditingSensorAzLedit,'string','')
    end
    beep
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit right (clockwise) azimuth limit
function SensorEditingSensorAzRedit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing if there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')
    ess.currPdValid = 0;
    val = mod(str2double(get(handles.SensorEditingSensorAzRedit,'string')),360);
    set(handles.SensorEditingSensorAzRedit,'string',num2str(val,'%4.2f'))
    
    ess.currentSensorParam.AZlimit(2) = mod(val,360);
    if ess.currentSensorParam.AZmax < 360
        if ess.currentSensorParam.AZlimit(2) > ess.currentSensorParam.AZlimit(1)
            if ess.currentSensorParam.AZlimit(2) - ess.currentSensorParam.AZlimit(1) > ...
                    ess.currentSensorParam.AZmax
                lAzLim = ess.currentSensorParam.AZlimit(2) - ess.currentSensorParam.AZmax;
                ess.currentSensorParam.AZlimit(1) = lAzLim;
                set(handles.SensorEditingSensorAzLedit,'string',num2str(lAzLim,'%4.2f'))
            end
        else
            if (360-ess.currentSensorParam.AZlimit(1))+ess.currentSensorParam.AZlimit(2) > ...
                    ess.currentSensorParam.AZmax
                lAzLim = 360 - (ess.currentSensorParam.AZmax - ess.currentSensorParam.AZlimit(2));
                ess.currentSensorParam.AZlimit(1) = lAzLim;
                set(handles.SensorEditingSensorAzLedit,'string',num2str(mod(lAzLim,360),'%4.2f'))                
            end
        end
    end
    
    cs = abs(ess.currentSensor);
    ess.volumeSensors{cs}.AZlimit = ess.currentSensorParam.AZlimit;
    delete(dsp.hVolSensorCoverage(cs))
    dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);
    
else
    if ess.currentSensor ~= 0
        set(handles.SensorEditingSensorAzLedit,'string',...
            num2str(ess.currentSensorParam.AZlimit(1),'%4.2f'))
    else
        set(handles.SensorEditingSensorAzLedit,'string','')
    end
    beep
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit volume sensor lower elevation limit
function SensorEditingSensorElLedit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing if there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')    
    val = str2double(get(handles.SensorEditingSensorElLedit,'string'));
    % Only allow elevation values in the range of -90:90
    if -90 <= val && val <= 90
        ess.currPdValid = 0;
        ess.currentSensorParam.ELlimit(1) = val;
        set(handles.SensorEditingSensorElLedit,'string',num2str(val,'%3.1f'))

        % Check max elevation and adjust upper limit if needed
        if ess.currentSensorParam.ELlimit(2) > ess.currentSensorParam.ELlimit(1) ...
                + ess.currentSensorParam.ELmax 
            ul = ess.currentSensorParam.ELlimit(1) + ess.currentSensorParam.ELmax;
            ess.currentSensorParam.ELlimit(2) = ul;
            set(handles.SensorEditingSensorElUedit,'string',num2str(ul,'%3.1f'))
        end  
        cs = abs(ess.currentSensor);
        ess.volumeSensors{cs}.ELlimit = ess.currentSensorParam.ELlimit;
        delete(dsp.hVolSensorCoverage(cs))
        dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);    
    end
else
    if ess.currentSensor ~= 0 && numel(ess.currentSensorParam.ELlimit) > 1
        set(handles.SensorEditingSensorElLedit,'string',...
            num2str(ess.currentSensorParam.ELlimit(1),'%3.1f'))
    else
        set(handles.SensorEditingSensorElLedit,'string','')
    end
    beep
end
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit volume sensor upper elevation limit
function SensorEditingSensorElUedit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing if there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')    
    val = str2double(get(handles.SensorEditingSensorElUedit,'string'));
    % Only allow elevation values in the range of -90:90
    if -90 <= val && val <= 90
        ess.currPdValid = 0;
        ess.currentSensorParam.ELlimit(2) = val;
        set(handles.SensorEditingSensorElUedit,'string',num2str(val,'%3.1f'))

        % Check max elevation and adjust lower limit if needed
        if ess.currentSensorParam.ELlimit(1) < ess.currentSensorParam.ELlimit(2) ...
                - ess.currentSensorParam.ELmax 
            ll = ess.currentSensorParam.ELlimit(2) - ess.currentSensorParam.ELmax;
            ess.currentSensorParam.ELlimit(1) = ll;
            set(handles.SensorEditingSensorElLedit,'string',num2str(ll,'%3.1f'))
        end  
        cs = abs(ess.currentSensor);
        ess.volumeSensors{cs}.ELlimit = ess.currentSensorParam.ELlimit;
        delete(dsp.hVolSensorCoverage(cs))
        dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);    
    end
else
    if ess.currentSensor ~= 0 && numel(ess.currentSensorParam.ELlimit) > 1
        set(handles.SensorEditingSensorElUedit,'string',...
            num2str(ess.currentSensorParam.ELlimit(2),'%3.1f'))
    else
        set(handles.SensorEditingSensorElUedit,'string','')
    end
    beep
end
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Executes on selection change in SensorEditingMountingHeightEdit.
function SensorEditingMountingHeightEdit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Allow editing if there is a current sensor, and the sensor class is 'volume'
if ess.currentSensor ~= 0 && strcmpi(ess.currentSensorParam.class,'volume')
    val = str2double(get(handles.SensorEditingMountingHeightEdit,'string'));
    cs = abs(ess.currentSensor);
    ess.currentSensorParam.mountHeight = val;
    ess.volumeSensors{cs}.mountHeight = val;
    site = getappdata(handles.MasterFigure,'siteData');
    gndHt = interp2(site.xMesh,site.yMesh,site.zElev,...
        ess.currentSensorParam.Xlocation,ess.currentSensorParam.Ylocation);
    ess.currentSensorParam.Zlocation = val+gndHt;
    ess.volumeSensors{cs}.Zlocation = val+gndHt;
    delete(dsp.hVolSensorCoverage(cs))
    dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);
    set(dsp.hVolSensorPos(cs),'zdata',val+gndHt)
    set(handles.SensorEditingSensorLocationEditZ1,'string',num2str(val+gndHt,'%4.2f'))
    ess.currPdValid = 0;
else
    if ess.currentSensor ~= 0
        set(handles.SensorEditingMountingHeightEdit,'string',ess.currentSensorParam.mountHeight)
    else
        set(handles.SensorEditingMountingHeightEdit,'string','')
    end
    beep
end
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Select sensor Pd method -- not currently implemented ***
function SensorEditingPdMethodPopup_Callback(hObject, eventdata, handles)
set(hObject,'value',3)
beep

%% Edit sensor label
function SensorEditingSensorLabelEdit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

if ess.currentSensor ~= 0 
    s = get(handles.SensorEditingSensorLabelEdit,'string');
    ess.currentSensorParam.sensorLabel = s;
    if ess.currentSensor >0
        ess.lineSensors{ess.currentSensor}.sensorLabel = s;
    else
        ess.volumeSensors{abs(ess.currentSensor)}.sensorLabel = s;
    end
end
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Edit sensor unit cost
function SensorEditingUnitCostEdit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');

if ess.currentSensor == 0
    beep
    return
else
    uc = str2double(get(handles.SensorEditingUnitCostEdit,'string'));
    
    if strcmpi(ess.currentSensorParam.class,'volume')
        % Volume sensor -- element cost = unit cost
        ec = uc;
        tc = ec + ess.currentSensorParam.supportCost;
        
        ess.currentSensorParam.unitCost = uc;
        ess.volumeSensors{abs(ess.currentSensor)}.unitCost = uc;
        
        ess.currentSensorParam.elementCost = ec;
        ess.volumeSensors{abs(ess.currentSensor)}.elementCost = ec;
        
        ess.currentSensorParam.totalCost = tc;
        ess.volumeSensors{abs(ess.currentSensor)}.totalCost = tc;    
    else
        % Line sensor element cost = unit cost * length of sensor run
        dX = diff(ess.currentSensorParam.Xlocation);
        dY = diff(ess.currentSensorParam.Ylocation);
        dZ = diff(ess.currentSensorParam.Zlocation);
        lineLen = norm([dX dY dZ]);
        ec = uc*lineLen;
        tc = ec + ess.currentSensorParam.supportCost;
        
        ess.currentSensorParam.unitCost = uc;
        ess.lineSensors{ess.currentSensor}.unitCost = uc;
        
        ess.currentSensorParam.elementCost = ec;
        ess.lineSensors{ess.currentSensor}.elementCost = ec;
        
        ess.currentSensorParam.totalCost = tc;
        ess.lineSensors{ess.currentSensor}.totalCost = tc;  
    end
    updateSensorEditData(ess,handles)
    setappdata(handles.MasterFigure,'sensorData',ess);
    updateESScumCost(handles)
end

%% Edit sensor element cost
function SensorEditingElementCostEdit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');

if ess.currentSensor == 0
    beep
    return
else
    ec = str2double(get(handles.SensorEditingElementCostEdit,'string'));
    tc = ec + ess.currentSensorParam.supportCost;
    
    ess.currentSensorParam.elementCost = ec;
    ess.currentSensorParam.totalCost = tc;
    
    if strcmpi(ess.currentSensorParam.class,'volume')
        ess.volumeSensors{abs(ess.currentSensor)}.elementCost = ec;
        ess.volumeSensors{abs(ess.currentSensor)}.totalCost = tc;    
    else
        ess.lineSensors{ess.currentSensor}.elementCost = ec;
        ess.lineSensors{ess.currentSensor}.totalCost = tc;  
    end
    updateSensorEditData(ess,handles)
    setappdata(handles.MasterFigure,'sensorData',ess);
    updateESScumCost(handles)
end

%% Edit sensor support cost
function SensorEditingSupportCostEdit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');

if ess.currentSensor == 0
    beep
    return
else
    sc = str2double(get(handles.SensorEditingSupportCostEdit,'string'));
    tc = sc + ess.currentSensorParam.elementCost;
    
    ess.currentSensorParam.supportCost = sc;
    ess.currentSensorParam.totalCost = tc;
    
    if strcmpi(ess.currentSensorParam.class,'volume')
        ess.volumeSensors{abs(ess.currentSensor)}.supportCost = sc;
        ess.volumeSensors{abs(ess.currentSensor)}.totalCost = tc;    
    else
        ess.lineSensors{ess.currentSensor}.supportCost = sc;
        ess.lineSensors{ess.currentSensor}.totalCost = tc;  
    end
    updateSensorEditData(ess,handles)
    setappdata(handles.MasterFigure,'sensorData',ess);
    updateESScumCost(handles)
end

%% Edit sensor total cost
function SensorEditingTotalCostEdit_Callback(hObject, eventdata, handles)
ess = getappdata(handles.MasterFigure,'sensorData');

if ess.currentSensor == 0
    beep
    return
else
    tc = str2double(get(handles.SensorEditingTotalCostEdit,'string'));
    ess.currentSensorParam.totalCost = tc;
    
    if strcmpi(ess.currentSensorParam.class,'volume')
        ess.volumeSensors{abs(ess.currentSensor)}.totalCost = tc;    
    else
        ess.lineSensors{ess.currentSensor}.totalCost = tc;  
    end
    updateSensorEditData(ess,handles)
    setappdata(handles.MasterFigure,'sensorData',ess);
    updateESScumCost(handles)
end

%% Edit Obstacles ***
function ObstacleEditingPopup_Callback(hObject, eventdata, handles)
% No action required
return

%% addLineSensor
function addLineSensor(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');

% Determine whether / which edge is selected
edgeIndex = find(hObject == dsp.hEdge);
if ~isempty(edgeIndex)
    % Get the current sensor type
    if  numel(ess.lineSensors) >= edgeIndex && ~isempty(ess.lineSensors{edgeIndex})
       confirm = questdlg({'Only one sensor is allowed per edge.       .', ...
       'Replace the current sensor?                '},'Replace sensor?','no');
        if ~strcmpi(confirm,'yes')
            return
        else
            delete(dsp.hLineSensors(edgeIndex))
            dsp.hLineSensors(edgeIndex) = 0;
            ess.lineSensors{edgeIndex} = [];
        end
    end
    currSensorIndx = get(handles.SensorEditingSelectSensor,'value');
    thisSensor = ess.sensorList(currSensorIndx);
    % Assign sensor XYZ coordinates based on terrain edge XYZ data and sensor mounting height
    if isempty(thisSensor.mountHeight)
        thisSensor.mountHeight = 0;
    end
    thisSensor.Xlocation = get(dsp.hEdge(edgeIndex),'xdata');
    thisSensor.Ylocation = get(dsp.hEdge(edgeIndex),'ydata');
    thisSensor.Zlocation = get(dsp.hEdge(edgeIndex)+thisSensor.mountHeight,'zdata');
    
    ess.sensorCounter = ess.sensorCounter + 1;
    thisSensor.sensorLabel = [ess.currentSensorParam.model int2str(ess.sensorCounter)];
    
    % Calculate edge length and sensor cost
    lineLen = norm([diff(thisSensor.Xlocation) diff(thisSensor.Ylocation) diff(thisSensor.Zlocation)]);
    thisSensor.elementCost = thisSensor.unitCost*lineLen;
    thisSensor.totalCost = thisSensor.elementCost + thisSensor.supportCost;
    
    ess.currentSensorParam = thisSensor;
    ess.lineSensors{edgeIndex} = thisSensor;
    ess.currentSensor = edgeIndex;
    
    dsp.hLineSensors(edgeIndex) = plot3(thisSensor.Xlocation,thisSensor.Ylocation,...
        thisSensor.Zlocation,'LineStyle',thisSensor.edgeStyle,...
        'color',thisSensor.edgeRGB,'linewidth',thisSensor.edgeWidth,'HitTest','off');
    ess.currPdValid = 0;
    updateSensorEditData(ess,handles)
    setappdata(handles.MasterFigure,'sensorData',ess);
    setappdata(handles.MasterFigure,'displayData',dsp);
    updateESScumCost(handles)   
else
    beep
end

%% addVolumeSensor
function addVolumeSensor(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');

if hObject == handles.MainAxes
    % Get current point
    site = getappdata(handles.MasterFigure,'siteData');
    cp = get(gca,'CurrentPoint');
    cp(:,3) = interp2(site.xMesh,site.yMesh,site.zElev,cp(2,1),cp(2,2));
    
    if isempty(ess.currentSensorParam.mountHeight)
        ess.currentSensorParam.mountHeight = 0;
    end
    ess.currentSensorParam.Xlocation = cp(1,1);
    ess.currentSensorParam.Ylocation = cp(1,2);
    ess.currentSensorParam.Zlocation = cp(1,3) + ess.currentSensorParam.mountHeight;
    ess.currentSensorParam.AZlimit = [0 ess.currentSensorParam.AZmax];
    ess.currentSensorParam.ELlimit = [-ess.currentSensorParam.ELmax/2 ...
        ess.currentSensorParam.ELmax/2];
    
    ess.sensorCounter = ess.sensorCounter + 1;    
    ess.currentSensorParam.sensorLabel = [ess.currentSensorParam.model int2str(ess.sensorCounter)];
    
    % Calculate edge length and sensor cost
    ess.currentSensorParam.elementCost = ess.currentSensorParam.unitCost;
    ess.currentSensorParam.totalCost = ess.currentSensorParam.elementCost +...
        ess.currentSensorParam.supportCost;    
    
    nVolSens = numel(ess.volumeSensors)+1;
    ess.volumeSensors{nVolSens} = ess.currentSensorParam;    
    ess.currentSensor = -nVolSens;
    
    dsp.hVolSensorCoverage(nVolSens) = makeVolSensorPatch(ess,handles);
    dsp.hVolSensorPos(nVolSens) = plot3(ess.currentSensorParam.Xlocation,...
        ess.currentSensorParam.Ylocation,ess.currentSensorParam.Zlocation,'k-^',...
        'marker','^','markersize',6','markerFaceColor','k','markerEdgeColor','k',...
        'linewidth',2,'lineStyle',ess.currentSensorParam.edgeStyle,'HitTest','off');
    
    ess.currPdValid = 0;
    updateSensorEditData(ess,handles)
    setappdata(handles.MasterFigure,'sensorData',ess);
    setappdata(handles.MasterFigure,'displayData',dsp);
    updateESScumCost(handles)        
else
    beep
end

%% deleteSensor
function deleteSensor(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');

% For left click confirm sensor deletion -- do not confirm for right click
rightClick = strcmpi(get(handles.MasterFigure,'SelectionType'),'alt');
if ~rightClick
    button = questdlg('Delete sensor?    .','Load site imagery','Yes','No','No');
    if ~strcmpi(button,'Yes')
        return
    end
end

lsIndex = find(hObject == dsp.hLineSensors);
if ~isempty(lsIndex)
    % Delete sensor, sensor symbol, and credit sensor cost
    delete(dsp.hLineSensors(lsIndex))
    dsp.hLineSensors(lsIndex) = 0;
    ess.cumCost = ess.cumCost - (ess.lineSensors{lsIndex}.elementCost + ...
        ess.lineSensors{lsIndex}.supportCost);
    set(handles.MOEessCostVal,'string',int2str(ess.cumCost))
    ess.lineSensors{lsIndex} = [];
else
    vsIndex = find(hObject == dsp.hVolSensorPos);
    if ~isempty(vsIndex)
        delete(dsp.hVolSensorPos(vsIndex))
        dsp.hVolSensorPos(vsIndex) = 0;
        delete(dsp.hVolSensorCoverage(vsIndex))
        dsp.hVolSensorCoverage(vsIndex) = 0;
        ess.cumCost = ess.cumCost - (ess.volumeSensors{vsIndex}.elementCost + ...
        ess.volumeSensors{vsIndex}.supportCost);
        set(handles.MOEessCostVal,'string',int2str(ess.cumCost))
        ess.volumeSensors{vsIndex} = [];
    else
        beep
    end
end
ess.currentSensor = 0;
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);
updateESScumCost(handles)

%% editSensor
function editSensor(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');

lsIndex = find(hObject == dsp.hLineSensors);
if ~isempty(lsIndex)
    % Delete sensor, sensor symbol, and credit sensor cost
    ess.currentSensor = lsIndex;
    ess.currentSensorParam = ess.lineSensors{lsIndex};
    updateSensorEditData(ess,handles)
else
    vsIndex = find(hObject == dsp.hVolSensorPos);
    if ~isempty(vsIndex)
        ess.currentSensor = -vsIndex;
        ess.currentSensorParam = ess.volumeSensors{vsIndex};
        updateSensorEditData(ess,handles)
        set(handles.MasterFigure,'WindowButtonMotionFcn',...
            'ssesPrototype(''dragSensor'',gcbo,[],guidata(gcbo))',...
            'WindowButtonUpFcn',...
            'ssesPrototype(''releaseDraggedSensor'',gcbo,[],guidata(gcbo))')
    else
        beep
    end
end
ess.currPdValid = 0;
setappdata(handles.MasterFigure,'sensorData',ess);
setappdata(handles.MasterFigure,'displayData',dsp);

%% Deslect current terrain / obstacle editor pushbutton
function deselectTerrainPushbuttons(ter,handles)
bgcol = [0.92549 0.913725 0.847059];
switch ter.meshEditMode
    case 1
        set(handles.selectTerrainMeshFromNpts,'BackgroundColor',bgcol)
    case 2
        set(handles.selectTerrainMeshFromEdgeAndPoint,'BackgroundColor',bgcol)
    case 3
        set(handles.selectAddInteriorVertex,'BackgroundColor',bgcol)
    case 4
        set(handles.selectDragVertex,'BackgroundColor',bgcol)
    case 5
        set(handles.selectDeleteVertex,'BackgroundColor',bgcol)
    case 6
        set(handles.selectDeleteEdge,'BackgroundColor',bgcol)
    case 7
        set(handles.selectPaintTerrain,'BackgroundColor',bgcol)
    case 10
        set(handles.ObstacleEditingAdd,'BackgroundColor',bgcol)
    case 11
        set(handles.ObstacleEditingDelete,'BackgroundColor',bgcol)
    case 12
        set(handles.ObstacleEditingEdit,'BackgroundColor',bgcol)
end

%% Update sensor editing display edit data
function updateSensorEditData(ess,handles)
dsp = getappdata(handles.MasterFigure,'displayData');

switch ess.editMode
    case 0
        % Generic popup menues and edit boxes
        setSensorEditingGenericData(handles,'off')
        
        % Volume sensor only edit boxes
        setSensorEditingVolumeData(handles,'off')
        
    case {1,3}
        sIndex = find(strcmpi(ess.currentSensorParam.model,...
            get(handles.SensorEditingSelectSensor,'string')));
        if ~isempty(sIndex)
            set(handles.SensorEditingSelectSensor,'visible','on','value',sIndex)
        else
            set(handles.SensorEditingSelectSensor,'visible','on')
        end
        setSensorEditingGenericData(handles,'on')
            
        set(handles.SensorEditingSensorClassDsipText,'visible','on',...
            'string',ess.currentSensorParam.class)
        
        sIndex = find(strcmpi(ess.currentSensorParam.PdMode,...
            get(handles.SensorEditingPdMethodPopup,'string')));
        if ~isempty(sIndex)
            set(handles.SensorEditingPdMethodPopup,'visible','on','value',sIndex)
        else
            set(handles.SensorEditingPdMethodPopup,'visible','on')
        end

        set(handles.SensorEditingSensorLabelEdit,'visible','on',...
            'string',ess.currentSensorParam.sensorLabel)
        set(handles.SensorEditingMountingHeightEdit,'visible','on',...
                'string',num2str(ess.currentSensorParam.mountHeight,'%4.2f'))
        
        set(handles.SensorEditingUnitCostEdit,'string',...
            num2str(ess.currentSensorParam.unitCost,'%8.2f'))
        set(handles.SensorEditingElementCostEdit,'string',...
            num2str(ess.currentSensorParam.elementCost,'%8.2f'))
        set(handles.SensorEditingSupportCostEdit,'string',...
            num2str(ess.currentSensorParam.supportCost,'%8.2f'))
        set(handles.SensorEditingTotalCostEdit,'string',...
            num2str(ess.currentSensorParam.totalCost,'%8.2f')) 
        
        if ess.currentSensor == 0
            % Display default parameter data
            set(handles.SensorEditingSensorLocationEditX1,'visible','off')
            set(handles.SensorEditingSensorLocationEditX2,'visible','off')    
            set(handles.SensorEditingSensorLocationEditY1,'visible','off')
            set(handles.SensorEditingSensorLocationEditY2,'visible','off')
            set(handles.SensorEditingSensorLocationEditZ1,'visible','off')
            set(handles.SensorEditingSensorLocationEditZ2,'visible','off')
            
            if strcmpi(ess.currentSensorParam.class,'volume')
                set(dsp.hEdge,'HitTest','off','ButtonDownFcn',[])
                set(handles.MainAxes,'HitTest','on','ButtonDownFcn',...
                    'ssesPrototype(''addVolumeSensor'',gcbo,[],guidata(gcbo))')
                setSensorEditingVolumeTitles(handles,'on')
            else
                set(dsp.hEdge,'HitTest','on','ButtonDownFcn',...
                'ssesPrototype(''addLineSensor'',gcbo,[],guidata(gcbo))')
                set(handles.MainAxes,'HitTest','off')
                setSensorEditingVolumeTitles(handles,'off')
            end
            set(handles.SensorEditingSensorElUedit,'visible','off')
            set(handles.SensorEditingSensorElLedit,'visible','off')
            set(handles.SensorEditingSensorAzRedit,'visible','off')
            set(handles.SensorEditingSensorAzLedit,'visible','off')

        else
              
                
            if ess.currentSensor > 0
                % If current sensor pointer is > 0 current sensor is a line sensor and 
                % the sensor list corresponds to terrain edges
                thisSensor = ess.lineSensors{ess.currentSensor};

                set(handles.SensorEditingSensorLabelEdit,'visible','on',...
                    'string',ess.currentSensorParam.sensorLabel)
                set(handles.SensorEditingSensorLocationEditX1,'visible','on',...
                    'string',num2str(thisSensor.Xlocation(1),'%4.2f'))
                set(handles.SensorEditingSensorLocationEditX2,'visible','on',...
                    'string',num2str(thisSensor.Xlocation(2),'%4.2f'))    
                set(handles.SensorEditingSensorLocationEditY1,'visible','on',...
                    'string',num2str(thisSensor.Ylocation(1),'%4.2f'))
                set(handles.SensorEditingSensorLocationEditY2,'visible','on',...
                    'string',num2str(thisSensor.Ylocation(2),'%4.2f'))
                set(handles.SensorEditingSensorLocationEditZ1,'visible','on',...
                    'string',num2str(thisSensor.Zlocation(1),'%4.2f'))
                set(handles.SensorEditingSensorLocationEditZ2,'visible','on',...
                    'string',num2str(thisSensor.Zlocation(2),'%4.2f'))

                setSensorEditingVolumeTitles(handles,'off')
            else
                % If current sensor pointer is < 0 current sensor is a line sensor and 
                % the pointer indexes directly into the list of volume sensors
                thisSensor = ess.volumeSensors{abs(ess.currentSensor)};

                set(handles.SensorEditingSensorLabelEdit,'visible','on',...
                    'string',thisSensor.sensorLabel)
                set(handles.SensorEditingSensorLocationEditX1,'visible','on',...
                    'string',num2str(thisSensor.Xlocation(1),'%4.2f'))
                set(handles.SensorEditingSensorLocationEditX2,'visible','off')

                set(handles.SensorEditingSensorLocationEditY1,'visible','on',...
                    'string',num2str(thisSensor.Ylocation(1),'%4.2f'))
                set(handles.SensorEditingSensorLocationEditY2,'visible','off')

                set(handles.SensorEditingSensorLocationEditZ1,'visible','on',...
                    'string',num2str(thisSensor.Zlocation(1),'%4.2f'))
                set(handles.SensorEditingSensorLocationEditZ2,'visible','off')

                set(handles.SensorEditingSensorElLedit,'visible','on',...
                    'string',num2str(thisSensor.ELlimit(1),'%4.2f'))
                set(handles.SensorEditingSensorElUedit,'visible','on',...
                    'string',num2str(thisSensor.ELlimit(2),'%4.2f'))
                set(handles.SensorEditingSensorAzLedit,'visible','on',...
                    'string',num2str(thisSensor.AZlimit(1),'%4.2f'))
                set(handles.SensorEditingSensorAzRedit,'visible','on',...
                    'string',num2str(thisSensor.AZlimit(2),'%4.2f'))

                set(handles.SensorEditingSensorElUtext,'visible','on')
                set(handles.SensorEditingSensorElLtext,'visible','on')
                set(handles.SensorEditingSensorAzRtext,'visible','on')
                set(handles.SensorEditingSensorAzLtext,'visible','on')
                set(handles.SensorEditingAzimuthText,'visible','on')
                set(handles.SensorEditingElevationCoverageText,'visible','on')
            end
        end
    case 2
        % No action for delete mode
end
setappdata(handles.MasterFigure,'displayData',dsp);

%% Sensor editing display sets
function setSensorEditingGenericTitles(handles,vis)
% Set visibility for generic sensor editing gui control text labels
set(handles.SensorEditingSensorTypeText,'visible',vis)        
set(handles.EditSensorsSensorClassText,'visible',vis)
set(handles.SensorEditingPdMethodText,'visible',vis) 
set(handles.SensorEditingMountingHeighttext,'visible',vis)
set(handles.SensorEditingSensorLabelText,'visible',vis)
set(handles.SensorEditingLocationText,'visible',vis)
set(handles.SensorEditingSensorLocationXtext,'visible',vis)
set(handles.SensorEditingSensorLocationYtext,'visible',vis)
set(handles.SensorEditingSensorLocationZtext,'visible',vis)
set(handles.SensorEditingUnitCostText,'visible',vis)
set(handles.SensorEditingElementCostText,'visible',vis)
set(handles.SensorEditingSupportCostText,'visible',vis)
set(handles.SensorEditingTotalCostText,'visible',vis)

function setSensorEditingVolumeTitles(handles,vis)
% Set visibility for generic sensor editing gui control text labels
set(handles.SensorEditingAzimuthText,'visible',vis)
set(handles.SensorEditingSensorAzLtext,'visible',vis)
set(handles.SensorEditingSensorAzRtext,'visible',vis)
set(handles.SensorEditingElevationCoverageText,'visible',vis)
set(handles.SensorEditingSensorElUtext,'visible',vis)
set(handles.SensorEditingSensorElLtext,'visible',vis)
set(handles.SensorEditingSensorElLedit,'visible',vis)
set(handles.SensorEditingSensorElUedit,'visible',vis)
set(handles.SensorEditingSensorAzLedit,'visible',vis)
set(handles.SensorEditingSensorAzRedit,'visible',vis)

function setSensorEditingGenericData(handles,vis)
% Set visibility for generic sensor editing gui controls
set(handles.SensorEditingSelectSensor,'visible',vis)
set(handles.SensorEditingSensorClassDsipText,'visible',vis)
set(handles.SensorEditingPdMethodPopup,'visible',vis)
set(handles.SensorEditingSensorLabelEdit,'visible',vis)
set(handles.SensorEditingMountingHeightEdit,'visible',vis)
set(handles.SensorEditingSensorLocationEditX1,'visible',vis)
set(handles.SensorEditingSensorLocationEditX2,'visible',vis)
set(handles.SensorEditingSensorLocationEditY1,'visible',vis)
set(handles.SensorEditingSensorLocationEditY2,'visible',vis)
set(handles.SensorEditingSensorLocationEditZ1,'visible',vis)
set(handles.SensorEditingSensorLocationEditZ2,'visible',vis)
set(handles.SensorEditingUnitCostEdit,'visible',vis)
set(handles.SensorEditingElementCostEdit,'visible',vis)
set(handles.SensorEditingSupportCostEdit,'visible',vis)
set(handles.SensorEditingTotalCostEdit,'visible',vis)

function setSensorEditingVolumeData(handles,vis)
% Set visibility for generic sensor editing gui control text labels
set(handles.SensorEditingSensorAzLedit,'visible',vis)
set(handles.SensorEditingSensorAzRedit,'visible',vis)
set(handles.SensorEditingSensorElLedit,'visible',vis)
set(handles.SensorEditingSensorElUedit,'visible',vis)

%% Specify defended point
function setDefendedPointPB_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ter = getappdata(handles.MasterFigure,'terrainData');
[x, y] = ginput(1);
defendedFace = findCorrectFace([],handles,x,y);
dsp.defendedPoint = defendedFace;
if isempty(dsp.hDefendedPoint)
    dsp.hDefendedPoint = plot3(ter.faceCenterXYZ(defendedFace,1),...
        ter.faceCenterXYZ(defendedFace,2),...
        ter.faceCenterXYZ(defendedFace,3)+ter.altOffset + .1,...
        'marker','o','markeredgecolor','g','markerfacecolor','g',...
        'markersize',8,'visible','on');
else
    set(dsp.hDefendedPoint,'xdata',ter.faceCenterXYZ(defendedFace,1),...
        'ydata',ter.faceCenterXYZ(defendedFace,2),...
        'zdata',ter.faceCenterXYZ(defendedFace,3)+ter.altOffset + .1,...
        'marker','o','markeredgecolor','g','markerfacecolor','g',...
        'markersize',8,'visible','on');
end
setappdata(handles.MasterFigure,'displayData',dsp)

%% Specify intruder source
function IntruderSourcePB_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ter = getappdata(handles.MasterFigure,'terrainData');
[x, y] = ginput(1);
intruderSource = findCorrectFace([],handles,x,y);
dsp.intruderSource = intruderSource;
if isempty(dsp.hIntruderSource)
    dsp.hIntruderSource = plot3(ter.faceCenterXYZ(intruderSource,1),...
        ter.faceCenterXYZ(intruderSource,2),...
        ter.faceCenterXYZ(intruderSource,3)+ter.altOffset + .1,...
        'marker','d','markeredgecolor','r','markerfacecolor','r',...
        'markersize',8,'visible','on');
else
    set(dsp.hIntruderSource,'xdata',ter.faceCenterXYZ(intruderSource,1),...
        'ydata',ter.faceCenterXYZ(intruderSource,2),...
        'zdata',ter.faceCenterXYZ(intruderSource,3)+ter.altOffset + .1,...
        'marker','d','markeredgecolor','r','markerfacecolor','r',...
        'markersize',8,'visible','on');
end
setappdata(handles.MasterFigure,'displayData',dsp)

%% Update terrain map to show minimum time route transit time
function updateTerrainMapWithActualMoveTime(dsp)
[t,index] = sort(dsp.spMoveTime);
zap = (t==0);
t = t(~zap); 
index = index(~zap);

timeVect = [10 20 30 40 50 100 200 300 400 500 1000];
for iCost = 1:(numel(timeVect)-1)
    ltCutoff = (t <= timeVect(iCost));
    set(dsp.hFace(index(ltCutoff)),'Visible','on','FaceColor',dsp.g2rRGB(iCost,1:3))
    t = t(~ltCutoff);
    index = index(~ltCutoff);            
end
set(dsp.hFace(index),'Visible','on','FaceColor',dsp.g2rRGB(end,1:3))

%% Update terrain map to show minimum time route transit time
function updateTerrainMapWithRelativeMoveTime(dsp)
[t,index] = sort(dsp.spMoveTime);
zap = (t==0);
t = t(~zap); 
index = index(~zap);

inc = floor(numel(t)/12);
done = 0;
for iCost = 1:11
    top = floor(inc*iCost);
    set(dsp.hFace(index(done+1:top)),'Visible','on','FaceColor',dsp.g2rRGB(iCost,1:3))
    done = top;            
end
set(dsp.hFace(index(done+1:end)),'Visible','on','FaceColor',dsp.g2rRGB(end,1:3))

%% Update terrain map to show minimum Pd route worst case (lowest) Pd
function updateTerrainMapWithPathPd(dsp)
[t,index] = sort(dsp.minPd);

PdVect =  [0  .5 .6 .7 .8 .9 .95 .99 .999 .9999];
colIndx = [12 10  9  8  7  6  5   3    2    1];
for iPd = 1:(numel(PdVect)-1)
    PdCutoff = (t <= PdVect(iPd));
    set(dsp.hFace(index(PdCutoff)),'Visible','on','FaceColor',...
        dsp.g2rRGB(colIndx(iPd),1:3))
    t = t(~PdCutoff);
    index = index(~PdCutoff);
    if isempty(index)
        return
    end
end
set(dsp.hFace(index),'Visible','on','FaceColor',dsp.g2rRGB(1,1:3))

%% Update terrain map to show face Pd
function updateTerrainMapWithFacePd(dsp)
ess = getappdata(dsp.hMasterFigure,'sensorData');

if ess.currPdValid == 0
    handles = getappdata(dsp.hMasterFigure,'UsedByGUIData_m');
    updatePd_Callback([], [], handles)
    ess = getappdata(dsp.hMasterFigure,'sensorData');
end
[t,index] = sort(ess.FacePd);
PdVect =  [0  .5 .6 .7 .8 .9 .95 .99 .999 .9999];
colIndx = [12 10  9  8  7  6  5   3    2    1];
for iPd = 1:(numel(PdVect)-1)
    PdCutoff = (t <= PdVect(iPd));
    set(dsp.hFace(index(PdCutoff)),'Visible','on','FaceColor',...
        dsp.g2rRGB(colIndx(iPd),1:3))
    t = t(~PdCutoff);
    index = index(~PdCutoff);
    if isempty(index)
        return
    end
end
set(dsp.hFace(index),'Visible','on','FaceColor',dsp.g2rRGB(1,1:3))

%% Check line-of-sight for blockage by terrain
function blocked = checkLOS(thisVS,tf,tgtHgt,ter,dsp,handles)
% Checks for LOS blockage by terrain

% Find the starting face
sf = findCorrectFace([],handles,thisVS.Xlocation,thisVS.Ylocation); % Sensor face
if sf == tf
    blocked = false;
    return
end

% Get sensor to target line-of-sight data
sfXYZ = [thisVS.Xlocation thisVS.Ylocation thisVS.Zlocation];   % Actual sensor face coordinates
tfXYZ = ter.faceCenterXYZ(tf,:);                               % Target face centroid coordinates
tfXYZ(3) = tfXYZ(3) + tgtHgt;
% set(dsp.mob_hVertex(sf),'markerfacecolor','w','markersize',10) % *** dev/debug
% set(dsp.mob_hVertex(tf),'markerfacecolor','w','markersize',10) % *** dev/debug
% plot3(handles.MainAxes, [sfXYZ(1) tfXYZ(1)],[sfXYZ(2) tfXYZ(2)],[sfXYZ(3) tfXYZ(3)],'yo-')  % *** dev/debug

% Set Sensor-to-target LOS vectors
losX = [sfXYZ(1) tfXYZ(1)];
losY = [sfXYZ(2) tfXYZ(2)];

% Calculate sensor-to-target triangle data 
dR = sqrt((losX(2)-losX(1))^2 + (losY(2)-losY(1))^2);   % Ground range
sZ = sfXYZ(3);
tZ = tfXYZ(3);
dZ = tZ - sZ;
dZdR = dZ/dR;
cI = [1 2;2 3; 3 1];

% hf = figure; ha = axes; hold on
% plot(ha,[0 dR],[sZ tZ],'bo-')

% Find starting face and vertex data
fv = ter.face2vert(sf,1:3);
vMat = ter.vertexXYZ(fv,1:3);
 
% Find LOS vector intersection with starting face edges
segInt = zeros(3,1);
pX = zeros(3,1);
pY = zeros(3,1);
for iV = 1:3   
    [segInt(iV),pX(iV),pY(iV)] = lineSegInt(losX,losY,vMat(cI(iV,:),1),vMat(cI(iV,:),2),-0.001,0)    ;
end
iE = find(segInt,1);    % ***

i = 1;
% switch depending on whether there is a single intercept or two intercepts
% Single intercept case
blocked = false;
xEdge = ter.adj(fv(cI(iE,1)),fv(cI(iE,2)));
% set(dsp.hEdge(xEdge),'color','y','linewidth',4)% *** dev/debug

segX = vMat(cI(iE,:),1);
% segY = vMat(cI(iE,:),2);
segZ = vMat(cI(iE,:),3);

intR = sqrt((pX(iE)-losX(1))^2 + (pY(iE)-losY(1))^2);
intZ = (pX(iE)-segX(1))/(segX(2)-segX(1)) * (segZ(2)-segZ(1)) + segZ(1);
IZ(i) = intZ;
IR(i) = intR;
if intZ > intR*dZdR + sZ
    blocked = true;
%     plot(ha,intR,intZ,'ro')% *** dev/debug
%     set(dsp.hEdge(xEdge),'color','k')% *** dev/debug
    return
else% *** dev/debug
%     plot(intR,intZ,'bo')% *** dev/debug
end
prevSegVert = fv(cI(iE,1:2));

nf = ter.edge2face(xEdge,:);
nf(nf == sf) = [];
done = (nf == tf);
while ~done
    i = i+1;
    blocked = false;
    % Get vertices for new face and find the new candidate edges
    fv = ter.face2vert(nf(1),1:3);
    posSeg = fv(cI);
    newEdges = find(~all(ismember(posSeg,prevSegVert),2))';
    
    % Find LOS vector intersection with new candiate edges
    vMat = ter.vertexXYZ(fv,1:3);
    segInt = zeros(3,1);
    pX = zeros(3,1);
    pY = zeros(3,1);
    for iV = newEdges   
        [segInt(iV),pX(iV),pY(iV)] = lineSegInt(losX,losY,vMat(cI(iV,:),1),vMat(cI(iV,:),2),-0.01,0);
    end
    iE = find(segInt);
    
    % check number of intersections
    if numel(iE) == 1
        xEdge = ter.adj(fv(cI(iE,1)),fv(cI(iE,2)));
%         set(dsp.hEdge(xEdge),'color','y','linewidth',4)% *** dev/debug

        segX = vMat(cI(iE,:),1);
    %     segY = vMat(cI(iE,:),2);
        segZ = vMat(cI(iE,:),3);

        intR = sqrt((pX(iE)-losX(1))^2 + (pY(iE)-losY(1))^2);
        intZ = (pX(iE)-segX(1))/(segX(2)-segX(1)) * (segZ(2)-segZ(1)) + segZ(1);
        IZ(i) = intZ;
        IR(i) = intR;
        if intZ > intR*dZdR + sZ
            blocked = true;
%             plot(ha,intR,intZ,'ro')
%             set(dsp.hEdge(xEdge),'color','k')% *** dev/debug
            return
%         else
%             plot(ha,intR,intZ,'bo')
        end
        prevSegVert = fv(cI(iE,1:2));
        pf = nf;
        nf = ter.edge2face(xEdge,:);
        nf(nf == pf) = [];
        done = nf == tf || nf == 0;
    else
        return
%         beep; disp('Unexpected case non-unique number of intercepts'); keyboard *** need to add
%         trapping code 
    end
end

%% Display / hide line sensors obstacles
function DisplaySelShowObstaclesCB_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
val = get(gcbo,'value');
if val
    set(dsp.hEdgeObstacles(dsp.hEdgeObstacles ~= 0),'visible','on')
else
    set(dsp.hEdgeObstacles(dsp.hEdgeObstacles ~= 0),'visible','off')
end

%% Display / hide line sensors
function DisplaySelShowLineSensorsCB_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
val = get(gcbo,'value');
if val
    set(dsp.hLineSensors(dsp.hLineSensors ~= 0),'visible','on')
else
    set(dsp.hLineSensors(dsp.hLineSensors ~= 0),'visible','off')
end

%% Display / hide volume sensor edges
function DisplaySelShowVolSensorEdgeCB_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
val = get(gcbo,'value');
if val
    set(dsp.hVolSensorCoverage(dsp.hVolSensorCoverage ~= 0),'edgealpha',0.4)
    set(dsp.hVolSensorPos(dsp.hVolSensorPos ~= 0),'visible','on')
else
    set(dsp.hVolSensorCoverage(dsp.hVolSensorCoverage ~= 0),'edgealpha',0)
    set(dsp.hVolSensorPos(dsp.hVolSensorPos ~= 0),'visible','off')
end

%% Set volume sensor face density
function DisplaySelVolSensorAlpha_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
val = get(gcbo,'value');
set(dsp.hVolSensorCoverage(dsp.hVolSensorCoverage ~= 0),'facealpha',val)

% --- Executes during object creation, after setting all properties.
function DisplaySelVolSensorAlpha_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

%% Drag [volume] sensor
function dragSensor(hObject, eventdata, handles)
% Drags volume sensor to new position
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');

if hObject == handles.MasterFigure
    % if callback object is master figure then perform dragging operation
    
    cp = get(gca,'CurrentPoint');
    xPos = cp(1,1);
    yPos = cp(1,2);
    
    % Do not update if current point is not within the plot area
    xl = get(handles.MainAxes,'XLim');
    if xPos < xl(1) || xl(2) < xPos
        return
    end
    yl = get(handles.MainAxes,'YLim');
    if yPos < yl(1) || yl(2) < yPos
        return
    end
    
    % New sensor location is acceptable -- accept new location and update edges and facets
    site = getappdata(handles.MasterFigure,'siteData');
    zPos = interp2(site.xMesh,site.yMesh,site.zElev,xPos,yPos) + ...
        ess.currentSensorParam.mountHeight;
    
    cs = abs(ess.currentSensor);
    ess.currentSensorParam.Xlocation(1) = xPos;    
    ess.volumeSensors{cs}.Xlocation(1) = xPos;
    
    ess.currentSensorParam.Ylocation(1) = yPos;    
    ess.volumeSensors{cs}.Ylocation(1) = yPos;
    
    ess.currentSensorParam.Zlocation(1) = zPos;    
    ess.volumeSensors{cs}.Zlocation(1) = zPos;
    
    delete(dsp.hVolSensorCoverage(cs))
    dsp.hVolSensorCoverage(cs) = makeVolSensorPatch(ess,handles);
    updateSensorEditData(ess,handles)
    set(dsp.hVolSensorPos(cs),'xdata',xPos,'ydata',yPos,'zdata',zPos) 
end
setappdata(handles.MasterFigure,'sensorData',ess)
setappdata(handles.MasterFigure,'displayData',dsp)
    
%% Release Drag Sensor
function releaseDraggedSensor(hObject, eventdata, handles)
% Terminate dragging
set(handles.MasterFigure,'WindowButtonMotionFcn',[],'WindowButtonUpFcn',[]);
% Perform final position update
dragSensor(handles.MasterFigure, eventdata, handles)

%% Update MOE data
function updateMOEdata(handles)
dsp = getappdata(handles.MasterFigure,'displayData');
ess = getappdata(handles.MasterFigure,'sensorData');

set(handles.MOEessCostVal,'string',int2str(ess.cumCost))
set(handles.MOEminRoutePdVal,'string',num2str(dsp.minPd(dsp.defendedPoint),'%4.3f'))
set(handles.MOEminPdRouteTimeVal,'string',num2str(dsp.minPdRouteTime,'%8.0f'))
set(handles.MOEminRouteReactVal,'string',num2str(dsp.minRxT,'%8.0f'))

%% Update electronic security system cumulative cost
function updateESScumCost(handles)
ess = getappdata(handles.MasterFigure,'sensorData');

cumCost = 0;
lsList = find(~cellfun(@isempty,ess.lineSensors));
vsList = find(~cellfun(@isempty,ess.volumeSensors));

for iLS = lsList
   cumCost = cumCost + ess.lineSensors{iLS}.totalCost;
end

for iVS = vsList
   cumCost = cumCost + ess.volumeSensors{iVS}.totalCost;
end

ess.cumCost = cumCost;
set(handles.MOEessCostVal,'string',num2str(ess.cumCost,'%8.2f'))
setappdata(handles.MasterFigure,'sensorData',ess);

%% Save Sensor Plan
function SensorEditingSaveSensorPlanPB_Callback(hObject, eventdata, handles)

ess = getappdata(handles.MasterFigure,'sensorData');

prompt = {'Enter plan name'};
dlg_title = 'Save sensor plan';
num_lines = 1;
def = {['Plan' int2str(numel(ess.savedPlanList) + 1)]};
planLabel = inputdlg(prompt,dlg_title,num_lines,def);

if isempty(planLabel)
    return
end

planLabel = planLabel{1};
planIndex = strmatch(planLabel, ess.savedPlanList,'exact');
if ~isempty(planIndex)
    warnStr = ['Plan ' planLabel ' already exists. Replace?     '];
    reply = questdlg(warnStr,'Replace Saved Plan?','Cancel');
    if strcmpi(reply,'cancel') || strcmpi(reply,'no')
        return
    end
else
    planIndex = numel(ess.savedPlanList) + 1;
end

% Pack plan data
planData.lineSensors = ess.lineSensors;
planData.volumeSensors = ess.volumeSensors;
planData.currentSensorParam = ess.currentSensorParam;
planData.cumCost = ess.cumCost;
planData.sensorCounter = ess.sensorCounter;

ess.savedPlanData{planIndex} = planData;
ess.savedPlanList{planIndex} = planLabel;
setappdata(handles.MasterFigure,'sensorData',ess);

%% Recall sensor plan
function SensorEditingRecallSensorPlanPB_Callback(hObject, eventdata, handles)

ess = getappdata(handles.MasterFigure,'sensorData');
dsp = getappdata(handles.MasterFigure,'displayData');

% Select plan to load
[planIndex,ok] = listdlg('ListString',ess.savedPlanList,'SelectionMode','single','PromptString','Select plan');
if ~ok
    return
end

% Reload plan data
planData = ess.savedPlanData{planIndex};
ess.currentSensor = 0;
ess.currentSensorParam = planData.currentSensorParam;
ess.lineSensors = planData.lineSensors;
ess.volumeSensors = planData.volumeSensors;
ess.EdgePd(:) = 0;
ess.FacePd(:) = 0;
ess.cumCost = planData.cumCost;
ess.currPdValid = 0;
ess.sensorCounter = planData.sensorCounter;

% Delete current sensor plan graphics
delete(dsp.hLineSensors(dsp.hLineSensors ~=0))
delete(dsp.hVolSensorPos(dsp.hVolSensorPos ~=0))
delete(dsp.hVolSensorCoverage(dsp.hVolSensorCoverage ~=0))

dsp.hLineSensors(dsp.hLineSensors ~=0) = 0;
dsp.hVolSensorPos(dsp.hVolSensorPos ~=0) = 0;
dsp.hVolSensorCoverage(dsp.hVolSensorCoverage ~=0) = 0;

% Redraw line sensors
lsIndex = find(~cellfun(@isempty, ess.lineSensors));
if ~isempty(lsIndex)
    for iLS = lsIndex
        thisSensor = ess.lineSensors{iLS};
        dsp.hLineSensors(iLS) = plot3(...
            thisSensor.Xlocation,thisSensor.Ylocation,thisSensor.Zlocation,...
            'LineStyle',thisSensor.edgeStyle,'color',thisSensor.edgeRGB,...
            'linewidth',thisSensor.edgeWidth,'HitTest','off');
    end
end

% Redraw volume sensors
vsIndex = find(~cellfun(@isempty, ess.volumeSensors));
if ~isempty(vsIndex)
    for iVS = vsIndex

        ess.currentSensorParam = ess.volumeSensors{iVS};
        dsp.hVolSensorCoverage(iVS) = makeVolSensorPatch(ess,handles);

        dsp.hVolSensorPos(iVS) = plot3(ess.currentSensorParam.Xlocation,...
            ess.currentSensorParam.Ylocation,ess.currentSensorParam.Zlocation,'k-^',...
            'marker','^','markersize',6','markerFaceColor','k','markerEdgeColor','k',...
            'linewidth',2,'lineStyle',ess.currentSensorParam.edgeStyle,'HitTest','off');
    end
    updateSensorEditData(ess,handles)
end
setappdata(handles.MasterFigure,'sensorData',ess)
setappdata(handles.MasterFigure,'displayData',dsp)
updateESScumCost(handles)

%% Restore default (full site, top down) view
function restoreDefaultView_Callback(hObject, eventdata, handles)
site = getappdata(handles.MasterFigure,'siteData');
set(handles.MainAxes,'xlim',site.xLimit,'ylim',site.yLimit)
axes(handles.MainAxes)
view(2)

%% Toggle imagery between 2D mode and 3D mode 
function Display2dImagePB_Callback(hObject, eventdata, handles)
s = get(hObject,'string');
if strcmpi(s,'2D Imagery')
    set(handles.Display2dImagePB,'string','3D Imagery')
    ter = getappdata(handles.MasterFigure,'terrainData');
    dsp = getappdata(handles.MasterFigure,'displayData');
    % Force texture map elevation to minimum vertex elevation
    minE = min(ter.vertexXYZ(:,3));
    zd = get(dsp.curTextureMap,'zdata');
    zd(:) = minE;
    set(dsp.curTextureMap,'zdata',zd);
    
else
    set(hObject,'string','2D Imagery')
    site = getappdata(handles.MasterFigure,'siteData');
    dsp = getappdata(handles.MasterFigure,'displayData');
    % Set texture map elevation to site elevation
    set(dsp.curTextureMap,'zdata',site.zElev);
    
end

%% Display / hide intruder source and defended point symbols
function DisplayShowIntruderDefender_Callback(hObject, eventdata, handles)
dsp = getappdata(handles.MasterFigure,'displayData');
val = get(gcbo,'value');
if val
    set([dsp.hDefendedPoint dsp.hIntruderSource],'visible','on')
else
    set([dsp.hDefendedPoint dsp.hIntruderSource],'visible','off')
end

%% Show display legend in separate window
function DisplayShowLegend_Callback(hObject, eventdata, handles)
hMF = handles.MasterFigure;
scrsz = get(0,'ScreenSize');
legPos = scrsz + [20 20 -40 -120];

figure('position',legPos,'color','w','ToolBar','none')
axes('ydir','reverse','xtick',[],'ytick',[],'position',[0.01 0.01 0.98 0.98]);
hold on
box on
ylim([0 20])
xlim([0 10])

ter = getappdata(hMF,'terrainData');
ess = getappdata(hMF,'sensorData');
obs = getappdata(hMF,'obstacleData');
dsp = getappdata(hMF,'displayData');

% Spacing
dW = 0.1;
ttY = 1.7;
ttYd = 2;

% Title
text(5,0.6,'SSES Display Coding','fontsize',16,'fontweight','bold',...
    'VerticalAlignment','middle','HorizontalAlignment','center');

% Terrain types
ttX = 0.2; 
ttXw = 1;
text(ttX+ttXw/2,ttY,'Terrain Types','fontweight','bold',...
    'VerticalAlignment','middle','HorizontalAlignment','center');
for iTer = 1:numel(ter.terrainList)
    tRGB = ter.terrainList(iTer).RGB;
    tType = [ter.terrainList(iTer).type '               '];
    tType = tType(1:15);
    if sum(tRGB) <= 1.1
        fgc = [1 1 1];
    else
        fgc = [0 0 0];
    end    
    text(ttX, ttYd+iTer,tType,'BackgroundColor',tRGB,'Color',fgc,...
        'VerticalAlignment','middle','FontName','FixedWidth')
end

% Face Pd
ttX = ttX+ttXw+dW; 
ttXw = 1;
text(ttX+ttXw/2,ttY,'Face/Path Pd','fontweight','bold',...
    'VerticalAlignment','middle','HorizontalAlignment','center');
PdVect =  [0  .5 .6 .7 .8 .9 .95 .99 .999];
colIndx = [12 10  9  8  7  6  5   3    2];
for iPd = 1:(numel(PdVect)-1)
    PdRGB = dsp.g2rRGB(colIndx(iPd),1:3);
    if PdVect(iPd) == 0
        pdStr = '  0             ';
    elseif PdVect(iPd) == .999
        pdStr = '> 0.999       ';
    else
        pdStr = ['  ' '<= ' num2str(PdVect(iPd),'%3.2f') '        '];
    end
    pdStr = [pdStr '             '];
    pdStr = pdStr(1:12);
    if sum(PdRGB) <= 1.1
        fgc = [1 1 1];
    else
        fgc = [0 0 0];
    end
    text(ttX+ttXw/2, ttYd+iPd,pdStr,'BackgroundColor',PdRGB,'Color',fgc,...
        'VerticalAlignment','middle','HorizontalAlignment','center','FontName','FixedWidth')
end

% Movement Time
ttX = ttX+ttXw+dW; 
ttXw = 1;
text(ttX+ttXw/2,ttY,'Act Move Time','HorizontalAlignment','center','fontweight','bold');

% Actual
timeVect = [10 20 30 40 50 100 200 300 400 500 1000];
for iT = 1:(numel(timeVect)+1)
    tRGB = dsp.g2rRGB(iT,1:3);
    if iT <  numel(timeVect)+1
        tStr = [' <= ' num2str(timeVect(iT),'%4.0f') ' sec         '];
    else
        tStr = ' > 1000 sec         ';
    end
    tStr = [tStr '              '];
    tStr = tStr(1:12);
    if sum(tRGB) <= 1.1
        fgc = [1 1 1];
    else
        fgc = [0 0 0];
    end
    text(ttX+ttXw/2, ttYd+iT,tStr,'BackgroundColor',tRGB,'Color',fgc,...
        'VerticalAlignment','middle','HorizontalAlignment','center','FontName','FixedWidth')
end

ttX = ttX+ttXw+dW;
ttXw = 1;
text(ttX+ttXw/2,ttY,{'Rel Move Time','Fastest'},'HorizontalAlignment','center','fontweight','bold');
for iT = 1:12
    tRGB = dsp.g2rRGB(iT,1:3);
    low = ((iT-1)/12)*100;
    high = ((iT)/12)*100;
    tStr = [' ' num2str(low,'%4.0f') ' to ' num2str(high,'%4.0f') '%'];
    tStr = [tStr '             '];
    tStr = tStr(1:12);
    if sum(tRGB) <= 1.1
        fgc = [1 1 1];
    else
        fgc = [0 0 0];
    end
    text(ttX+ttXw/2, ttYd+iT,tStr,'BackgroundColor',tRGB,'Color',fgc,...
        'VerticalAlignment','middle','HorizontalAlignment','center','FontName','FixedWidth')
end

% Obstacle Types
ttX = ttX+ttXw+dW;
ttXw = 1.6;
text(ttX+ttXw/2,ttY,'Obstacle Type','HorizontalAlignment','center','fontweight','bold');
for iO = 1:numel(obs.obstacleList)
    oRGB = obs.obstacleList(iO).RGB;
    text(ttX, ttYd+iO,obs.obstacleList(iO).type,'Color',oRGB,...
        'VerticalAlignment','middle','FontName','FixedWidth')
    plot([ttX+1.1 ttX+ttXw],[ttYd+iO ttYd+iO],'-','color',oRGB,'LineWidth',obs.obstacleList(iO).width,...
        'LineStyle',obs.obstacleList(iO).style);
    
end

% Sensor Types
sensClass = {ess.sensorList.class}';
isLine = find(strcmpi('line',sensClass));
isVol = find(strcmpi('volume',sensClass));

% Line sensors
ttX = ttX+ttXw+3*dW;
ttXw = 1.5;
text(ttX+ttXw/2,ttY,'Line Sensor Type','HorizontalAlignment','center','fontweight','bold');
for iL = 1:numel(isLine)
    lRGB = ess.sensorList(isLine(iL)).edgeRGB;
    text(ttX, ttYd+iL,ess.sensorList(isLine(iL)).model,'Color',lRGB,...
        'VerticalAlignment','middle','FontName','FixedWidth')
    plot([ttX+1.1 ttX+ttXw],[ttYd+iL ttYd+iL],'-','color',lRGB,...
        'LineWidth',ess.sensorList(isLine(iL)).edgeWidth,...
        'LineStyle',ess.sensorList(isLine(iL)).edgeStyle);
    
end

% Vol sensors
ttX = ttX+ttXw+3*dW;
ttXw = 1.2;
text(ttX+ttXw/2,ttY,'Volume Sensor Type','HorizontalAlignment','center','fontweight','bold');
for iV = 1:numel(isVol)
    vRGB = ess.sensorList(isVol(iV)).faceRGB;
    veRGB = ess.sensorList(isVol(iV)).edgeRGB;
    modelStr = strrep(ess.sensorList(isVol(iV)).model,'_',' ');
    modelStr = [modelStr '             '];
    modelStr = modelStr(1:16);
    text(ttX, ttYd+iV,modelStr,'BackgroundColor',vRGB,...
        'EdgeColor',veRGB,'VerticalAlignment','middle','FontName','FixedWidth')
    
end

%% Create / redraw volume sensor patch
function h = makeVolSensorPatch(ess,handles,segAngle)

% Extract data from current sensor
azLimL = ess.currentSensorParam.AZlimit(1);
azLimR = ess.currentSensorParam.AZlimit(2);
elLimL = ess.currentSensorParam.ELlimit(1);
elLimU = ess.currentSensorParam.ELlimit(2);
mountHt = ess.currentSensorParam.mountHeight +ess.currentSensorParam.Zlocation;
x0 = ess.currentSensorParam.Xlocation;
y0 = ess.currentSensorParam.Ylocation;
rng = ess.currentSensorParam.effRange;

% If segment angle was not suppled set 15 deg defaule
if nargin < 3
    segAngle = 15;
end

% check for special cases for azimuth limits
if azLimL > azLimR
    % "north crossing" case
    azLimL = azLimL-360;
elseif azLimL == 0 && azLimR == 0
    azLimR = 360;
end

% Make azimuth vector
azExt = azLimR-azLimL;
nAZ = ceil(azExt/segAngle)+1;
azVect = linspace(azLimL,azLimR,nAZ);

xVect = x0 + sind(azVect)*rng;
yVect = y0 + cosd(azVect)*rng;

% Set upper and lower elevation limits
zU = mountHt + sind(elLimU)*rng;
zL = mountHt + sind(elLimL)*rng;

% Construct patch data
tpXe = [];
tpYe = [];
tpZe = [];
tpX = [];
tpY = [];

for i = 1:(numel(azVect)-1)
    % Top and bottom "fan"
    tpX = [tpX [x0; xVect(i); xVect(i+1); x0]];
    tpY = [tpY [y0; yVect(i); yVect(i+1); y0]];
    
    % Outer "rim"
    tpXe = [tpXe [xVect(i); xVect(i); xVect(i+1); xVect(i+1)]];
    tpYe = [tpYe [yVect(i); yVect(i); yVect(i+1); yVect(i+1)]];
    tpZe = [tpZe [zL; zU; zU; zL]];
end
% Elevation
tpZL = tpX*0+mountHt;
tpZL(2:3,:) = zL;
tpZU = tpX*0+mountHt;
tpZU(2:3,:) = zU;

% Side pieces -- omit for sensor with 360 degree coverage
if ~(azLimL == 0 && azLimL == 360)
    tpXs = [[x0; xVect(1); xVect(1); x0] [x0; xVect(end); xVect(end); x0]];
    tpYs = [[y0; yVect(1); yVect(1); y0] [y0; yVect(end); yVect(end); y0]];
    tpZs = repmat([mountHt; zL; zU; mountHt],1,2);
else
    tpXs = [];
    tpYs = [];
    tpZs = [];
end

% Assemble into single patch
pX = [tpX  tpX  tpXe tpXs];
pY = [tpY  tpY  tpYe tpYs];
pZ = [tpZL tpZU tpZe tpZs];

% Plot patch in current axes
h = patch(pX,pY,pZ,ess.currentSensorParam.faceRGB,...
    'FaceAlpha',ess.currentSensorParam.faceAlpha,...
    'EdgeColor',ess.currentSensorParam.edgeRGB,...
    'EdgeAlpha',ess.currentSensorParam.faceAlpha,...
    'LineWidth',ess.currentSensorParam.edgeWidth,...
    'LineStyle',ess.currentSensorParam.edgeStyle,...
    'HitTest','off');

%% Generic creat function for edit boxes
function genericEdit_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

%% Specific popup box create functions -- replace with generic as time permits
function ObstacleEditingPopup_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
 
% --- Executes during object creation, after setting all properties.
function TextureMapDispSel_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function MobNetEdgeDispSel_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function TerrainPartitionVerticesDispSel_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function SensorEditingPdMethodPopup_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
%%  Auto created functions that serve no purpose?
function TessAxis_Callback(hObject,eventdata,handles)
% GUIDE created -- believe this is not required
return

% --- Executes on mouse press over axes background.
function MainAxes_ButtonDownFcn(hObject, eventdata, handles)
% GUIDE created -- believe this is not required
return

%% 
function beepInvalidAction(varargin)
% Effectively disables button down or other call backs for objects that should not be hit or
% selected -- placeholder for future error / exception trapping
beep




